@@ -43,6 +43,11 @@
  *
  * #ClutterAlpha is used to "drive" a #ClutterBehaviour instance.
  *
+ * <figure id="alpha-functions">
+ *   <title>Graphic representation of some alpha functions</title>
+ *   <graphic fileref="alpha-func.png" format="PNG"/>
+ * </figure>
+ *
  * Since: 0.2
  */
 
@@ -97,7 +102,7 @@ timeline_new_frame_cb (ClutterTimeline *timeline,
 
       g_object_notify (G_OBJECT (alpha), "alpha");
       g_object_unref (alpha);
-  }
+    }
 }
 
 static void 
@@ -397,6 +402,14 @@ clutter_alpha_new_full (ClutterTimeline  *timeline,
   return retval;
 }
 
+/**
+ * CLUTTER_ALPHA_RAMP_INC:
+ *
+ * Convenience symbol for clutter_ramp_inc_func().
+ *
+ * Since: 0.2
+ */
+
 /**
  * clutter_ramp_inc_func:
  * @alpha: a #ClutterAlpha
@@ -424,6 +437,14 @@ clutter_ramp_inc_func (ClutterAlpha *alpha,
   return (current_frame_num * CLUTTER_ALPHA_MAX_ALPHA) / n_frames;
 }
 
+/**
+ * CLUTTER_ALPHA_RAMP_DEC:
+ *
+ * Convenience symbol for clutter_ramp_dec_func().
+ *
+ * Since: 0.2
+ */
+
 /**
  * clutter_ramp_dec_func:
  * @alpha: a #ClutterAlpha
@@ -453,6 +474,14 @@ clutter_ramp_dec_func (ClutterAlpha *alpha,
          / n_frames;
 }
 
+/**
+ * CLUTTER_ALPHA_RAMP:
+ *
+ * Convenience symbol for clutter_ramp_func().
+ *
+ * Since: 0.2
+ */
+
 /**
  * clutter_ramp_func:
  * @alpha: a #ClutterAlpha
@@ -543,7 +572,7 @@ sincx_func (ClutterAlpha *alpha,
 
   CLUTTER_NOTE (ALPHA, "sine: %2f\n", CLUTTER_FIXED_TO_DOUBLE (sine));
 
-  return CLUTTER_FIXED_INT (sine * CLUTTER_ALPHA_MAX_ALPHA);
+  return CLUTTER_FIXED_TO_INT (sine * CLUTTER_ALPHA_MAX_ALPHA);
 }
 
 /* NB: angle is not in radians but in muliples of PI, i.e., 2.0
@@ -574,6 +603,14 @@ sinc_func (ClutterAlpha *alpha,
 }
 #endif
 
+/**
+ * CLUTTER_ALPHA_SINE:
+ *
+ * Convenience symbol for clutter_sine_func().
+ *
+ * Since: 0.2
+ */
+
 /**
  * clutter_sine_func:
  * @alpha: a #ClutterAlpha
@@ -598,6 +635,14 @@ clutter_sine_func (ClutterAlpha *alpha,
 #endif
 }
 
+/**
+ * CLUTTER_ALPHA_SINE_INC:
+ *
+ * Convenience symbol for clutter_sine_inc_func().
+ *
+ * Since: 0.2
+ */
+
 /**
  * clutter_sine_inc_func:
  * @alpha: a #ClutterAlpha
@@ -632,6 +677,14 @@ clutter_sine_inc_func (ClutterAlpha *alpha,
   return ((guint32)sine) >> CFX_Q;
 }
 
+/**
+ * CLUTTER_ALPHA_SINE_DEC:
+ *
+ * Convenience symbol for clutter_sine_dec_func().
+ *
+ * Since: 0.2
+ */
+
 /**
  * clutter_sine_dec_func:
  * @alpha: a #ClutterAlpha
@@ -666,6 +719,14 @@ clutter_sine_dec_func (ClutterAlpha *alpha,
   return ((guint32)sine) >> CFX_Q;
 }
 
+/**
+ * CLUTTER_ALPHA_SINE_HALF:
+ *
+ * Convenience symbol for clutter_sine_half_func().
+ *
+ * Since: 0.4
+ */
+
 /**
  * clutter_sine_half_func:
  * @alpha: a #ClutterAlpha
@@ -700,6 +761,14 @@ clutter_sine_half_func (ClutterAlpha *alpha,
   return ((guint32)sine) >> CFX_Q;
 }
 
+/**
+ * CLUTTER_ALPHA_SQUARE:
+ *
+ * Convenience symbol for clutter_square_func().
+ *
+ * Since: 0.4
+ */
+
 /**
  * clutter_square_func:
  * @alpha: a #ClutterAlpha
@@ -728,6 +797,13 @@ clutter_square_func (ClutterAlpha *alpha,
                                               : 0;
 }
 
+/**
+ * CLUTTER_ALPHA_SMOOTHSTEP_INC:
+ *
+ * Convenience symbol for clutter_smoothstep_inc_func().
+ *
+ * Since: 0.4
+ */
 
 /**
  * clutter_smoothstep_inc_func:
@@ -776,6 +852,14 @@ clutter_smoothstep_inc_func (ClutterAlpha  *alpha,
   return CFX_INT (r * CLUTTER_ALPHA_MAX_ALPHA);
 }
 
+/**
+ * CLUTTER_ALPHA_SMOOTHSTEP_DEC:
+ *
+ * Convenience symbol for clutter_smoothstep_dec_func().
+ *
+ * Since: 0.4
+ */
+
 /**
  * clutter_smoothstep_dec_func:
  * @alpha: a #ClutterAlpha
@@ -795,6 +879,14 @@ clutter_smoothstep_dec_func (ClutterAlpha  *alpha,
   return CLUTTER_ALPHA_MAX_ALPHA - clutter_smoothstep_inc_func (alpha, dummy);
 }
 
+/**
+ * CLUTTER_ALPHA_EXP_INC:
+ *
+ * Convenience symbol for clutter_exp_inc_func()
+ *
+ * Since: 0.4
+ */
+
 /**
  * clutter_exp_inc_func:
  * @alpha: a #ClutterAlpha
@@ -838,6 +930,13 @@ clutter_exp_inc_func (ClutterAlpha *alpha,
   return result;
 }
 
+/**
+ * CLUTTER_ALPHA_EXP_DEC:
+ *
+ * Convenience symbold for clutter_exp_dec_func().
+ *
+ * Since: 0.4
+ */
 
 /**
  * clutter_exp_dec_func:
