@@ -25,7 +25,7 @@
 
 /**
  * SECTION:clutter-behaviour-rotate
- * @short_description: A behaviour class to rotate actors
+ * @short_description: A behaviour controlling rotation
  *
  * A #ClutterBehaviourRotate rotate actors between a starting and ending
  * angle on a given axis.
@@ -54,7 +54,7 @@ G_DEFINE_TYPE (ClutterBehaviourRotate,
 
 struct _ClutterBehaviourRotatePrivate
 {
-  ClutterFixed angle_begin;
+  ClutterFixed angle_start;
   ClutterFixed angle_end;
 
   ClutterRotateAxis axis;
@@ -74,7 +74,7 @@ enum
 {
   PROP_0,
 
-  PROP_ANGLE_BEGIN,
+  PROP_ANGLE_START,
   PROP_ANGLE_END,
   PROP_AXIS,
   PROP_DIRECTION,
@@ -83,45 +83,43 @@ enum
   PROP_CENTER_Z
 };
 
-static void 
+static void
 alpha_notify_foreach (ClutterBehaviour *behaviour,
 		      ClutterActor     *actor,
 		      gpointer          data)
 {
-  ClutterFixed                   angle;
-  ClutterBehaviourRotate        *rotate_behaviour;
+  ClutterFixed angle = GPOINTER_TO_UINT (data);
+  ClutterBehaviourRotate *rotate_behaviour;
   ClutterBehaviourRotatePrivate *priv;
 
   rotate_behaviour = CLUTTER_BEHAVIOUR_ROTATE (behaviour);
   priv = rotate_behaviour->priv;
 
-  angle = GPOINTER_TO_UINT(data);
+  clutter_actor_set_rotationx (actor, priv->axis,
+                               angle,
+                               priv->center_x,
+                               priv->center_y,
+                               priv->center_z);
+}
 
-  switch (priv->axis)
-    {
-    case CLUTTER_X_AXIS:
-      clutter_actor_rotate_xx (actor,
-			       angle,
-			       priv->center_y, priv->center_z);
-      break;
-    case CLUTTER_Y_AXIS:
-      clutter_actor_rotate_yx (actor,
-			       angle,
-			       priv->center_x, priv->center_z);
-      break;
-    case CLUTTER_Z_AXIS:
-      clutter_actor_rotate_zx (actor,
-			       angle,
-			       priv->center_x, priv->center_y);
-      break;
-    }
+static inline
+ClutterFixed clamp_angle (ClutterFixed a)
+{
+  ClutterFixed a1, a2;
+  gint rounds;
+
+  rounds = a / CFX_360;
+  a1 = rounds * CFX_360;
+  a2 = a - a1;
+
+  return a2;
 }
 
 static void
 clutter_behaviour_rotate_alpha_notify (ClutterBehaviour *behaviour,
                                        guint32           alpha_value)
 {
-  ClutterFixed factor, angle, diff;
+  ClutterFixed factor, angle, start, end;
   ClutterBehaviourRotate *rotate_behaviour;
   ClutterBehaviourRotatePrivate *priv;
 
@@ -130,74 +128,20 @@ clutter_behaviour_rotate_alpha_notify (ClutterBehaviour *behaviour,
 
   factor = CLUTTER_INT_TO_FIXED (alpha_value) / CLUTTER_ALPHA_MAX_ALPHA;
   angle = 0;
-  
-  switch (priv->direction)
+
+  start = priv->angle_start;
+  end   = priv->angle_end;
+
+  if (priv->direction == CLUTTER_ROTATE_CW && start >= end)
     {
-    case CLUTTER_ROTATE_CW:
-      if (priv->angle_end >= priv->angle_begin)
-	{
-	  angle = CLUTTER_FIXED_MUL (factor, 
-				     (priv->angle_end - priv->angle_begin));
-	  angle += priv->angle_begin;
-	}
-      else
-	{
-	  /* Work out the angular length of the arch represented by the
-	   * end angle in CCW direction
-	   */
-	  if (priv->angle_begin >= CLUTTER_INT_TO_FIXED (360))
-	    {
-	      ClutterFixed rounds, a1, a2;
-
-	      rounds = priv->angle_begin / 360;
-	      a1 = rounds * 360;
-	      a2 = - (priv->angle_begin - a1);
-
-	      diff = a1 + a2 + priv->angle_end;
-	    }
-	  else
-	    {
-	      diff = CLUTTER_INT_TO_FIXED (360) 
-		     - priv->angle_begin
-                     + priv->angle_end;
-	    }
-      
-	  angle = CLUTTER_FIXED_MUL (diff, factor);
-	  angle += priv->angle_begin;
-	}
-      break;
-    case CLUTTER_ROTATE_CCW:
-      if (priv->angle_end <= priv->angle_begin)
-	{
-	  angle = CLUTTER_FIXED_MUL (factor, 
-				     (priv->angle_begin - priv->angle_end));
-	  angle = priv->angle_begin - angle;
-	}
-      else
-	{
-	  /* Work out the angular length of the arch represented by the
-	   * end angle in CCW direction
-	   */
-	  if (priv->angle_end >= CLUTTER_INT_TO_FIXED (360))
-	    {
-	      ClutterFixed rounds, a1, a2;
-
-	      rounds = priv->angle_end / 360;
-	      a1 = rounds * 360;
-	      a2 = - (priv->angle_end - a1);
-
-	      diff = a1 + a2 + priv->angle_begin;
-	    }
-	  else
-	    {
-	      diff = CLUTTER_INT_TO_FIXED (360) 
-		     - priv->angle_end
-                     + priv->angle_begin;
-	    }
-	  angle = priv->angle_begin - CLUTTER_FIXED_MUL (diff, factor);
-	}
-      break;
+      end += CFX_360;
     }
+  else if (priv->direction == CLUTTER_ROTATE_CCW && start <= end)
+    {
+      end -= CFX_360;
+    }
+
+  angle = CFX_MUL ((end - start), factor)  + start;
 
   clutter_behaviour_actors_foreach (behaviour,
 				    alpha_notify_foreach,
@@ -218,8 +162,8 @@ clutter_behaviour_rotate_set_property (GObject      *gobject,
 
   switch (prop_id)
     {
-    case PROP_ANGLE_BEGIN:
-      priv->angle_begin = CLUTTER_FLOAT_TO_FIXED (g_value_get_double (value));
+    case PROP_ANGLE_START:
+      priv->angle_start = CLUTTER_FLOAT_TO_FIXED (g_value_get_double (value));
       break;
     case PROP_ANGLE_END:
       priv->angle_end = CLUTTER_FLOAT_TO_FIXED (g_value_get_double (value));
@@ -266,8 +210,8 @@ clutter_behaviour_rotate_get_property (GObject    *gobject,
 
   switch (prop_id)
     {
-    case PROP_ANGLE_BEGIN:
-      g_value_set_double (value, CLUTTER_FIXED_TO_DOUBLE (priv->angle_begin));
+    case PROP_ANGLE_START:
+      g_value_set_double (value, CLUTTER_FIXED_TO_DOUBLE (priv->angle_start));
       break;
     case PROP_ANGLE_END:
       g_value_set_double (value, CLUTTER_FIXED_TO_DOUBLE (priv->angle_end));
@@ -305,26 +249,26 @@ clutter_behaviour_rotate_class_init (ClutterBehaviourRotateClass *klass)
   behaviour_class->alpha_notify = clutter_behaviour_rotate_alpha_notify;
 
   /**
-   * ClutterBehaviourRotate:angle-begin:
+   * ClutterBehaviourRotate:angle-start:
    *
-   * The initial angle from whence the rotation should begin.
+   * The initial angle from whence the rotation should start.
    *
    * Since: 0.4
    */
   g_object_class_install_property (gobject_class,
-                                   PROP_ANGLE_BEGIN,
-                                   g_param_spec_double ("angle-begin",
+                                   PROP_ANGLE_START,
+                                   g_param_spec_double ("angle-start",
                                                         "Angle Begin",
                                                         "Initial angle",
                                                         0.0,
-                                                        CLUTTER_ANGLE_MAX_DEG,
+                                                        360.0,
                                                         0.0,
                                                         CLUTTER_PARAM_READWRITE));
   /**
    * ClutterBehaviourRotate:angle-end:
    *
    * The final angle to where the rotation should end.
-   * 
+   *
    * Since: 0.4
    */
   g_object_class_install_property (gobject_class,
@@ -333,8 +277,8 @@ clutter_behaviour_rotate_class_init (ClutterBehaviourRotateClass *klass)
                                                         "Angle End",
                                                         "Final angle",
                                                         0.0,
-                                                        CLUTTER_ANGLE_MAX_DEG,
                                                         360.0,
+                                                        0.0,
                                                         CLUTTER_PARAM_READWRITE));
   /**
    * ClutterBehaviourRotate:axis:
@@ -369,7 +313,7 @@ clutter_behaviour_rotate_class_init (ClutterBehaviourRotateClass *klass)
   /**
    * ClutterBehaviourRotate:center-x:
    *
-   * The x center of rotation. 
+   * The x center of rotation.
    *
    * Since: 0.4
    */
@@ -385,7 +329,7 @@ clutter_behaviour_rotate_class_init (ClutterBehaviourRotateClass *klass)
   /**
    * ClutterBehaviourRotate:center-y:
    *
-   * The y center of rotation. 
+   * The y center of rotation.
    *
    * Since: 0.4
    */
@@ -400,7 +344,7 @@ clutter_behaviour_rotate_class_init (ClutterBehaviourRotateClass *klass)
   /**
    * ClutterBehaviourRotate:center-z:
    *
-   * The z center of rotation. 
+   * The z center of rotation.
    *
    * Since: 0.4
    */
@@ -423,8 +367,8 @@ clutter_behaviour_rotate_init (ClutterBehaviourRotate *rotate)
 
   rotate->priv = priv = CLUTTER_BEHAVIOUR_ROTATE_GET_PRIVATE (rotate);
 
-  priv->angle_begin = CLUTTER_FLOAT_TO_FIXED (0.0);
-  priv->angle_end = CLUTTER_FLOAT_TO_FIXED (360.0);
+  priv->angle_start = CLUTTER_FLOAT_TO_FIXED (0.0);
+  priv->angle_end = CLUTTER_FLOAT_TO_FIXED (0.0);
   priv->axis = CLUTTER_Z_AXIS;
   priv->direction = CLUTTER_ROTATE_CW;
   priv->center_x = priv->center_y = priv->center_z = 0;
@@ -435,12 +379,14 @@ clutter_behaviour_rotate_init (ClutterBehaviourRotate *rotate)
  * @alpha: a #ClutterAlpha, or %NULL
  * @axis: the rotation axis
  * @direction: the rotation direction
- * @angle_begin: the starting angle
- * @angle_end: the final angle
+ * @angle_start: the starting angle in degrees, between 0 and 360.
+ * @angle_end: the final angle in degrees, between 0 and 360.
  *
  * Creates a new #ClutterBehaviourRotate. This behaviour will rotate actors
- * bound to it on @axis, following @direction, between @angle_begin and
- * @angle_end.
+ * bound to it on @axis, following @direction, between @angle_start and
+ * @angle_end. Angles >= 360 degrees will be clamped to the canonical interval
+ * <0, 360), if angle_start == angle_end, the behaviour will carry out a
+ * single rotation of 360 degrees.
  *
  * Return value: the newly created #ClutterBehaviourRotate.
  *
@@ -450,7 +396,7 @@ ClutterBehaviour *
 clutter_behaviour_rotate_new (ClutterAlpha           *alpha,
                               ClutterRotateAxis       axis,
                               ClutterRotateDirection  direction,
-                              gdouble                 angle_begin,
+                              gdouble                 angle_start,
                               gdouble                 angle_end)
 {
   g_return_val_if_fail (alpha == NULL || CLUTTER_IS_ALPHA (alpha), NULL);
@@ -459,7 +405,7 @@ clutter_behaviour_rotate_new (ClutterAlpha           *alpha,
                        "alpha", alpha,
                        "axis", axis,
                        "direction", direction,
-                       "angle-begin", angle_begin,
+                       "angle-start", angle_start,
                        "angle-end", angle_end,
                        NULL);
 }
@@ -469,8 +415,10 @@ clutter_behaviour_rotate_new (ClutterAlpha           *alpha,
  * @alpha: a #ClutterAlpha or %NULL
  * @axis: the rotation axis
  * @direction: the rotation direction
- * @angle_begin: the starting angle, in fixed point notation
- * @angle_end: the final angle, in fixed point notation
+ * @angle_start: the starting angle, in fixed point notation in degrees,
+ * between 0 and 360.
+ * @angle_end: the final angle, in fixed point notation in degrees, between 0
+ * and 360.
  *
  * Creates a new #ClutterBehaviourRotate. This is the fixed point version
  * of clutter_behaviour_rotate_new().
@@ -483,7 +431,7 @@ ClutterBehaviour *
 clutter_behaviour_rotate_newx (ClutterAlpha           *alpha,
                                ClutterRotateAxis       axis,
                                ClutterRotateDirection  direction,
-                               ClutterFixed            angle_begin,
+                               ClutterFixed            angle_start,
                                ClutterFixed            angle_end)
 {
   ClutterBehaviour *retval;
@@ -501,8 +449,8 @@ clutter_behaviour_rotate_newx (ClutterAlpha           *alpha,
    * and then back again to fixed.
    */
   priv = CLUTTER_BEHAVIOUR_ROTATE_GET_PRIVATE (retval);
-  priv->angle_begin = angle_begin;
-  priv->angle_end = angle_end;
+  priv->angle_start = clamp_angle (angle_start);
+  priv->angle_end   = clamp_angle (angle_end);
 
   return retval;
 }
@@ -601,7 +549,7 @@ clutter_behaviour_rotate_set_direction (ClutterBehaviourRotate *rotate,
 /**
  * clutter_behaviour_rotate_get_bounds:
  * @rotate: a #ClutterBehaviourRotate
- * @angle_begin: return value for the initial angle
+ * @angle_start: return value for the initial angle
  * @angle_end: return value for the final angle
  *
  * Retrieves the rotation boundaries of the rotate behaviour.
@@ -610,7 +558,7 @@ clutter_behaviour_rotate_set_direction (ClutterBehaviourRotate *rotate,
  */
 void
 clutter_behaviour_rotate_get_bounds (ClutterBehaviourRotate *rotate,
-                                     gdouble                *angle_begin,
+                                     gdouble                *angle_start,
                                      gdouble                *angle_end)
 {
   ClutterBehaviourRotatePrivate *priv;
@@ -619,8 +567,8 @@ clutter_behaviour_rotate_get_bounds (ClutterBehaviourRotate *rotate,
 
   priv = rotate->priv;
 
-  if (angle_begin)
-    *angle_begin = CLUTTER_FIXED_TO_DOUBLE (priv->angle_begin);
+  if (angle_start)
+    *angle_start = CLUTTER_FIXED_TO_DOUBLE (priv->angle_start);
 
   if (angle_end)
     *angle_end = CLUTTER_FIXED_TO_DOUBLE (priv->angle_end);
@@ -629,29 +577,30 @@ clutter_behaviour_rotate_get_bounds (ClutterBehaviourRotate *rotate,
 /**
  * clutter_behaviour_rotate_set_bounds:
  * @rotate: a #ClutterBehaviourRotate
- * @angle_begin: initial angle
- * @angle_end: final angle
+ * @angle_start: initial angle in degrees, between 0 and 360.
+ * @angle_end: final angle in degrees, between 0 and 360.
  *
- * Sets the initial and final angles of a rotation behaviour.
+ * Sets the initial and final angles of a rotation behaviour; angles >= 360
+ * degrees get clamped to the canonical interval <0, 360).
  *
  * Since: 0.4
  */
 void
 clutter_behaviour_rotate_set_bounds (ClutterBehaviourRotate *rotate,
-                                     gdouble                 angle_begin,
+                                     gdouble                 angle_start,
                                      gdouble                 angle_end)
 {
   g_return_if_fail (CLUTTER_IS_BEHAVIOUR_ROTATE (rotate));
 
   clutter_behaviour_rotate_set_boundsx (rotate,
-                                        CLUTTER_FLOAT_TO_FIXED (angle_begin),
+                                        CLUTTER_FLOAT_TO_FIXED (angle_start),
                                         CLUTTER_FLOAT_TO_FIXED (angle_end));
 }
 
 /**
  * clutter_behaviour_rotate_get_boundsx:
  * @rotate: a #ClutterBehaviourRotate
- * @angle_begin: return value for the initial angle
+ * @angle_start: return value for the initial angle
  * @angle_end: return value for the final angle
  *
  * Retrieves the rotation boundaries of the rotate behaviour. This is
@@ -661,17 +610,17 @@ clutter_behaviour_rotate_set_bounds (ClutterBehaviourRotate *rotate,
  */
 void
 clutter_behaviour_rotate_get_boundsx (ClutterBehaviourRotate *rotate,
-                                      ClutterFixed           *angle_begin,
+                                      ClutterFixed           *angle_start,
                                       ClutterFixed           *angle_end)
 {
   ClutterBehaviourRotatePrivate *priv;
-  
+
   g_return_if_fail (CLUTTER_IS_BEHAVIOUR_ROTATE (rotate));
-  
+
   priv = rotate->priv;
 
-  if (angle_begin);
-    *angle_begin = priv->angle_begin;
+  if (angle_start);
+    *angle_start = priv->angle_start;
 
   if (angle_end)
     *angle_end = priv->angle_end;
@@ -680,8 +629,10 @@ clutter_behaviour_rotate_get_boundsx (ClutterBehaviourRotate *rotate,
 /**
  * clutter_behaviour_rotate_set_boundsx:
  * @rotate: a #ClutterBehaviourRotate
- * @angle_begin: initial angle, in fixed point notation
- * @angle_end: final angle, in fixed point notation
+ * @angle_start: initial angle, in fixed point notation in degrees, between 0
+ * and 360.
+ * @angle_end: final angle, in fixed point notation in degress, between 0 and
+ * 360.
  *
  * Fixed point version of clutter_behaviour_rotate_set_bounds().
  *
@@ -689,11 +640,11 @@ clutter_behaviour_rotate_get_boundsx (ClutterBehaviourRotate *rotate,
  */
 void
 clutter_behaviour_rotate_set_boundsx (ClutterBehaviourRotate *rotate,
-                                      ClutterFixed            angle_begin,
+                                      ClutterFixed            angle_start,
                                       ClutterFixed            angle_end)
 {
   ClutterBehaviourRotatePrivate *priv;
-  
+
   g_return_if_fail (CLUTTER_IS_BEHAVIOUR_ROTATE (rotate));
 
   priv = rotate->priv;
@@ -701,16 +652,16 @@ clutter_behaviour_rotate_set_boundsx (ClutterBehaviourRotate *rotate,
   g_object_ref (rotate);
   g_object_freeze_notify (G_OBJECT (rotate));
 
-  if (priv->angle_begin != angle_begin)
+  if (priv->angle_start != angle_start)
     {
-      priv->angle_begin = angle_begin;
+      priv->angle_start = clamp_angle (angle_start);
 
-      g_object_notify (G_OBJECT (rotate), "angle-begin");
+      g_object_notify (G_OBJECT (rotate), "angle-start");
     }
 
   if (priv->angle_end != angle_end)
     {
-      priv->angle_end = angle_end;
+      priv->angle_end = clamp_angle (angle_end);
 
       g_object_notify (G_OBJECT (rotate), "angle-end");
     }
