@@ -27,6 +27,7 @@
 #define _HAVE_CLUTTER_COLOR_H
 
 #include <glib-object.h>
+#include <clutter/clutter-fixed.h>
 
 G_BEGIN_DECLS
 
@@ -34,6 +35,15 @@ G_BEGIN_DECLS
 
 typedef struct _ClutterColor ClutterColor;
 
+/**
+ * ClutterColor:
+ * @red: red component, between 0 and 255
+ * @green: green component, between 0 and 255
+ * @blue: blue component, between 0 and 255
+ * @alpha: alpha component, between 0 and 255
+ *
+ * Color representation.
+ */
 struct _ClutterColor
 {
   guint8 red;
@@ -71,6 +81,15 @@ void          clutter_color_shadex     (const ClutterColor *src,
                                         ClutterColor       *dest,
                                         ClutterFixed        shade);
 
+void          clutter_color_to_hlsx     (const ClutterColor *src,
+					 ClutterFixed       *hue,
+					 ClutterFixed       *luminance,
+					 ClutterFixed       *saturation);
+void          clutter_color_from_hlsx   (ClutterColor       *dest,
+					 ClutterFixed        hue,
+					 ClutterFixed        luminance,
+					 ClutterFixed        saturation);
+
 void          clutter_color_to_hls     (const ClutterColor *src,
                                         guint8             *hue,
                                         guint8             *luminance,
