@@ -31,7 +31,7 @@
  *
  * The #ClutterEffectTemplate class provides a simple API for applying
  * pre-defined effects to a single actor. It works as a wrapper around
- * the #ClutterBehaviour objects 
+ * the #ClutterBehaviour objects
  *
  * Since: 0.4
  */
@@ -63,9 +63,9 @@ typedef struct ClutterEffectClosure
   ClutterTimeline          *timeline;
   ClutterAlpha             *alpha;
   ClutterBehaviour         *behave;
-  
+
   gulong                    signal_id;
-  
+
   ClutterEffectCompleteFunc completed_func;
   gpointer                  completed_data;
   ClutterEffectTemplate     *template;
@@ -83,6 +83,9 @@ struct _ClutterEffectTemplatePrivate
 {
   ClutterTimeline *timeline;
 
+  guint do_clone : 1;
+  guint dirty    : 1;
+
   ClutterAlphaFunc alpha_func;
   gpointer alpha_data;
   GDestroyNotify alpha_notify;
@@ -92,8 +95,8 @@ enum
 {
   PROP_0,
 
-  PROP_ALPHA_FUNC,
   PROP_TIMELINE,
+  PROP_DO_CLONE
 };
 
 static void
@@ -133,25 +136,25 @@ clutter_effect_template_dispose (GObject *object)
 }
 
 static void
-clutter_effect_template_set_property (GObject      *object, 
+clutter_effect_template_set_property (GObject      *object,
 				      guint         prop_id,
-				      const GValue *value, 
+				      const GValue *value,
 				      GParamSpec   *pspec)
 {
   ClutterEffectTemplate        *template;
   ClutterEffectTemplatePrivate *priv;
 
   template  = CLUTTER_EFFECT_TEMPLATE (object);
-  priv      = template->priv; 
+  priv      = template->priv;
 
-  switch (prop_id) 
+  switch (prop_id)
     {
-    case PROP_ALPHA_FUNC:
-      priv->alpha_func = g_value_get_pointer (value);
-      break;
     case PROP_TIMELINE:
-      priv->timeline = g_value_get_object (value);
-      g_object_ref(priv->timeline);
+      priv->timeline = g_value_dup_object (value);
+      break;
+    case PROP_DO_CLONE:
+      clutter_effect_template_set_timeline_clone (template,
+						  g_value_get_boolean (value));
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -160,9 +163,9 @@ clutter_effect_template_set_property (GObject      *object,
 }
 
 static void
-clutter_effect_template_get_property (GObject    *object, 
+clutter_effect_template_get_property (GObject    *object,
 				      guint       prop_id,
-				      GValue     *value, 
+				      GValue     *value,
 				      GParamSpec *pspec)
 {
   ClutterEffectTemplate        *template;
@@ -171,18 +174,18 @@ clutter_effect_template_get_property (GObject    *object,
   template  = CLUTTER_EFFECT_TEMPLATE (object);
   priv      = template->priv;
 
-  switch (prop_id) 
+  switch (prop_id)
     {
-    case PROP_ALPHA_FUNC:
-      g_value_set_pointer (value, priv->alpha_func);
-      break;
     case PROP_TIMELINE:
       g_value_set_object (value, priv->timeline);
       break;
+    case PROP_DO_CLONE:
+      g_value_set_boolean (value, priv->do_clone);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
-    } 
+    }
 }
 
 static void
@@ -197,21 +200,6 @@ clutter_effect_template_class_init (ClutterEffectTemplateClass *klass)
   object_class->set_property = clutter_effect_template_set_property;
   object_class->get_property = clutter_effect_template_get_property;
 
-  /**
-   * ClutterEffectTemplate:alpha-func:
-   *
-   * #ClutterAlphaFunc to be used by the template
-   *
-   * Since: 0.4
-   */
-  g_object_class_install_property 
-           (object_class,
-	    PROP_ALPHA_FUNC,
-	    g_param_spec_pointer ("alpha-func",
-				  "Alpha-Function",
-				  "Alpha reference Function", 
-				  G_PARAM_CONSTRUCT_ONLY |
-				  CLUTTER_PARAM_READWRITE));
   /**
    * ClutterEffectTemplate:timeline:
    *
@@ -219,7 +207,7 @@ clutter_effect_template_class_init (ClutterEffectTemplateClass *klass)
    *
    * Since: 0.4
    */
-  g_object_class_install_property 
+  g_object_class_install_property
            (object_class,
 	    PROP_TIMELINE,
 	    g_param_spec_object ("timeline",
@@ -228,12 +216,32 @@ clutter_effect_template_class_init (ClutterEffectTemplateClass *klass)
 				 CLUTTER_TYPE_TIMELINE,
 				 G_PARAM_CONSTRUCT_ONLY |
 				 CLUTTER_PARAM_READWRITE));
+
+  /**
+   * ClutterEffectTemplate:clone:
+   *
+   * Controls if effects should clone or reference the templated timeline
+   *
+   * Since: 0.6
+   */
+  g_object_class_install_property
+           (object_class,
+	    PROP_DO_CLONE,
+	    g_param_spec_boolean ("clone",
+				  "Clone",
+				  "controls if effects should clone or reference the templated timeline",
+				  TRUE,
+				  CLUTTER_PARAM_READWRITE));
+
 }
 
 static void
 clutter_effect_template_init (ClutterEffectTemplate *self)
 {
   self->priv = EFFECT_TEMPLATE_PRIVATE (self);
+
+  self->priv->do_clone = TRUE;
+  self->priv->dirty = TRUE;
 }
 
 static void
@@ -255,8 +263,54 @@ clutter_effect_template_set_alpha_func (ClutterEffectTemplate *self,
   priv->alpha_data = alpha_data;
   priv->alpha_notify = alpha_notify;
   priv->alpha_func = alpha_func;
+
+  priv->dirty = FALSE;
 }
 
+/**
+ * clutter_effect_template_set_timeline_clone:
+ * @template_:  A #ClutterEffectTemplate
+ * @setting: A boolean indicating if effects should clone the timeline.
+ *
+ * Sets if effects using this template should make a copy of the
+ * templates timeline (default) or reference the effects timeline.
+ *
+ * Since: 0.6
+ */
+void
+clutter_effect_template_set_timeline_clone (ClutterEffectTemplate *template_,
+					    gboolean               setting)
+{
+  g_return_if_fail (CLUTTER_IS_EFFECT_TEMPLATE (template_));
+
+  if (template_->priv->do_clone != setting)
+    {
+      template_->priv->do_clone = setting;
+
+      g_object_notify (G_OBJECT (template_), "clone");
+    }
+}
+
+/**
+ * clutter_effect_template_get_timeline_clone:
+ * @template_:  A #ClutterEffectTemplate
+ *
+ * Gets whether timelines should be cloned when creating a new
+ * effect or just referenced.
+ *
+ * Return value: %TRUE if the templates timeline is to be cloned.
+ *
+ * Since: 0.6
+ */
+gboolean
+clutter_effect_template_get_timeline_clone (ClutterEffectTemplate *template_)
+{
+  g_return_val_if_fail (CLUTTER_IS_EFFECT_TEMPLATE (template_), FALSE);
+
+  return template_->priv->do_clone;
+}
+
+
 /**
  * clutter_effect_template_new:
  * @timeline:  A #ClutterTimeline for the template (will be cloned)
@@ -276,8 +330,8 @@ clutter_effect_template_set_alpha_func (ClutterEffectTemplate *self,
  *
  * Since: 0.4
  */
-ClutterEffectTemplate *
-clutter_effect_template_new (ClutterTimeline *timeline, 
+ClutterEffectTemplate*
+clutter_effect_template_new (ClutterTimeline *timeline,
 			     ClutterAlphaFunc alpha_func)
 {
   ClutterEffectTemplate *retval;
@@ -285,7 +339,7 @@ clutter_effect_template_new (ClutterTimeline *timeline,
   g_return_val_if_fail (CLUTTER_IS_TIMELINE (timeline), NULL);
   g_return_val_if_fail (alpha_func != NULL, NULL);
 
-  retval = g_object_new (CLUTTER_TYPE_EFFECT_TEMPLATE, 
+  retval = g_object_new (CLUTTER_TYPE_EFFECT_TEMPLATE,
                          "timeline", timeline,
                          NULL);
 
@@ -334,11 +388,98 @@ clutter_effect_template_new_full (ClutterTimeline  *timeline,
                          "timeline", timeline,
                          NULL);
 
-  clutter_effect_template_set_alpha_func (retval, alpha_func, user_data, notify);
+  clutter_effect_template_set_alpha_func (retval,
+                                          alpha_func,
+                                          user_data, notify);
 
   return retval;
 }
 
+/**
+ * clutter_effect_template_new_for_duration:
+ * @msecs: the duration of the effects, in milliseconds
+ * @alpha_func: an alpha function to use for the template
+ *
+ * Creates a new #ClutterEffectTemplate, to be used with the effects API.
+ *
+ * A #ClutterEffectTemplate binds a timeline and an alpha function and can
+ * be used as a template for multiple calls of clutter_effect_fade(),
+ * clutter_effect_move() and clutter_effect_scale().
+ *
+ * This API is intended for simple animations involving a single actor;
+ * for more complex animations, you should see #ClutterBehaviour and the
+ * derived classes.
+ *
+ * This function creates a #ClutterTimeline with a duration of @msecs
+ * milliseconds and transfers ownership of the timeline object to the
+ * returned #ClutterEffectTemplate.
+ *
+ * Return value: the newly created #ClutterEffectTemplate object
+ *
+ * Since: 0.6
+ */
+ClutterEffectTemplate *
+clutter_effect_template_new_for_duration (guint            msecs,
+                                          ClutterAlphaFunc alpha_func)
+{
+  ClutterTimeline *timeline;
+  ClutterEffectTemplate *retval;
+
+  g_return_val_if_fail (msecs > 0, NULL);
+  g_return_val_if_fail (alpha_func != NULL, NULL);
+
+  timeline = clutter_timeline_new_for_duration (msecs);
+  retval = clutter_effect_template_new (timeline, alpha_func);
+
+  /* the effect template takes ownership of the timeline */
+  g_object_unref (timeline);
+
+  return retval;
+}
+
+/**
+ * clutter_effect_template_construct:
+ * @template_: a #ClutterEffectTemplate
+ * @timeline: a #ClutterTimeline
+ * @alpha_func: an alpha function to use for the template
+ * @user_data: data to be passed to the alpha function, or %NULL
+ * @notify: function to be called when disposing the alpha function's use
+ *   data, or %NULL
+ *
+ * Constructs a #ClutterEffectTemplate, to be used with the effects API.
+ *
+ * This function can only be called once after the creation of @template_
+ * and is only useful for language bindings.
+ *
+ * Since: 0.6
+ */
+void
+clutter_effect_template_construct (ClutterEffectTemplate *template_,
+                                   ClutterTimeline       *timeline,
+                                   ClutterAlphaFunc       alpha_func,
+                                   gpointer               user_data,
+                                   GDestroyNotify         notify)
+{
+  ClutterEffectTemplatePrivate *priv;
+
+  g_return_if_fail (CLUTTER_IS_EFFECT_TEMPLATE (template_));
+  g_return_if_fail (CLUTTER_IS_TIMELINE (timeline));
+  g_return_if_fail (alpha_func != NULL);
+
+  if (!template_->priv->dirty)
+    return;
+
+  priv = template_->priv;
+
+  if (priv->timeline)
+    g_object_unref (priv->timeline);
+
+  priv->timeline = g_object_ref (timeline);
+  clutter_effect_template_set_alpha_func (template_,
+                                          alpha_func,
+                                          user_data, notify);
+}
+
 static void
 clutter_effect_closure_destroy (ClutterEffectClosure *c)
 {
@@ -355,7 +496,7 @@ clutter_effect_closure_destroy (ClutterEffectClosure *c)
 
 static ClutterEffectClosure *
 clutter_effect_closure_new (ClutterEffectTemplate *template,
-			    ClutterActor          *actor, 
+			    ClutterActor          *actor,
 			    GCallback              complete)
 {
   ClutterEffectClosure *c;
@@ -368,7 +509,15 @@ clutter_effect_closure_new (ClutterEffectTemplate *template,
 
   c->template = template;
   c->actor    = actor;
-  c->timeline = clutter_timeline_clone (priv->timeline);
+
+  if (clutter_effect_template_get_timeline_clone (template))
+    c->timeline = clutter_timeline_clone (priv->timeline);
+  else
+    {
+      c->timeline = priv->timeline;
+      g_object_ref (priv->timeline);
+    }
+
   c->alpha    = clutter_alpha_new_full (c->timeline,
 					priv->alpha_func,
 					priv->alpha_data,
@@ -396,11 +545,10 @@ on_effect_complete (ClutterTimeline *timeline,
  * clutter_effect_fade:
  * @template_: A #ClutterEffectTemplate
  * @actor: A #ClutterActor to apply the effect to.
- * @start_opacity: Initial opacity value to apply to actor
- * @end_opacity: Final opacity value to apply to actor
- * @completed_func: A #ClutterEffectCompleteFunc to call on effect
+ * @opacity_end: Final opacity value to apply to actor
+ * @func: A #ClutterEffectCompleteFunc to call on effect
  *   completion or %NULL
- * @completed_data: Data to pass to supplied  #ClutterEffectCompleteFunc
+ * @data: Data to pass to supplied  #ClutterEffectCompleteFunc
  *   or %NULL
  *
  * Simple effect for fading a single #ClutterActor.
@@ -408,34 +556,34 @@ on_effect_complete (ClutterTimeline *timeline,
  * Return value: a #ClutterTimeline for the effect. Will be unrefed by
  * the effect when completed.
  *
- * Since: 0.4
+ * Since: 0.6
  */
 ClutterTimeline *
 clutter_effect_fade (ClutterEffectTemplate     *template_,
 		     ClutterActor              *actor,
-		     guint8                     start_opacity,
-		     guint8                     end_opacity,
-		     ClutterEffectCompleteFunc  completed_func,
-		     gpointer                   completed_data)
+		     guint8                     opacity_end,
+		     ClutterEffectCompleteFunc  func,
+		     gpointer                   data)
 {
   ClutterEffectClosure *c;
+  guint8 opacity_start;
 
   c = clutter_effect_closure_new (template_,
-				  actor, 
+				  actor,
 				  G_CALLBACK (on_effect_complete));
 
-  c->completed_func = completed_func;
-  c->completed_data = completed_data;
+  c->completed_func = func;
+  c->completed_data = data;
 
-  clutter_actor_set_opacity (actor, start_opacity);
+  opacity_start = clutter_actor_get_opacity (actor);
+
+  c->behave = clutter_behaviour_opacity_new (c->alpha,
+					     opacity_start,
+					     opacity_end);
 
-  c->behave = clutter_behaviour_opacity_new (c->alpha, 
-					     start_opacity, 
-					     end_opacity);
-  
   clutter_behaviour_apply (c->behave, actor);
   clutter_timeline_start (c->timeline);
-  
+
   return c->timeline;
 }
 
@@ -443,11 +591,10 @@ clutter_effect_fade (ClutterEffectTemplate     *template_,
  * clutter_effect_depth:
  * @template_: A #ClutterEffectTemplate
  * @actor: A #ClutterActor to apply the effect to.
- * @start_depth: Initial depth value to apply to actor
- * @end_depth: Final depth value to apply to actor
- * @completed_func: A #ClutterEffectCompleteFunc to call on effect
+ * @depth_end: Final depth value to apply to actor
+ * @func: A #ClutterEffectCompleteFunc to call on effect
  *   completion or %NULL
- * @completed_data: Data to pass to supplied  #ClutterEffectCompleteFunc
+ * @data: Data to pass to supplied  #ClutterEffectCompleteFunc
  *   or %NULL
  *
  * Simple effect for changing the depth of a single #ClutterActor.
@@ -455,30 +602,32 @@ clutter_effect_fade (ClutterEffectTemplate     *template_,
  * Return value: a #ClutterTimeline for the effect. Will be unrefed by
  * the effect when completed.
  *
- * Since: 0.5
+ * Since: 0.6
  */
 ClutterTimeline *
 clutter_effect_depth (ClutterEffectTemplate     *template_,
-		     ClutterActor               *actor,
-		     gint                       start_depth,
-		     gint                       end_depth,
-		     ClutterEffectCompleteFunc  completed_func,
-		     gpointer                   completed_data)
+		      ClutterActor               *actor,
+		      gint                       depth_end,
+		      ClutterEffectCompleteFunc  func,
+		      gpointer                   data)
 {
   ClutterEffectClosure *c;
+  gint depth_start;
 
   c = clutter_effect_closure_new (template_,
-				  actor, 
+				  actor,
 				  G_CALLBACK (on_effect_complete));
 
-  c->completed_func = completed_func;
-  c->completed_data = completed_data;
+  c->completed_func = func;
+  c->completed_data = data;
+
+  depth_start = clutter_actor_get_depth (actor);
+
+  c->behave = clutter_behaviour_depth_new (c->alpha, depth_start, depth_end);
 
-  c->behave = clutter_behaviour_depth_new (c->alpha, start_depth, end_depth);
-  
   clutter_behaviour_apply (c->behave, actor);
   clutter_timeline_start (c->timeline);
-  
+
   return c->timeline;
 }
 
@@ -486,11 +635,61 @@ clutter_effect_depth (ClutterEffectTemplate     *template_,
  * clutter_effect_move:
  * @template_: A #ClutterEffectTemplate
  * @actor: A #ClutterActor to apply the effect to.
+ * @x: X coordinate of the destination
+ * @y: Y coordinate of the destination
+ * @func: A #ClutterEffectCompleteFunc to call on effect
+ *   completion or %NULL
+ * @data: Data to pass to supplied  #ClutterEffectCompleteFunc
+ *   or %NULL
+ *
+ * Simple effect for moving a single #ClutterActor along to a
+ * destination point.
+ *
+ * Return value: a #ClutterTimeline for the effect. Will be unreferenced by
+ *   the effect when completed.
+ *
+ * Since: 0.6
+ */
+ClutterTimeline *
+clutter_effect_move (ClutterEffectTemplate     *template_,
+		     ClutterActor              *actor,
+                     gint                       x,
+                     gint                       y,
+		     ClutterEffectCompleteFunc  func,
+		     gpointer                   data)
+{
+  ClutterEffectClosure *c;
+  ClutterKnot knots[2];
+
+  c = clutter_effect_closure_new (template_,
+				  actor,
+				  G_CALLBACK (on_effect_complete));
+
+  c->completed_func = func;
+  c->completed_data = data;
+
+  knots[0].x = clutter_actor_get_x (actor);
+  knots[0].y = clutter_actor_get_y (actor);
+
+  knots[1].x = x;
+  knots[1].y = y;
+
+  c->behave = clutter_behaviour_path_new (c->alpha, knots, 2);
+
+  clutter_behaviour_apply (c->behave, actor);
+  clutter_timeline_start (c->timeline);
+
+  return c->timeline;
+}
+/**
+ * clutter_effect_path:
+ * @template_: A #ClutterEffectTemplate
+ * @actor: A #ClutterActor to apply the effect to.
  * @knots: An array of #ClutterKnots representing path for the actor
  * @n_knots: Number of #ClutterKnots in passed array.
- * @completed_func: A #ClutterEffectCompleteFunc to call on effect
+ * @func: A #ClutterEffectCompleteFunc to call on effect
  *   completion or %NULL
- * @completed_data: Data to pass to supplied  #ClutterEffectCompleteFunc
+ * @data: Data to pass to supplied  #ClutterEffectCompleteFunc
  *   or %NULL
  *
  * Simple effect for moving a single #ClutterActor along a path.
@@ -498,33 +697,33 @@ clutter_effect_depth (ClutterEffectTemplate     *template_,
  * Return value: a #ClutterTimeline for the effect. Will be unreferenced by
  *   the effect when completed.
  *
- * Since: 0.4
+ * Since: 0.6
  */
 ClutterTimeline *
-clutter_effect_move (ClutterEffectTemplate     *template_,
+clutter_effect_path (ClutterEffectTemplate     *template_,
 		     ClutterActor              *actor,
 		     const ClutterKnot         *knots,
 		     guint                      n_knots,
-		     ClutterEffectCompleteFunc  completed_func,
-		     gpointer                   completed_data)
+		     ClutterEffectCompleteFunc  func,
+		     gpointer                   data)
 {
   ClutterEffectClosure *c;
 
   c = clutter_effect_closure_new (template_,
-				  actor, 
+				  actor,
 				  G_CALLBACK (on_effect_complete));
 
-  c->completed_func = completed_func;
-  c->completed_data = completed_data;
+  c->completed_func = func;
+  c->completed_data = data;
 
   if (n_knots)
     clutter_actor_set_position (actor, knots[0].x, knots[0].y);
 
   c->behave = clutter_behaviour_path_new (c->alpha, knots, n_knots);
-  
+
   clutter_behaviour_apply (c->behave, actor);
   clutter_timeline_start (c->timeline);
-  
+
   return c->timeline;
 }
 
@@ -532,12 +731,11 @@ clutter_effect_move (ClutterEffectTemplate     *template_,
  * clutter_effect_scale:
  * @template_: A #ClutterEffectTemplate
  * @actor: A #ClutterActor to apply the effect to.
- * @scale_begin: Initial scale factor to apply to actor
- * @scale_end: Final scale factor to apply to actor
- * @gravity: A #ClutterGravity for the scale.
- * @completed_func: A #ClutterEffectCompleteFunc to call on effect
+ * @x_scale_end: Final X axis scale factor to apply to actor
+ * @y_scale_end: Final Y axis scale factor to apply to actor
+ * @func: A #ClutterEffectCompleteFunc to call on effect
  *   completion or NULL
- * @completed_data: Data to pass to supplied  #ClutterEffectCompleteFunc
+ * @data: Data to pass to supplied  #ClutterEffectCompleteFunc
  *   or NULL
  *
  * Simple effect for scaling a single #ClutterActor.
@@ -545,216 +743,95 @@ clutter_effect_move (ClutterEffectTemplate     *template_,
  * Return value: a #ClutterTimeline for the effect. Will be unreferenced by
  *   the effect when completed.
  *
- * Since: 0.4
+ * Since: 0.6
  */
 ClutterTimeline *
 clutter_effect_scale (ClutterEffectTemplate     *template_,
 		      ClutterActor              *actor,
-		      gdouble                    scale_begin,
-		      gdouble                    scale_end,
-		      ClutterGravity             gravity,
-		      ClutterEffectCompleteFunc  completed_func,
-		      gpointer                   completed_data)
+		      gdouble                    x_scale_end,
+		      gdouble                    y_scale_end,
+		      ClutterEffectCompleteFunc  func,
+		      gpointer                   data)
 {
   ClutterEffectClosure *c;
+  gdouble x_scale_start, y_scale_start;
 
   c = clutter_effect_closure_new (template_,
-				  actor, 
+				  actor,
 				  G_CALLBACK (on_effect_complete));
 
-  c->completed_func = completed_func;
-  c->completed_data = completed_data;
+  c->completed_func = func;
+  c->completed_data = data;
 
-  clutter_actor_set_scale_with_gravity (actor, 
-					scale_begin, scale_begin, gravity);
+  clutter_actor_get_scale (actor, &x_scale_start, &y_scale_start);
+  c->behave = clutter_behaviour_scale_new (c->alpha,
+					   x_scale_start, y_scale_start,
+					   x_scale_end, y_scale_end);
 
-  c->behave = clutter_behaviour_scale_new (c->alpha, 
-					   scale_begin,
-					   scale_end,
-					   gravity);
-  
   clutter_behaviour_apply (c->behave, actor);
   clutter_timeline_start (c->timeline);
-  
+
   return c->timeline;
 }
 
 /**
- * clutter_effect_rotate_x:
- * @template_: A #ClutterEffectTemplate
- * @actor: A #ClutterActor to apply the effect to.
- * @angle_begin: Initial angle to apply to actor
- * @angle_end: Final angle to apply to actor
- * @center_y: Position on Y axis to rotate about.
- * @center_z: Position on Z axis to rotate about.
- * @direction: A #ClutterRotateDirection for the rotation.
- * @completed_func: A #ClutterEffectCompleteFunc to call on effect
- *   completion or NULL
- * @completed_data: Data to pass to supplied  #ClutterEffectCompleteFunc
- *   or NULL
+ * clutter_effect_rotate:
+ * @template_: a #ClutterEffectTemplate
+ * @actor: a #ClutterActor to apply the effect to.
+ * @axis: axis of rotation
+ * @angle: final angle to apply to actor
+ * @center_x: position on X axis to rotate about.
+ * @center_y: position on Y axis to rotate about.
+ * @center_z: position on Z axis to rotate about.
+ * @direction: a #ClutterRotateDirection for the rotation.
+ * @func: a #ClutterEffectCompleteFunc to call on effect
+ *   completion or %NULL
+ * @data: user data to pass to supplied @func or %NULL
  *
- * Simple effect for rotating a single #ClutterActor about y axis.
+ * Simple effect for rotating a single #ClutterActor.
  *
  * Return value: a #ClutterTimeline for the effect. Will be unreferenced by
  *   the effect when completed.
  *
- * Since: 0.4
+ * Since: 0.6
  */
 ClutterTimeline *
-clutter_effect_rotate_x (ClutterEffectTemplate     *template_,
-			 ClutterActor              *actor,
-			 gdouble                    angle_begin,
-			 gdouble                    angle_end,
-			 gint                       center_y,
-			 gint                       center_z,
-			 ClutterRotateDirection     direction,
-			 ClutterEffectCompleteFunc  completed_func,
-			 gpointer                   completed_data)
+clutter_effect_rotate (ClutterEffectTemplate     *template_,
+                       ClutterActor              *actor,
+                       ClutterRotateAxis          axis,
+                       gdouble                    angle_end,
+                       gint                       center_x,
+                       gint                       center_y,
+                       gint                       center_z,
+                       ClutterRotateDirection     direction,
+                       ClutterEffectCompleteFunc  func,
+                       gpointer                   data)
 {
   ClutterEffectClosure *c;
+  gdouble angle_start;
 
   c = clutter_effect_closure_new (template_,
-				  actor, 
+				  actor,
 				  G_CALLBACK (on_effect_complete));
 
-  c->completed_func = completed_func;
-  c->completed_data = completed_data;
+  c->completed_func = func;
+  c->completed_data = data;
 
-
-  clutter_actor_rotate_x (actor, angle_begin, center_y, center_y);
+  angle_start = clutter_actor_get_rotation (actor, axis, NULL, NULL, NULL);
 
   c->behave = clutter_behaviour_rotate_new (c->alpha,
-					    CLUTTER_X_AXIS,
+                                            axis,
 					    direction,
-					    angle_begin,
+					    angle_start,
 					    angle_end);
   g_object_set (c->behave,
+                "center-x", center_x,
 		"center-y", center_y,
 		"center-z", center_z,
 		NULL);
-  
+
   clutter_behaviour_apply (c->behave, actor);
   clutter_timeline_start (c->timeline);
-  
-  return c->timeline;
-}
 
-/**
- * clutter_effect_rotate_y:
- * @template_: A #ClutterEffectTemplate
- * @actor: A #ClutterActor to apply the effect to.
- * @angle_begin: Initial angle to apply to actor
- * @angle_end: Final angle to apply to actor
- * @center_x: Position on X axis to rotate about.
- * @center_z: Position on Z axis to rotate about.
- * @direction: A #ClutterRotateDirection for the rotation.
- * @completed_func: A #ClutterEffectCompleteFunc to call on effect
- *   completion or NULL
- * @completed_data: Data to pass to supplied  #ClutterEffectCompleteFunc
- *   or NULL
- *
- * Simple effect for rotating a single #ClutterActor about y axis.
- *
- * Return value: a #ClutterTimeline for the effect. Will be unreferenced by
- *   the effect when completed.
- *
- * Since: 0.4
- */
-ClutterTimeline *
-clutter_effect_rotate_y (ClutterEffectTemplate     *template_,
-			 ClutterActor              *actor,
-			 gdouble                    angle_begin,
-			 gdouble                    angle_end,
-			 gint                       center_x,
-			 gint                       center_z,
-			 ClutterRotateDirection     direction,
-			 ClutterEffectCompleteFunc  completed_func,
-			 gpointer                   completed_data)
-{
-  ClutterEffectClosure *c;
-
-  c = clutter_effect_closure_new (template_,
-				  actor, 
-				  G_CALLBACK (on_effect_complete));
-
-  c->completed_func = completed_func;
-  c->completed_data = completed_data;
-
-
-  clutter_actor_rotate_y (actor, angle_begin, center_x, center_z);
-
-  c->behave = clutter_behaviour_rotate_new (c->alpha,
-					    CLUTTER_Y_AXIS,
-					    direction,
-					    angle_begin,
-					    angle_end);
-  g_object_set (c->behave,
-		"center-x", center_x,
-		"center-z", center_z,
-		NULL);
-  
-  clutter_behaviour_apply (c->behave, actor);
-  clutter_timeline_start (c->timeline);
-  
-  return c->timeline;
-}
-
-/**
- * clutter_effect_rotate_z:
- * @template_: A #ClutterEffectTemplate
- * @actor: A #ClutterActor to apply the effect to.
- * @angle_begin: Initial angle to apply to actor
- * @angle_end: Final angle to apply to actor
- * @center_x: Position on X axis to rotate about.
- * @center_y: Position on Y axis to rotate about.
- * @direction: A #ClutterRotateDirection for the rotation.
- * @completed_func: A #ClutterEffectCompleteFunc to call on effect
- *   completion or NULL
- * @completed_data: Data to pass to supplied  #ClutterEffectCompleteFunc
- *   or NULL
- *
- * Simple effect for rotating a single #ClutterActor about z axis.
- *
- * Return value: a #ClutterTimeline for the effect. Will be unreferenced by
- *   the effect when completed.
- *
- * Since: 0.4
- */
-ClutterTimeline *
-clutter_effect_rotate_z (ClutterEffectTemplate     *template_,
-			 ClutterActor              *actor,
-			 gdouble                    angle_begin,
-			 gdouble                    angle_end,
-			 gint                       center_x,
-			 gint                       center_y,
-			 ClutterRotateDirection     direction,
-			 ClutterEffectCompleteFunc  completed_func,
-			 gpointer                   completed_data)
-{
-  ClutterEffectClosure *c;
-
-  c = clutter_effect_closure_new (template_,
-				  actor, 
-				  G_CALLBACK (on_effect_complete));
-
-  c->completed_func = completed_func;
-  c->completed_data = completed_data;
-
-
-  clutter_actor_rotate_z (actor, angle_begin, center_x, center_y);
-
-  c->behave = clutter_behaviour_rotate_new (c->alpha,
-					    CLUTTER_Z_AXIS,
-					    direction,
-					    angle_begin,
-					    angle_end);
-  g_object_set (c->behave,
-		"center-x", center_x,
-		"center-y", center_y,
-		NULL);
-  
-  clutter_behaviour_apply (c->behave, actor);
-  clutter_timeline_start (c->timeline);
-  
   return c->timeline;
 }
