@@ -39,12 +39,6 @@
  * Windowing events handled by Clutter.
  */
 
-/* multiple button click detection */
-static guint32 button_click_time[2] = { 0, 0 };
-static guint32 button_number[2] = { -1, -1 };
-static gint    button_x[2] = { 0, 0 };
-static gint    button_y[2] = { 0, 0 };
-
 /**
  * clutter_event_type:
  * @event: a #ClutterEvent
@@ -76,24 +70,7 @@ clutter_event_get_time (ClutterEvent *event)
 {
   g_return_val_if_fail (event != NULL, CLUTTER_CURRENT_TIME);
 
-  switch (event->type)
-    {
-    case CLUTTER_KEY_PRESS:
-    case CLUTTER_KEY_RELEASE:
-      return event->key.time;
-    case CLUTTER_BUTTON_PRESS:
-    case CLUTTER_2BUTTON_PRESS:
-    case CLUTTER_BUTTON_RELEASE:
-      return event->button.time;
-    case CLUTTER_MOTION:
-      return event->motion.time;
-    case CLUTTER_SCROLL:
-      return event->scroll.time;
-    default:
-      break;
-    }
-
-  return CLUTTER_CURRENT_TIME;
+  return event->any.time;
 }
 
 /**
@@ -117,8 +94,6 @@ clutter_event_get_state (ClutterEvent *event)
     case CLUTTER_KEY_RELEASE:
       return event->key.modifier_state;
     case CLUTTER_BUTTON_PRESS:
-    case CLUTTER_2BUTTON_PRESS:
-    case CLUTTER_BUTTON_RELEASE:
       return event->button.modifier_state;
     case CLUTTER_MOTION:
       return event->motion.modifier_state;
@@ -161,10 +136,10 @@ clutter_event_get_coords (ClutterEvent *event,
     case CLUTTER_DESTROY_NOTIFY:
     case CLUTTER_CLIENT_MESSAGE:
     case CLUTTER_DELETE:
+    case CLUTTER_ENTER:
+    case CLUTTER_LEAVE:
       break;
     case CLUTTER_BUTTON_PRESS:
-    case CLUTTER_2BUTTON_PRESS:
-    case CLUTTER_3BUTTON_PRESS:
     case CLUTTER_BUTTON_RELEASE:
       event_x = event->button.x;
       event_y = event->button.y;
@@ -186,6 +161,25 @@ clutter_event_get_coords (ClutterEvent *event,
     *y = event_y;
 }
 
+/**
+ * clutter_event_get_source:
+ * @event: a #ClutterEvent
+ *
+ * Retrieves the source #ClutterActor the event originated from, or
+ * NULL if the event has no source.
+ *
+ * Return value: a #ClutterActor
+ *
+ * Since: 0.6
+ */
+ClutterActor*
+clutter_event_get_source (ClutterEvent *event)
+{
+  g_return_val_if_fail (event != NULL, NULL);
+
+  return event->any.source;
+}
+
 /**
  * clutter_button_event_button:
  * @buttev: a #ClutterButtonEvent
@@ -304,7 +298,7 @@ clutter_event_get_type (void)
   static GType our_type = 0;
 
   if (!our_type)
-    our_type = g_boxed_type_register_static ("ClutterEvent",
+    our_type = g_boxed_type_register_static (I_("ClutterEvent"),
 		    			     (GBoxedCopyFunc) clutter_event_copy,
 					     (GBoxedFreeFunc) clutter_event_free);
   return our_type;
@@ -321,13 +315,9 @@ clutter_event_get_type (void)
 ClutterEvent *
 clutter_event_new (ClutterEventType type)
 {
-  ClutterEventPrivate *real_event;
   ClutterEvent *new_event;
 
-  real_event = g_slice_new0 (ClutterEventPrivate);
-  real_event->flags = 0;
-
-  new_event = (ClutterEvent *) real_event;
+  new_event = g_slice_new0 (ClutterEvent);
   new_event->type = new_event->any.type = type;
 
   return new_event;
@@ -351,6 +341,19 @@ clutter_event_copy (ClutterEvent *event)
   new_event = clutter_event_new (CLUTTER_NOTHING);
   *new_event = *event;
 
+  /* deep copies or references must be added here */
+  switch (new_event->type)
+    {
+    case CLUTTER_ENTER:
+    case CLUTTER_LEAVE:
+      if (new_event->crossing.related)
+        g_object_ref (new_event->crossing.related);
+      break;
+
+    default:
+      break;
+    }
+
   return new_event;
 }
 
@@ -365,7 +368,11 @@ clutter_event_free (ClutterEvent *event)
 {
   if (G_LIKELY (event))
     {
-      g_slice_free (ClutterEventPrivate, (ClutterEventPrivate *) event);
+      if ((event->type == CLUTTER_LEAVE || event->type == CLUTTER_ENTER) &&
+          event->crossing.related)
+        g_object_unref (event->crossing.related);
+
+      g_slice_free (ClutterEvent, event);
     }
 }
 
@@ -383,7 +390,6 @@ ClutterEvent *
 clutter_event_get (void)
 {
   ClutterMainContext *context = clutter_context_get_default ();
-  GList *item;
 
   if (!context->events_queue)
     return NULL;
@@ -391,22 +397,7 @@ clutter_event_get (void)
   if (g_queue_is_empty (context->events_queue))
     return NULL;
 
-  /* find the first non pending item */
-  item = context->events_queue->tail;
-  while (item)
-    {
-      ClutterEvent *event = item->data;
-
-      if (!(CLUTTER_EVENT_IS_PENDING (event)))
-        {
-          g_queue_remove (context->events_queue, event);
-          return event;
-        }
-
-      item = item->prev;
-    }
-
-  return NULL;
+  return g_queue_pop_tail (context->events_queue);
 }
 
 /**
@@ -423,45 +414,43 @@ ClutterEvent *
 clutter_event_peek (void)
 {
   ClutterMainContext *context = clutter_context_get_default ();
-  GList *item;
 
-  if (!context->events_queue)
+  g_return_val_if_fail (context != NULL, NULL);
+  
+  if (context->events_queue == NULL)
     return NULL;
 
   if (g_queue_is_empty (context->events_queue))
     return NULL;
 
-  /* find the first non pending item */
-  item = context->events_queue->tail;
-  while (item)
-    {
-      ClutterEvent *event = item->data;
-      if (!(CLUTTER_EVENT_IS_PENDING (event)))
-        return event;
-
-      item = item->prev;
-    }
-
-  return NULL;
+  return g_queue_peek_tail (context->events_queue);
 }
 
 /**
  * clutter_event_put:
  * @event: a #ClutterEvent
  *
- * Puts a copy of the event on the back on the event queue. 
+ * Puts a copy of the event on the back of the event queue. The event will have
+ * the #CLUTTER_EVENT_FLAG_SYNTHETIC flag set. If the source is set event
+ * signals will be emitted for this source and capture/bubbling for it's
+ * ancestors. If the source is not set it will be generated by picking or use
+ * the actor that currently has keyboard focus.
  *
- * Since: 0.4
+ * Since: 0.6
  */
 void
 clutter_event_put (ClutterEvent *event)
 {
   ClutterMainContext *context = clutter_context_get_default ();
+  ClutterEvent       *event_copy;
 
-  g_return_if_fail (event != NULL);
-  g_return_if_fail (context->events_queue != NULL);
+  /* FIXME: check queue is valid */
+  g_return_if_fail (context != NULL);
 
-  g_queue_push_head (context->events_queue, clutter_event_copy (event));
+  event_copy = clutter_event_copy (event);
+  event_copy->any.flags |= CLUTTER_EVENT_FLAG_SYNTHETIC;
+
+  g_queue_push_head (context->events_queue, event_copy);
 }
 
 /**
@@ -477,95 +466,12 @@ gboolean
 clutter_events_pending (void)
 {
   ClutterMainContext *context = clutter_context_get_default ();
-  GList *item;
 
   g_return_val_if_fail (context != NULL, FALSE);
 
   if (!context->events_queue)
     return FALSE;
 
-  if (g_queue_is_empty (context->events_queue))
-    return FALSE;
-
-  /* find the first non pending item */
-  item = context->events_queue->tail;
-  while (item)
-    {
-      ClutterEvent *event = item->data;
-      if (!(CLUTTER_EVENT_IS_PENDING (event)))
-        return TRUE;
-
-      item = item->prev;
-    }
-
-  return FALSE;
+  return g_queue_is_empty (context->events_queue) == FALSE;
 }
 
-/* Backend helpers (private) */
-
-static void
-synthesize_click (ClutterBackend *backend,
-		  ClutterEvent   *event,
-		  gint            n_clicks)
-{
-  ClutterEvent temp_event;
-
-  temp_event = *event;
-  temp_event.type = (n_clicks == 2) ? CLUTTER_2BUTTON_PRESS
-                                    : CLUTTER_3BUTTON_PRESS;
-
-  clutter_event_put (&temp_event);
-}
-
-/* post process a button to synthesize double clicks etc */
-void
-_clutter_event_button_generate (ClutterBackend *backend,
-                                ClutterEvent   *event)
-{
-  guint double_click_time, double_click_distance;
-
-  double_click_distance = clutter_backend_get_double_click_distance (backend);
-  double_click_time = clutter_backend_get_double_click_time (backend);
-
-  if ((event->button.time < (button_click_time[1] + 2 * double_click_time)) 
-      && (event->button.button == button_number[1]) 
-      && (ABS (event->button.x - button_x[1]) <= double_click_distance) 
-      && (ABS (event->button.y - button_y[1]) <= double_click_distance))
-    {
-      synthesize_click (backend, event, 3);
-            
-      button_click_time[1] = 0;
-      button_click_time[0] = 0;
-      button_number[1] = -1;
-      button_number[0] = -1;
-      button_x[0] = button_x[1] = 0;
-      button_y[0] = button_y[1] = 0;
-    }
-  else if ((event->button.time < (button_click_time[0] + double_click_time))
-           && (event->button.button == button_number[0])
-           && (ABS (event->button.x - button_x[0]) <= double_click_distance)
-           && (ABS (event->button.y - button_y[0]) <= double_click_distance))
-    {
-      synthesize_click (backend, event, 2);
-      
-      button_click_time[1] = button_click_time[0];
-      button_click_time[0] = event->button.time;
-      button_number[1] = button_number[0];
-      button_number[0] = event->button.button;
-      button_x[1] = button_x[0];
-      button_x[0] = event->button.x;
-      button_y[1] = button_y[0];
-      button_y[0] = event->button.y;
-    }
-  else
-    {
-      button_click_time[1] = 0;
-      button_click_time[0] = event->button.time;
-      button_number[1] = -1;
-      button_number[0] = event->button.button;
-      button_x[1] = 0;
-      button_x[0] = event->button.x;
-      button_y[1] = 0;
-      button_y[0] = event->button.y;
-    }
-}
