@@ -29,15 +29,18 @@
  * actors.
  *
  * A #ClutterGroup is an Actor which contains multiple child actors positioned
- * relative to the #ClutterGroup position. Other operations such as scaling, 
- * rotating and clipping of the group will child actors.
+ * relative to the #ClutterGroup position. Other operations such as scaling,
+ * rotating and clipping of the group will apply to the child actors.
  *
  * A #ClutterGroup's size is defined by the size and position of it
  * it children. Resize requests via parent #ClutterActor API will be
  * ignored.
  */
 
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif
+
 #include <stdarg.h>
 
 #include "clutter-group.h"
@@ -101,36 +104,48 @@ clutter_group_paint (ClutterActor *actor)
     }
 
   cogl_pop_matrix();
-  
+
   CLUTTER_NOTE (PAINT, "ClutterGroup paint leave");
 }
 
-static void                  
-clutter_group_pick (ClutterActor       *actor, 
+static void
+clutter_group_unrealize (ClutterActor *actor)
+{
+  clutter_container_foreach (CLUTTER_CONTAINER (actor),
+                             CLUTTER_CALLBACK (clutter_actor_unrealize),
+                             NULL);
+}
+
+static void
+clutter_group_pick (ClutterActor       *actor,
 		    const ClutterColor *color)
 {
-  /* Just forward to the paint call which in turn will trigger 
-   * the child actors also getting 'picked'. To make ourselves
-   * 'sensitive' to clicks we could also paint a bounding rect
-   * but this is not currently done.  
-  */
-  clutter_group_paint (actor);
+  /* Chain up so we get a bounding box pained (if we are reactive) */
+  CLUTTER_ACTOR_CLASS (clutter_group_parent_class)->pick (actor, color);
+
+  /* Just forward to the paint call which in turn will trigger
+   * the child actors also getting 'picked'.
+   */
+  if (CLUTTER_ACTOR_IS_MAPPED (actor))
+    clutter_group_paint (actor);
 }
 
 
 static void
-clutter_group_request_coords (ClutterActor        *self,
-			      ClutterActorBox     *box)
+clutter_group_request_coords (ClutterActor    *self,
+			      ClutterActorBox *box)
 {
   ClutterActorBox cbox;
 
   clutter_actor_query_coords (self, &cbox);
 
   /* Only positioning works.
-   * Sizing requests fail, use scale() instead 
+   * Sizing requests fail, use scale() instead
   */
   box->x2 = box->x1 + (cbox.x2 - cbox.x1);
   box->y2 = box->y1 + (cbox.y2 - cbox.y1);
+
+  CLUTTER_ACTOR_CLASS (clutter_group_parent_class)->request_coords (self, box);
 }
 
 static void
@@ -147,39 +162,63 @@ clutter_group_query_coords (ClutterActor        *self,
   /* FIXME: Cache these values */
   box->x2 = box->x1;
   box->y2 = box->y1;
-  
+
   if (child_item)
     {
-      do 
+      do
 	{
-	  ClutterActor *child = CLUTTER_ACTOR(child_item->data);
+	  ClutterActor    *child = CLUTTER_ACTOR(child_item->data);
+	  ClutterActorBox  cbox;
 
-	  /* Once added we include in sizing - doesn't matter if visible */
-	  /* if (CLUTTER_ACTOR_IS_VISIBLE (child)) */
+#if 0     /* XXX - Leave this post 0.6 ??? */
+	  if (clutter_actor_is_scaled (child) ||
+	      clutter_actor_is_rotated (child))
 	    {
-	      ClutterActorBox cbox;
-	      
-	      clutter_actor_query_coords (child, &cbox);
-	      
-	      /* Ignore any children with offscreen ( negaive )
-               * positions.
-	       *
-               * Also x1 and x2 will be set by parent caller.
-               *
-               * FIXME: this assumes that children are not rotated in anyway.
-	       */
-	      if (box->x2 - box->x1 < cbox.x2)
-		box->x2 = cbox.x2 + box->x1;
+	      ClutterVertex vtx[4];
 
-	      if (box->y2 - box->y1 < cbox.y2)
-		box->y2 = cbox.y2 + box->y1;
+	      clutter_actor_get_relative_vertices (child, self, vtx);
+	      clutter_actor_box_get_from_vertices (vtx, &cbox);
 	    }
+	  else
+#endif
+	    {
+	      gint            anchor_x;
+	      gint            anchor_y;
+
+	      clutter_actor_query_coords (child, &cbox);
+
+	      /*
+	       * Must adjust these by the anchor point, as we need the box
+	       * to be relative to the top-left corner of the parent
+	       */
+	      clutter_actor_get_anchor_pointu (child, &anchor_x, &anchor_y);
+
+	      cbox.x1 -= anchor_x;
+	      cbox.x2 -= anchor_x;
+	      cbox.y1 -= anchor_y;
+	      cbox.y2 -= anchor_y;
+	    }
+
+	  /* FIXME: now that we go into the trouble of working out the
+	   * projected sizes, we should do better than this (probably resize
+	   * the box in all direction as required).
+	   *
+	   * Ignore any children with offscreen ( negaive )
+	   * positions.
+	   *
+	   * Also x1 and x2 will be set by parent caller.
+	   */
+	  if (box->x2 - box->x1 < cbox.x2)
+	    box->x2 = cbox.x2 + box->x1;
+
+	  if (box->y2 - box->y1 < cbox.y2)
+	    box->y2 = cbox.y2 + box->y1;
 	}
       while ((child_item = g_list_next(child_item)) != NULL);
     }
 }
 
-static void 
+static void
 clutter_group_dispose (GObject *object)
 {
   ClutterGroup *self = CLUTTER_GROUP (object);
@@ -190,7 +229,7 @@ clutter_group_dispose (GObject *object)
       g_list_foreach (priv->children, (GFunc) clutter_actor_destroy, NULL);
       priv->children = NULL;
     }
-  
+
   G_OBJECT_CLASS (clutter_group_parent_class)->dispose (object);
 }
 
@@ -220,7 +259,7 @@ clutter_group_real_add (ClutterContainer *container,
   ClutterGroupPrivate *priv = group->priv;
 
   g_object_ref (actor);
-  
+
   /* the old ClutterGroup::add signal was emitted before the
    * actor was added to the group, so that the class handler
    * would actually add it. we need to emit the ::add signal
@@ -232,8 +271,8 @@ clutter_group_real_add (ClutterContainer *container,
   clutter_actor_set_parent (actor, CLUTTER_ACTOR (group));
 
   g_signal_emit_by_name (container, "actor-added", actor);
-  
-  clutter_group_sort_depth_order (group); 
+
+  clutter_group_sort_depth_order (group);
 
   g_object_unref (actor);
 }
@@ -246,7 +285,7 @@ clutter_group_real_remove (ClutterContainer *container,
   ClutterGroupPrivate *priv = group->priv;
 
   g_object_ref (actor);
-  
+
   /* the old ClutterGroup::remove signal was emitted before the
    * actor was removed from the group. see the comment in
    * clutter_group_real_add() above for why we need to emit ::remove
@@ -262,7 +301,7 @@ clutter_group_real_remove (ClutterContainer *container,
    * are holding a reference on it, it's still valid
    */
   g_signal_emit_by_name (container, "actor-removed", actor);
-  
+
   if (CLUTTER_ACTOR_IS_VISIBLE (CLUTTER_ACTOR (group)))
     clutter_actor_queue_redraw (CLUTTER_ACTOR (group));
 
@@ -282,12 +321,123 @@ clutter_group_real_foreach (ClutterContainer *container,
     (* callback) (CLUTTER_ACTOR (l->data), user_data);
 }
 
+static void
+clutter_group_real_raise (ClutterContainer *container,
+                          ClutterActor     *actor,
+                          ClutterActor     *sibling)
+{
+  ClutterGroup *self = CLUTTER_GROUP (container);
+  ClutterGroupPrivate *priv = self->priv;
+
+  priv->children = g_list_remove (priv->children, actor);
+
+  /* Raise at the top */
+  if (!sibling)
+    {
+      GList *last_item;
+
+      last_item = g_list_last (priv->children);
+
+      if (last_item)
+	sibling = last_item->data;
+
+      priv->children = g_list_append (priv->children, actor);
+    }
+  else
+    {
+      gint pos;
+
+      pos = g_list_index (priv->children, sibling) + 1;
+
+      priv->children = g_list_insert (priv->children, actor, pos);
+    }
+
+  /* set Z ordering a value below, this will then call sort
+   * as values are equal ordering shouldn't change but Z
+   * values will be correct.
+   *
+   * FIXME: optimise
+   */
+  if (sibling &&
+      clutter_actor_get_depth (sibling) != clutter_actor_get_depth (actor))
+    {
+      clutter_actor_set_depth (actor, clutter_actor_get_depth (sibling));
+    }
+}
+
+static void
+clutter_group_real_lower (ClutterContainer *container,
+                          ClutterActor     *actor,
+                          ClutterActor     *sibling)
+{
+  ClutterGroup *self = CLUTTER_GROUP (container);
+  ClutterGroupPrivate *priv = self->priv;
+
+  priv->children = g_list_remove (priv->children, actor);
+
+  /* Push to bottom */
+  if (!sibling)
+    {
+      GList *last_item;
+
+      last_item = g_list_first (priv->children);
+
+      if (last_item)
+	sibling = last_item->data;
+
+      priv->children = g_list_prepend (priv->children, actor);
+    }
+  else
+    {
+      gint pos;
+
+      pos = g_list_index (priv->children, sibling) + 1;
+
+      priv->children = g_list_insert (priv->children, actor, pos);
+    }
+
+  /* See comment in group_raise for this */
+  if (sibling &&
+      clutter_actor_get_depth (sibling) != clutter_actor_get_depth (actor))
+    {
+      clutter_actor_set_depth (actor, clutter_actor_get_depth (sibling));
+    }
+}
+
+static gint
+sort_z_order (gconstpointer a,
+              gconstpointer b)
+{
+  int depth_a, depth_b;
+
+  depth_a = clutter_actor_get_depth (CLUTTER_ACTOR(a));
+  depth_b = clutter_actor_get_depth (CLUTTER_ACTOR(b));
+
+  return (depth_a - depth_b);
+}
+
+static void
+clutter_group_real_sort_depth_order (ClutterContainer *container)
+{
+  ClutterGroup *self = CLUTTER_GROUP (container);
+  ClutterGroupPrivate *priv = self->priv;
+
+  priv->children = g_list_sort (priv->children, sort_z_order);
+
+  if (CLUTTER_ACTOR_IS_VISIBLE (CLUTTER_ACTOR (self)))
+    clutter_actor_queue_redraw (CLUTTER_ACTOR (self));
+
+}
+
 static void
 clutter_container_iface_init (ClutterContainerIface *iface)
 {
   iface->add = clutter_group_real_add;
   iface->remove = clutter_group_real_remove;
   iface->foreach = clutter_group_real_foreach;
+  iface->raise = clutter_group_real_raise;
+  iface->lower = clutter_group_real_lower;
+  iface->sort_depth_order = clutter_group_real_sort_depth_order;
 }
 
 static void
@@ -297,13 +447,14 @@ clutter_group_class_init (ClutterGroupClass *klass)
   ClutterActorClass *actor_class = CLUTTER_ACTOR_CLASS (klass);
 
   object_class->dispose = clutter_group_dispose;
-  
+
   actor_class->paint           = clutter_group_paint;
   actor_class->pick            = clutter_group_pick;
   actor_class->show_all        = clutter_group_real_show_all;
   actor_class->hide_all        = clutter_group_real_hide_all;
   actor_class->request_coords  = clutter_group_request_coords;
   actor_class->query_coords    = clutter_group_query_coords;
+  actor_class->unrealize       = clutter_group_unrealize;
 
   /**
    * ClutterGroup::add:
@@ -368,23 +519,6 @@ clutter_group_new (void)
   return g_object_new (CLUTTER_TYPE_GROUP, NULL);
 }
 
-/**
- * clutter_group_add:
- * @group: A #ClutterGroup
- * @actor: A #ClutterActor 
- *
- * Adds a new child #ClutterActor to the #ClutterGroup.
- *
- * @Deprecated: 0.4: This function is obsolete, use
- *   clutter_container_add_actor() instead.
- */
-void
-clutter_group_add (ClutterGroup *group,
-                   ClutterActor *actor)
-{
-  clutter_container_add_actor (CLUTTER_CONTAINER (group), actor);
-}
-
 /**
  * clutter_group_add_many_valist:
  * @group: a #ClutterGroup
@@ -411,7 +545,7 @@ clutter_group_add_many_valist (ClutterGroup *group,
  * @first_actor: the #ClutterActor actor to add to the group
  * @Varargs: additional actors to add to the group
  *
- * Adds a NULL-terminated list of actors to a group.  This function is
+ * Adds a %NULL-terminated list of actors to a group.  This function is
  * equivalent to calling clutter_group_add() for each member of the list.
  *
  * @Deprecated: 0.4: This function is obsolete, use clutter_container_add()
@@ -432,7 +566,7 @@ clutter_group_add_many (ClutterGroup *group,
 /**
  * clutter_group_remove
  * @group: A #ClutterGroup
- * @actor: A #ClutterActor 
+ * @actor: A #ClutterActor
  *
  * Removes a child #ClutterActor from the parent #ClutterGroup.
  *
@@ -472,9 +606,9 @@ clutter_group_remove_all (ClutterGroup *group)
 /**
  * clutter_group_get_children:
  * @self: A #ClutterGroup
- * 
+ *
  * Get a list containing all actors contained in the group.
- * 
+ *
  * Return value: A list of #ClutterActors. You  should free the returned
  *   list using g_list_free() when finished using it.
  *
@@ -492,9 +626,9 @@ clutter_group_get_children (ClutterGroup *self)
  * @self: A #ClutterGroup
  *
  * Gets the number of actors held in the group.
- * 
+ *
  * Return value: The number of child actors held in the group.
- * 
+ *
  * Since: 0.2
  **/
 gint
@@ -508,8 +642,8 @@ clutter_group_get_n_children (ClutterGroup *self)
 /**
  * clutter_group_get_nth_child:
  * @self: A #ClutterGroup
- * @index_: the position of the requested actor. 
- * 
+ * @index_: the position of the requested actor.
+ *
  * Gets a groups child held at @index_ in stack.
  *
  * Return value: A Clutter actor or NULL if @index_ is invalid.
@@ -525,107 +659,29 @@ clutter_group_get_nth_child (ClutterGroup *self,
   return g_list_nth_data (self->priv->children, index_);
 }
 
-/**
- * clutter_group_find_child_by_id:
- * @self: A #ClutterGroup
- * @id: A unique #Clutteractor ID
- *
- * Finds a child actor of a group by its unique ID. Search recurses
- * into any child groups.
- *
- * Returns: the #ClutterActor if found, or NULL.
- */
-ClutterActor *
-clutter_group_find_child_by_id (ClutterGroup *self,
-				guint         id)
-{
-  ClutterActor *actor = NULL;
-  GList *l;
-
-  g_return_val_if_fail (CLUTTER_IS_GROUP (self), NULL);
-
-  if (clutter_actor_get_id (CLUTTER_ACTOR (self)) == id)
-    return CLUTTER_ACTOR (self);
-
-  for (l = self->priv->children; l; l = l->next)
-    {
-      ClutterActor *child = l->data;
-
-      if (clutter_actor_get_id (child) == id)
-        {
-          actor = child;
-          break;
-        }
-
-      if (CLUTTER_IS_GROUP (child))
-        {
-          actor = clutter_group_find_child_by_id (CLUTTER_GROUP (child), id);
-          if (actor)
-            break;
-	}
-      else if (CLUTTER_IS_BOX (child))
-        {
-          /* HACK, HACK, HACK. this has been fixes in 0.6 */
-          actor = _clutter_box_find_child_by_id (CLUTTER_BOX (child), id);
-          if (actor)
-            break;
-        }
-    }
-
-  return actor;
-}
-
 /**
  * clutter_group_raise:
  * @self: a #ClutterGroup
  * @actor: a #ClutterActor
  * @sibling: a #ClutterActor
  *
- * FIXME
+ * Raises @actor to @sibling level in the depth ordering.
+ *
+ * Deprecated: 0.6: Use clutter_container_raise_child() instead.
  */
 void
-clutter_group_raise (ClutterGroup   *self,
-		     ClutterActor *actor, 
+clutter_group_raise (ClutterGroup *self,
+		     ClutterActor *actor,
 		     ClutterActor *sibling)
 {
-  ClutterGroupPrivate *priv;
-  gint                 pos;
+  g_return_if_fail (CLUTTER_IS_GROUP (self));
+  g_return_if_fail (CLUTTER_IS_ACTOR (actor));
+  g_return_if_fail (sibling == NULL || CLUTTER_IS_ACTOR (sibling));
 
-  g_return_if_fail (actor != sibling);
-
-  priv = self->priv;
-
-  pos = g_list_index (priv->children, actor) + 1;
-
-  priv->children = g_list_remove (priv->children, actor); 
-
-  if (sibling == NULL)
-    {
-      GList *last_item;
-
-      /* Raise top */
-      last_item = g_list_last (priv->children);
-
-      if (last_item)
-	sibling = last_item->data;
-      
-      priv->children = g_list_append (priv->children, actor);
-    }
-  else
-    {
-      priv->children = g_list_insert (priv->children, actor, pos);
-    }
-
-  /* set Z ordering a value below, this will then call sort
-   * as values are equal ordering shouldn't change but Z
-   * values will be correct.
-   * FIXME: optimise
-   */
-  if ( sibling &&
-       clutter_actor_get_depth(sibling) != clutter_actor_get_depth (actor))
-    clutter_actor_set_depth (actor,
-			     clutter_actor_get_depth (sibling));
+  if (actor == sibling)
+    return;
 
+  clutter_container_raise_child (CLUTTER_CONTAINER (self), actor, sibling);
 }
 
 /**
@@ -634,78 +690,38 @@ clutter_group_raise (ClutterGroup   *self,
  * @actor: a #ClutterActor
  * @sibling: a #ClutterActor
  *
- * FIXME
+ * Lowers @actor to @sibling level in the depth ordering.
+ *
+ * Deprecated: 0.6: Use clutter_container_lower_child() instead
  */
 void
-clutter_group_lower (ClutterGroup   *self,
-		     ClutterActor *actor, 
+clutter_group_lower (ClutterGroup *self,
+		     ClutterActor *actor,
 		     ClutterActor *sibling)
 {
-  ClutterGroupPrivate *priv;
-  gint               pos;
+  g_return_if_fail (CLUTTER_IS_GROUP (self));
+  g_return_if_fail (CLUTTER_IS_ACTOR (actor));
+  g_return_if_fail (sibling == NULL || CLUTTER_IS_ACTOR (sibling));
 
-  g_return_if_fail (actor != sibling);
+  if (actor == sibling)
+    return;
 
-  priv = self->priv;
-
-  pos = g_list_index (priv->children, actor) - 1;
-
-  priv->children = g_list_remove (priv->children, actor); 
-
-  if (sibling == NULL)
-    {
-      GList *last_item;
-      /* Raise top */
-      last_item = g_list_first (priv->children);
-
-      if (last_item)
-	sibling = last_item->data;
-
-      priv->children = g_list_prepend (priv->children, actor);
-    }
-  else
-    priv->children = g_list_insert (priv->children, actor, pos);
-
-  /* See comment in group_raise for this */
-  if (sibling &&
-      clutter_actor_get_depth(sibling) != clutter_actor_get_depth(actor))
-    clutter_actor_set_depth (actor,
-			     clutter_actor_get_depth(sibling));
-}
-
-static gint 
-sort_z_order (gconstpointer a,
-              gconstpointer b)
-{
-  if (clutter_actor_get_depth (CLUTTER_ACTOR(a)) 
-         == clutter_actor_get_depth (CLUTTER_ACTOR(b))) 
-    return 0;
-
-  if (clutter_actor_get_depth (CLUTTER_ACTOR(a)) 
-        > clutter_actor_get_depth (CLUTTER_ACTOR(b))) 
-    return 1;
-
-  return -1;
+  clutter_container_lower_child (CLUTTER_CONTAINER (self), actor, sibling);
 }
 
 /**
  * clutter_group_sort_depth_order:
  * @self: A #ClutterGroup
  *
- * Sorts a #ClutterGroup's children by there depth value.  
- * This function should not be used by applications. 
- **/
+ * Sorts a #ClutterGroup's children by their depth value.
+ * This function should not be used by applications.
+ *
+ * Deprecated: 0.6: Use clutter_container_sort_depth_order() instead.
+ */
 void
 clutter_group_sort_depth_order (ClutterGroup *self)
 {
-  ClutterGroupPrivate *priv;
+  g_return_if_fail (CLUTTER_IS_GROUP (self));
 
-  g_return_if_fail (CLUTTER_IS_GROUP(self));
-
-  priv = self->priv;
-
-  priv->children = g_list_sort (priv->children, sort_z_order);
-
-  if (CLUTTER_ACTOR_IS_VISIBLE (CLUTTER_ACTOR(self)))
-    clutter_actor_queue_redraw (CLUTTER_ACTOR(self));
+  clutter_container_sort_depth_order (CLUTTER_CONTAINER (self));
 }
