@@ -23,8 +23,8 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#ifndef _HAVE_CLUTTER_GROUP_H
-#define _HAVE_CLUTTER_GROUP_H
+#ifndef __CLUTTER_GROUP_H__
+#define __CLUTTER_GROUP_H__
 
 #include <glib-object.h>
 #include <clutter/clutter-actor.h>
@@ -76,26 +76,30 @@ struct _ClutterGroupClass
 		  ClutterActor *child);
 
   /* padding for future expansion */
-  void (*_clutter_group_1) (void);
-  void (*_clutter_group_2) (void);
-  void (*_clutter_group_3) (void);
-  void (*_clutter_group_4) (void);
-  void (*_clutter_group_5) (void);
-  void (*_clutter_group_6) (void);
+  void (*_clutter_reserved1) (void);
+  void (*_clutter_reserved2) (void);
+  void (*_clutter_reserved3) (void);
+  void (*_clutter_reserved4) (void);
+  void (*_clutter_reserved5) (void);
+  void (*_clutter_reserved6) (void);
 };
 
 GType         clutter_group_get_type         (void) G_GNUC_CONST;
 ClutterActor *clutter_group_new              (void);
-ClutterActor *clutter_group_find_child_by_id (ClutterGroup    *self,
-					      guint            id);
 ClutterActor *clutter_group_get_nth_child    (ClutterGroup    *self,
                                               gint             index_);
 gint          clutter_group_get_n_children   (ClutterGroup    *self);
 void          clutter_group_remove_all       (ClutterGroup    *group);
 
+#define clutter_group_add(group,actor)                  G_STMT_START {  \
+  if (CLUTTER_IS_GROUP ((group)) && CLUTTER_IS_ACTOR ((actor)))         \
+    {                                                                   \
+      ClutterContainer *_container = (ClutterContainer *) (group);      \
+      ClutterActor *_actor = (ClutterActor *) (actor);                  \
+      clutter_container_add_actor (_container, _actor);                 \
+    }                                                   } G_STMT_END
+
 #ifndef CLUTTER_DISABLE_DEPRECATED
-void          clutter_group_add              (ClutterGroup    *group,
-                                              ClutterActor    *actor);
 void          clutter_group_add_many         (ClutterGroup    *group,
                                               ClutterActor    *first_actor,
                                               ...) G_GNUC_NULL_TERMINATED;
@@ -104,8 +108,6 @@ void          clutter_group_add_many_valist  (ClutterGroup    *group,
                                               va_list          var_args);
 void          clutter_group_remove           (ClutterGroup    *group,
                                               ClutterActor    *actor);
-#endif /* CLUTTER_DISABLE_DEPRECATED */
-
 void          clutter_group_raise            (ClutterGroup    *self,
 					      ClutterActor    *actor, 
 					      ClutterActor    *sibling);
@@ -113,7 +115,8 @@ void          clutter_group_lower            (ClutterGroup    *self,
 					      ClutterActor    *actor, 
 					      ClutterActor    *sibling);
 void          clutter_group_sort_depth_order (ClutterGroup    *self);
+#endif /* CLUTTER_DISABLE_DEPRECATED */
 
 G_END_DECLS
 
-#endif
+#endif /* __CLUTTER_GROUP_H__ */
