@@ -27,13 +27,14 @@
  * SECTION:clutter-label
  * @short_description: Actor for displaying text
  *
- * #ClutterLabel is a #ClutterTexture that displays text.
+ * #ClutterLabel is a #ClutterActor that displays text using Pango.
  */
 
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif
 
 #include "clutter-label.h"
-#include "clutter-layout.h"
 #include "clutter-main.h"
 #include "clutter-enum-types.h"
 #include "clutter-private.h"
@@ -44,13 +45,7 @@
 
 #define DEFAULT_FONT_NAME	"Sans 10"
 
-static void clutter_layout_iface_init (ClutterLayoutIface *iface);
-
-G_DEFINE_TYPE_WITH_CODE (ClutterLabel,
-                         clutter_label,
-                         CLUTTER_TYPE_ACTOR,
-                         G_IMPLEMENT_INTERFACE (CLUTTER_TYPE_LAYOUT,
-                                                clutter_layout_iface_init));
+G_DEFINE_TYPE (ClutterLabel, clutter_label, CLUTTER_TYPE_ACTOR)
 
 /* Probably move into main */
 static PangoClutterFontMap  *_font_map = NULL;
@@ -64,11 +59,11 @@ enum
   PROP_COLOR,
   PROP_ATTRIBUTES,
   PROP_USE_MARKUP,
-  PROP_ALIGNMENT, 		/* FIXME */
+  PROP_ALIGNMENT,
   PROP_WRAP,
   PROP_WRAP_MODE,
+  PROP_JUSTIFY,
   PROP_ELLIPSIZE,
-  PROP_LAYOUT_FLAGS,
 };
 
 #define CLUTTER_LABEL_GET_PRIVATE(obj) \
@@ -77,11 +72,11 @@ enum
 struct _ClutterLabelPrivate
 {
   PangoContext         *context;
-  PangoFontDescription *desc;
+  PangoFontDescription *font_desc;
   
   ClutterColor          fgcol;
 
-  ClutterLayoutFlags    layout_flags;
+  ClutterActorBox       allocation;
   
   gchar                *text;
   gchar                *font_name;
@@ -96,13 +91,334 @@ struct _ClutterLabelPrivate
   guint                 ellipsize        : 3;
   guint                 single_line_mode : 1;
   guint                 wrap_mode        : 3;
+  guint                 justify          : 1;
 
   PangoAttrList        *attrs;
   PangoAttrList        *effective_attrs;
   PangoLayout          *layout;
   gint                  width_chars;
+  gint                  wrap_width;
 };
 
+static gint
+get_label_char_width (ClutterLabel *label)
+{
+  ClutterLabelPrivate *priv = label->priv;
+  PangoContext *context;
+  PangoFontMetrics *metrics;
+  gint char_width, digit_width, char_pixels, w;
+        
+  context = pango_layout_get_context (priv->layout);
+  metrics = pango_context_get_metrics (context, priv->font_desc, 
+                                       pango_context_get_language (context));
+  char_width = pango_font_metrics_get_approximate_char_width (metrics);
+  digit_width = pango_font_metrics_get_approximate_digit_width (metrics);
+  char_pixels = MAX (char_width, digit_width);
+  pango_font_metrics_unref (metrics);
+
+  if (priv->width_chars < 0)
+    {
+      PangoRectangle rect;
+
+      pango_layout_set_width (priv->layout, -1);
+      pango_layout_get_extents (priv->layout, NULL, &rect);
+
+      w = char_pixels * 5;
+      w = MIN (rect.width, w);
+    }
+  else
+    {
+      /* enforce minimum width for ellipsized labels at ~5 chars */
+      w = char_pixels * MAX (priv->width_chars, 5);
+    }
+
+  return w;
+}
+
+static gint
+get_label_wrap_width (ClutterLabel *label)
+{
+  ClutterLabelPrivate *priv = label->priv;
+
+  if (priv->wrap_width < 0)
+    {
+      if (priv->width_chars > 0)
+        priv->wrap_width = get_label_char_width (label);
+      else
+        {
+          PangoLayout *tmp;
+
+          tmp = pango_layout_new (_context);
+          pango_layout_set_font_description (tmp, priv->font_desc);
+          pango_layout_set_text (tmp, "This is a very long string, which "
+                                      "should be enough for a wrap width.",
+                                 -1);
+          pango_layout_get_size (tmp, &priv->wrap_width, NULL);
+          g_object_unref (tmp);
+        }
+    }
+
+  return priv->wrap_width;
+}
+
+static void
+clutter_label_ensure_layout (ClutterLabel *label)
+{
+  ClutterLabelPrivate *priv;
+  ClutterUnit raw_width;
+  gint width;
+
+  priv  = label->priv;
+
+  /* use the last size requested, if any */
+  raw_width = priv->allocation.x2 - priv->allocation.x1;
+  width = CLUTTER_UNITS_TO_DEVICE (raw_width);
+
+  if (!priv->layout)
+    {
+      priv->layout = pango_layout_new (_context);
+
+      if (priv->effective_attrs)
+	pango_layout_set_attributes (priv->layout, priv->effective_attrs);
+  
+      pango_layout_set_alignment (priv->layout, priv->alignment);
+      pango_layout_set_ellipsize (priv->layout, priv->ellipsize);
+      pango_layout_set_single_paragraph_mode (priv->layout, 
+					      priv->single_line_mode);
+      
+      pango_layout_set_font_description (priv->layout, priv->font_desc);
+      pango_layout_set_justify (priv->layout, priv->justify);
+     
+      if (priv->text)
+        { 
+          if (!priv->use_markup)
+            pango_layout_set_text (priv->layout, priv->text, -1);
+          else
+            pango_layout_set_markup (priv->layout, priv->text, -1);
+        }
+      
+      if (priv->ellipsize != PANGO_ELLIPSIZE_NONE)
+        pango_layout_set_width (priv->layout, raw_width > 0 ? CLUTTER_UNITS_TO_PANGO_UNIT (raw_width)
+                                                            : -1);
+      else if (priv->wrap)
+        {
+          pango_layout_set_wrap  (priv->layout, priv->wrap_mode);
+
+          if (width > 0)
+	    pango_layout_set_width (priv->layout, CLUTTER_UNITS_TO_PANGO_UNIT (raw_width));
+          else
+            {
+              /* this was adapted from the GtkLabel code */
+              ClutterActor *stage = clutter_stage_get_default ();
+              gint stage_width = clutter_actor_get_width (stage);
+              gint longest_paragraph, height, wrap_width;
+              PangoRectangle logical_rect;
+
+              pango_layout_set_width (priv->layout, -1);
+              pango_layout_get_extents (priv->layout, NULL, &logical_rect);
+
+              width = logical_rect.width;
+
+              longest_paragraph = width;
+
+              wrap_width = get_label_wrap_width (label);
+              width = MAX (width, wrap_width);
+              width = MIN (width, PANGO_SCALE * (stage_width + 1) / 2);
+
+              pango_layout_set_width (priv->layout, width);
+              pango_layout_get_extents (priv->layout, NULL, &logical_rect);
+              width = logical_rect.width;
+              height = logical_rect.height;
+
+              if (longest_paragraph > 0)
+                {
+                  gint n_lines, perfect_width;
+
+                  n_lines = pango_layout_get_line_count (priv->layout);
+                  perfect_width = (longest_paragraph + n_lines - 1) / n_lines;
+
+                  if (perfect_width < width)
+                    {
+                      pango_layout_set_width (priv->layout, perfect_width);
+                      pango_layout_get_extents (priv->layout, NULL,
+                                                &logical_rect);
+
+                      if (logical_rect.height <= height)
+                        width = logical_rect.width = width;
+                      else
+                        {
+                          gint mid_width = (perfect_width + width) / 2;
+
+                          if (mid_width > perfect_width)
+                            {
+                              pango_layout_set_width (priv->layout, mid_width);
+                              pango_layout_get_extents (priv->layout, NULL,
+                                                        &logical_rect);
+
+                              if (logical_rect.height <= height);
+                                width = logical_rect.width;
+                            }
+                        }
+                    }
+                }
+
+              pango_layout_set_width (priv->layout, width);
+            }
+	}
+      else
+	pango_layout_set_width (priv->layout, raw_width > 0 ? CLUTTER_UNITS_TO_PANGO_UNIT (raw_width) 
+                                                            : -1);
+    }
+
+  CLUTTER_NOTE (ACTOR, "Label width set to %d pixels", width);
+}
+
+static void
+clutter_label_clear_layout (ClutterLabel *label)
+{
+  if (label->priv->layout)
+    {
+      g_object_unref (label->priv->layout);
+      label->priv->layout = NULL;
+    }
+}
+
+static void
+clutter_label_paint (ClutterActor *self)
+{
+  ClutterLabel        *label = CLUTTER_LABEL (self);
+  ClutterLabelPrivate *priv = label->priv;
+
+  if (priv->font_desc == NULL || priv->text == NULL)
+    {
+      CLUTTER_NOTE (ACTOR, "layout: %p, desc: %p, text %p",
+		    priv->layout,
+		    priv->font_desc ? priv->font_desc : 0x0,
+		    priv->text ? priv->text : 0x0);
+      return;
+    }
+
+  CLUTTER_NOTE (PAINT, "painting label (text:`%s')", priv->text);
+
+  clutter_label_ensure_layout (label);
+
+  priv->fgcol.alpha = clutter_actor_get_opacity (self);
+
+  pango_clutter_render_layout (priv->layout, 0, 0, &priv->fgcol, 0);
+}
+
+static void
+clutter_label_query_coords (ClutterActor    *self,
+			    ClutterActorBox *box)
+{
+  ClutterLabel *label = CLUTTER_LABEL(self);
+  ClutterLabelPrivate *priv;
+  ClutterActorBox layout_box = { 0, };
+  PangoRectangle logical_rect;
+
+  priv = label->priv;
+
+  if (priv->wrap)
+    clutter_label_clear_layout (label);
+
+  clutter_label_ensure_layout (label);
+
+  pango_layout_get_extents (priv->layout, NULL, &logical_rect);
+
+  layout_box.x1 = box->x1;
+  layout_box.x2 = box->x1 + CLUTTER_UNITS_FROM_PANGO_UNIT (logical_rect.width);
+  layout_box.y1 = box->y1;
+  layout_box.y2 = box->y1 + CLUTTER_UNITS_FROM_PANGO_UNIT (logical_rect.height);
+
+  if ((priv->allocation.x2 - priv->allocation.x1) > 0)
+    {
+      ClutterUnit alloc_width, alloc_height;
+
+      alloc_width = priv->allocation.x2 - priv->allocation.x1;
+      alloc_height = priv->allocation.y2 - priv->allocation.y1;
+
+      if ((alloc_width >= (layout_box.x2 - layout_box.x1)) &&
+          (alloc_height >= (layout_box.y2 - layout_box.y1)))
+        *box = priv->allocation;
+      else
+        *box = layout_box;
+
+      return;
+    }
+  else
+    *box = layout_box;
+}
+
+static void
+clutter_label_request_coords (ClutterActor    *self,
+			      ClutterActorBox *box)
+{
+  ClutterLabel *label = CLUTTER_LABEL (self);
+  ClutterLabelPrivate *priv = label->priv;
+
+  if (priv->ellipsize)
+    {
+      if (priv->layout)
+        {
+          gint width;
+          PangoRectangle logical;
+
+          width = CLUTTER_UNITS_TO_PANGO_UNIT (box->x2 - box->x1);
+
+          pango_layout_set_width (priv->layout, -1);
+          pango_layout_get_extents (priv->layout, NULL, &logical);
+
+          if (logical.width > width)
+            pango_layout_set_width (priv->layout, width);
+        }
+    }
+  else
+    clutter_label_clear_layout (label);
+
+  priv->allocation = *box;
+
+  CLUTTER_ACTOR_CLASS (clutter_label_parent_class)->request_coords (self, box);
+}
+
+static void 
+clutter_label_dispose (GObject *object)
+{
+  ClutterLabel         *self = CLUTTER_LABEL(object);
+  ClutterLabelPrivate  *priv;  
+
+  priv = self->priv;
+
+  if (priv->layout)
+    {
+      g_object_unref (priv->layout);
+      priv->layout = NULL;
+    }
+
+  if (priv->context)
+    {
+      g_object_unref (priv->context);
+      priv->context = NULL;
+    }
+
+  G_OBJECT_CLASS (clutter_label_parent_class)->dispose (object);
+}
+
+static void 
+clutter_label_finalize (GObject *object)
+{
+  ClutterLabel         *self = CLUTTER_LABEL(object);
+  ClutterLabelPrivate  *priv;  
+
+  priv = self->priv;
+  
+  if (priv->font_desc)
+    pango_font_description_free (priv->font_desc);
+
+  g_free (priv->text);
+  g_free (priv->font_name);
+
+  G_OBJECT_CLASS (clutter_label_parent_class)->finalize (object);
+}
 
 static void
 clutter_label_set_property (GObject      *object, 
@@ -138,7 +454,10 @@ clutter_label_set_property (GObject      *object,
       break;
     case PROP_WRAP:
       clutter_label_set_line_wrap (label, g_value_get_boolean (value));
-      break;	  
+      break;
+    case PROP_JUSTIFY:
+      clutter_label_set_justify (label, g_value_get_boolean (value));
+      break;
     case PROP_WRAP_MODE:
       clutter_label_set_line_wrap_mode (label, g_value_get_enum (value));
       break;	  
@@ -161,7 +480,7 @@ clutter_label_get_property (GObject    *object,
   ClutterLabelPrivate *priv;
   ClutterColor         color;
 
-  label = CLUTTER_LABEL(object);
+  label = CLUTTER_LABEL (object);
   priv = label->priv;
 
   switch (prop_id) 
@@ -185,6 +504,9 @@ clutter_label_get_property (GObject    *object,
     case PROP_USE_MARKUP:
       g_value_set_boolean (value, priv->use_markup);
       break;
+    case PROP_JUSTIFY:
+      g_value_set_boolean (value, priv->justify);
+      break;
     case PROP_WRAP:
       g_value_set_boolean (value, priv->wrap);
       break;
@@ -194,198 +516,24 @@ clutter_label_get_property (GObject    *object,
     case PROP_ELLIPSIZE:
       g_value_set_enum (value, priv->ellipsize);
       break;
-    case PROP_LAYOUT_FLAGS:
-      g_value_set_flags (value, priv->layout_flags);
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
     } 
 }
 
-static void
-clutter_label_ensure_layout (ClutterLabel *label,
-                             gint          width)
-{
-  ClutterLabelPrivate  *priv;
-
-  priv   = label->priv;
-
-  if (!priv->layout)
-    {
-      priv->layout = pango_layout_new (_context);
-
-      if (priv->effective_attrs)
-	pango_layout_set_attributes (priv->layout, priv->effective_attrs);
-  
-      pango_layout_set_alignment (priv->layout, priv->alignment);
-      pango_layout_set_ellipsize (priv->layout, priv->ellipsize);
-      pango_layout_set_single_paragraph_mode (priv->layout, 
-					      priv->single_line_mode);
-      
-      pango_layout_set_font_description (priv->layout, priv->desc);
-      
-      if (!priv->use_markup)
-        pango_layout_set_text (priv->layout, priv->text, -1);
-      else
-        pango_layout_set_markup (priv->layout, priv->text, -1);
-      
-      if (priv->wrap)
-	pango_layout_set_wrap  (priv->layout, priv->wrap_mode);
-
-      if ((priv->ellipsize || priv->wrap) && width > 0)
-	{
-	  pango_layout_set_width (priv->layout, width * PANGO_SCALE);
-	}
-      else
-	pango_layout_set_width (priv->layout, -1);
-    }
-}
-
-static void
-clutter_label_clear_layout (ClutterLabel *label)
-{
-  if (label->priv->layout)
-    {
-      g_object_unref (label->priv->layout);
-      label->priv->layout = NULL;
-    }
-}
-
-void
-clutter_label_paint (ClutterActor *self)
-{
-  ClutterLabel         *label;
-  ClutterLabelPrivate  *priv;
-
-  label  = CLUTTER_LABEL(self);
-  priv   = label->priv;
-  
-  if (priv->desc == NULL || priv->text == NULL)
-    {
-      CLUTTER_NOTE (ACTOR, "layout: %p , desc: %p, text %p",
-		    priv->layout,
-		    priv->desc,
-		    priv->text);
-      return;
-    }
-
-  clutter_label_ensure_layout (label, clutter_actor_get_width (self));
-
-  priv->fgcol.alpha = clutter_actor_get_opacity(self);
-
-  pango_clutter_render_layout (priv->layout, 0, 0, &priv->fgcol, 0);
-}
-
-static void
-clutter_label_query_coords (ClutterActor        *self,
-			    ClutterActorBox     *box)
-{
-  ClutterLabel         *label = CLUTTER_LABEL(self);
-  ClutterLabelPrivate  *priv;  
-  PangoRectangle        logical_rect;
-
-  priv = label->priv;
-
-  clutter_label_ensure_layout (label, CLUTTER_UNITS_TO_INT (box->x2 - box->x1));
-
-  pango_layout_get_extents (priv->layout, NULL, &logical_rect);
-
-  box->x2 = box->x1 + CLUTTER_UNITS_FROM_PANGO_UNIT (logical_rect.width);
-  box->y2 = box->y1 + CLUTTER_UNITS_FROM_PANGO_UNIT (logical_rect.height);
-
-  return;
-}
-
-static void
-clutter_label_request_coords (ClutterActor        *self,
-			      ClutterActorBox     *box)
-{
-  /* do we need to do anything ? */
-  clutter_label_clear_layout (CLUTTER_LABEL (self));
-}
-
-static void 
-clutter_label_dispose (GObject *object)
-{
-  ClutterLabel         *self = CLUTTER_LABEL(object);
-  ClutterLabelPrivate  *priv;  
-
-  priv = self->priv;
-  
-  if (priv->layout)
-    {
-      g_object_unref (priv->layout);
-      priv->layout = NULL;
-    }
-  
-  if (priv->desc)
-    {
-      pango_font_description_free (priv->desc);    
-      priv->desc = NULL;
-    }
-
-  g_free (priv->text);
-  priv->text = NULL;
-
-  g_free (priv->font_name);
-  priv->font_name = NULL;
-      
-  if (priv->context)
-    {
-      g_object_unref (priv->context);
-      priv->context = NULL;
-    }
-
-  G_OBJECT_CLASS (clutter_label_parent_class)->dispose (object);
-}
-
-static void 
-clutter_label_finalize (GObject *object)
-{
-  G_OBJECT_CLASS (clutter_label_parent_class)->finalize (object);
-}
-
-static ClutterLayoutFlags
-clutter_layout_label_get_layout_flags (ClutterLayout *layout)
-{
-  return CLUTTER_LABEL (layout)->priv->layout_flags;
-}
-
-static void
-clutter_layout_label_height_for_width (ClutterLayout *layout,
-                                       ClutterUnit    width,
-                                       ClutterUnit   *height)
-{
-  ClutterLabel *label = CLUTTER_LABEL (layout);
-  gint layout_height;
-
-  clutter_label_ensure_layout (label, CLUTTER_UNITS_TO_INT (width));
-  pango_layout_get_pixel_size (label->priv->layout, NULL, &layout_height);
-
-  if (height)
-    *height = CLUTTER_UNITS_FROM_INT (layout_height);
-}
-
-static void
-clutter_layout_iface_init (ClutterLayoutIface *iface)
-{
-  iface->get_layout_flags = clutter_layout_label_get_layout_flags;
-  iface->height_for_width = clutter_layout_label_height_for_width;
-}
-
 static void
 clutter_label_class_init (ClutterLabelClass *klass)
 {
-  GObjectClass        *gobject_class = G_OBJECT_CLASS (klass);
+  GObjectClass      *gobject_class = G_OBJECT_CLASS (klass);
   ClutterActorClass *actor_class = CLUTTER_ACTOR_CLASS (klass);
 
   actor_class->paint           = clutter_label_paint;
   actor_class->request_coords  = clutter_label_request_coords;
   actor_class->query_coords    = clutter_label_query_coords;
 
-  gobject_class->finalize   = clutter_label_finalize;
-  gobject_class->dispose    = clutter_label_dispose;
+  gobject_class->finalize     = clutter_label_finalize;
+  gobject_class->dispose      = clutter_label_dispose;
   gobject_class->set_property = clutter_label_set_property;
   gobject_class->get_property = clutter_label_get_property;
 
@@ -395,16 +543,14 @@ clutter_label_class_init (ClutterLabelClass *klass)
 			  "Font Name",
 			  "Pango font description",
 			  NULL,
-			  G_PARAM_CONSTRUCT | CLUTTER_PARAM_READWRITE));
-
+			  CLUTTER_PARAM_READWRITE));
   g_object_class_install_property
     (gobject_class, PROP_TEXT,
      g_param_spec_string ("text",
 			  "Text",
 			  "Text to render",
 			  NULL,
-			  G_PARAM_CONSTRUCT | CLUTTER_PARAM_READWRITE));
-
+			  CLUTTER_PARAM_READWRITE));
   g_object_class_install_property
     (gobject_class, PROP_COLOR,
      g_param_spec_boxed ("color",
@@ -412,65 +558,70 @@ clutter_label_class_init (ClutterLabelClass *klass)
 			 "Font Colour",
 			 CLUTTER_TYPE_COLOR,
 			 CLUTTER_PARAM_READWRITE));
-
   g_object_class_install_property 
     (gobject_class, PROP_ATTRIBUTES,
      g_param_spec_boxed ("attributes",
 			 "Attributes",
-			 "A list of style attributes to apply to the" 
+			 "A list of style attributes to apply to the " 
 			 "text of the label",
 			 PANGO_TYPE_ATTR_LIST,
 			 CLUTTER_PARAM_READWRITE));
-
   g_object_class_install_property 
     (gobject_class, PROP_USE_MARKUP,
      g_param_spec_boolean ("use-markup",
 			   "Use markup",
-			   "The text of the label includes XML markup." 
+			   "The text of the label includes XML markup. " 
 			   "See pango_parse_markup()",
 			   FALSE,
 			   CLUTTER_PARAM_READWRITE));
-
   g_object_class_install_property 
     (gobject_class, PROP_WRAP,
      g_param_spec_boolean ("wrap",
 			   "Line wrap",
 			   "If set, wrap lines if the text becomes too wide",
-			   TRUE,
+			   FALSE,
 			   CLUTTER_PARAM_READWRITE));
-
   g_object_class_install_property 
     (gobject_class, PROP_WRAP_MODE,
      g_param_spec_enum ("wrap-mode",
 			"Line wrap mode",
-			"If wrap is set, controls how linewrapping is done",
+			"If wrap is set, controls how line-wrapping is done",
 			PANGO_TYPE_WRAP_MODE,
 			PANGO_WRAP_WORD,
 			CLUTTER_PARAM_READWRITE));
-
   g_object_class_install_property 
     (gobject_class, PROP_ELLIPSIZE,
      g_param_spec_enum ( "ellipsize",
 			 "Ellipsize",
-			 "The preferred place to ellipsize the string,"
+			 "The preferred place to ellipsize the string, "
 			 "if the label does not have enough room to "
 			 "display the entire string",
 			 PANGO_TYPE_ELLIPSIZE_MODE,
 			 PANGO_ELLIPSIZE_NONE,
 			 CLUTTER_PARAM_READWRITE));
-
   g_object_class_install_property 
     (gobject_class, PROP_ALIGNMENT,
      g_param_spec_enum ( "alignment",
 			 "Alignment",
-			 "The preferred alignment for the string,",
+			 "The preferred alignment for the string",
 			 PANGO_TYPE_ALIGNMENT,
 			 PANGO_ALIGN_LEFT,
 			 CLUTTER_PARAM_READWRITE));
-
-  g_object_class_override_property (gobject_class,
-                                    PROP_LAYOUT_FLAGS,
-                                    "layout-flags");
+  /**
+   * ClutterLabel:justify:
+   *
+   * Whether the contents of the label should be justified on both
+   * margins.
+   *
+   * Since: 0.6
+   */
+  g_object_class_install_property (gobject_class,
+                                   PROP_JUSTIFY,
+                                   g_param_spec_boolean ("justify",
+                                                         "Justify",
+                                                         "Whether the contents of the label should be justified",
+                                                         FALSE,
+                                                         CLUTTER_PARAM_READWRITE));
 
   g_type_class_add_private (gobject_class, sizeof (ClutterLabelPrivate));
 }
@@ -498,15 +649,19 @@ clutter_label_init (ClutterLabel *self)
     }
 
   priv->alignment     = PANGO_ALIGN_LEFT;
-  priv->wrap          = TRUE;
+  priv->wrap          = FALSE;
   priv->wrap_mode     = PANGO_WRAP_WORD;
   priv->ellipsize     = PANGO_ELLIPSIZE_NONE;
   priv->use_underline = FALSE;
   priv->use_markup    = FALSE;
+  priv->justify       = FALSE;
+
   priv->layout        = NULL;
   priv->text          = NULL;
   priv->attrs         = NULL;
-  priv->layout_flags  = CLUTTER_LAYOUT_HEIGHT_FOR_WIDTH;
+
+  priv->width_chars   = -1;
+  priv->wrap_width    = -1;
 
   priv->fgcol.red     = 0;
   priv->fgcol.green   = 0;
@@ -514,9 +669,7 @@ clutter_label_init (ClutterLabel *self)
   priv->fgcol.alpha   = 255;
 
   priv->font_name     = g_strdup (DEFAULT_FONT_NAME);
-  priv->desc          = pango_font_description_from_string (priv->font_name);
-
-  CLUTTER_MARK();
+  priv->font_desc     = pango_font_description_from_string (priv->font_name);
 }
 
 /**
@@ -532,16 +685,10 @@ ClutterActor*
 clutter_label_new_with_text (const gchar *font_name,
 		             const gchar *text)
 {
-  ClutterActor *label;
-
-  CLUTTER_MARK();
-
-  label = clutter_label_new ();
-
-  clutter_label_set_font_name (CLUTTER_LABEL(label), font_name);
-  clutter_label_set_text (CLUTTER_LABEL(label), text);
-
-  return label;
+  return g_object_new (CLUTTER_TYPE_LABEL,
+                       "font-name", font_name,
+                       "text", text,
+                       NULL);
 }
 
 /**
@@ -550,23 +697,21 @@ clutter_label_new_with_text (const gchar *font_name,
  * @text: the text to be displayed
  * @color: #ClutterColor for text
  *
- * Creates a new #ClutterLabel displaying @text with color @color 
+ * Creates a new #ClutterLabel displaying @text with @color 
  * using @font_name.
  *
  * Return value: a #ClutterLabel
  */
 ClutterActor*
-clutter_label_new_full (const gchar  *font_name,
-			const gchar  *text,
-			ClutterColor *color)
+clutter_label_new_full (const gchar        *font_name,
+			const gchar        *text,
+			const ClutterColor *color)
 {
-  /* FIXME: really new_with_text should take color argument... */
-  ClutterActor *label;
-
-  label = clutter_label_new_with_text (font_name, text);
-  clutter_label_set_color (CLUTTER_LABEL(label), color);
-
-  return label;
+  return g_object_new (CLUTTER_TYPE_LABEL,
+                       "font-name", font_name,
+                       "text", text,
+                       "color", color,
+                       NULL);
 }
 
 /**
@@ -638,7 +783,7 @@ clutter_label_set_text (ClutterLabel *label,
  *
  * Return value: a string containing the font name, in a format
  *   understandable by pango_font_description_from_string().  The
- *   string is owned by #ClutterLabel and should not be modified
+ *   string is owned by @label and should not be modified
  *   or freed.
  */
 G_CONST_RETURN gchar *
@@ -691,17 +836,18 @@ clutter_label_set_font_name (ClutterLabel *label,
   g_free (priv->font_name);
   priv->font_name = g_strdup (font_name);
   
-  if (priv->desc)
-    pango_font_description_free (priv->desc);
+  if (priv->font_desc)
+    pango_font_description_free (priv->font_desc);
 
-  priv->desc = desc;
+  priv->font_desc = desc;
+  priv->wrap_width = -1;
 
   if (label->priv->text && label->priv->text[0] != '\0')
     {
       clutter_label_clear_layout (label);      
 
-      if (CLUTTER_ACTOR_IS_VISIBLE (CLUTTER_ACTOR(label)))
-	clutter_actor_queue_redraw (CLUTTER_ACTOR(label));
+      if (CLUTTER_ACTOR_IS_VISIBLE (label))
+	clutter_actor_queue_redraw (CLUTTER_ACTOR (label));
     }
   
   g_object_notify (G_OBJECT (label), "font-name");
@@ -961,7 +1107,7 @@ clutter_label_get_layout (ClutterLabel *label)
 {
   g_return_val_if_fail (CLUTTER_IS_LABEL (label), NULL);
 
-  clutter_label_ensure_layout (label, -1);
+  clutter_label_ensure_layout (label);
 
   return label->priv->layout;
 }
@@ -1131,7 +1277,7 @@ clutter_label_set_alignment (ClutterLabel   *label,
  *
  * Returns the label's text alignment
  *
- * Return value: The labels #PangoAlignment
+ * Return value: The label's #PangoAlignment
  *
  * Since 0.2
  **/
@@ -1143,3 +1289,54 @@ clutter_label_get_alignment (ClutterLabel *label)
   return label->priv->alignment;
 }
 
+/**
+ * clutter_label_set_justify:
+ * @label: a #ClutterLabel
+ * @justify: whether the text should be justified
+ *
+ * Sets whether the text of the @label actor should be justified
+ * on both margins. This setting is ignored if Clutter is compiled
+ * against Pango &lt; 1.18.
+ *
+ * Since: 0.6
+ */
+void
+clutter_label_set_justify (ClutterLabel *label,
+                           gboolean      justify)
+{
+  ClutterLabelPrivate *priv;
+
+  g_return_if_fail (CLUTTER_IS_LABEL (label));
+
+  priv = label->priv;
+
+  if (priv->justify != justify)
+    {
+      priv->justify = justify;
+
+      clutter_label_clear_layout (label);
+
+      if (CLUTTER_ACTOR_IS_VISIBLE (label))
+	clutter_actor_queue_redraw (CLUTTER_ACTOR (label));
+
+      g_object_notify (G_OBJECT (label), "justify");
+    }
+}
+
+/**
+ * clutter_label_get_justify:
+ * @label: a #ClutterLabel
+ *
+ * Retrieves whether the label should justify the text on both margins.
+ *
+ * Return value: %TRUE if the text should be justified
+ *
+ * Since: 0.6
+ */
+gboolean
+clutter_label_get_justify (ClutterLabel *label)
+{
+  g_return_val_if_fail (CLUTTER_IS_LABEL (label), FALSE);
+
+  return label->priv->justify;
+}
