@@ -26,12 +26,14 @@
 /**
  * SECTION:clutter-stage
  * @short_description: Top level visual element to which actors are placed.
- * 
+ *
  * #ClutterStage is a top level 'window' on which child actors are placed
  * and manipulated.
  */
 
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif
 
 #include "clutter-backend.h"
 #include "clutter-stage.h"
@@ -55,40 +57,38 @@ struct _ClutterStagePrivate
 {
   ClutterColor        color;
   ClutterPerspective  perspective;
+  ClutterFog          fog;
 
   guint is_fullscreen     : 1;
   guint is_offscreen      : 1;
   guint is_cursor_visible : 1;
   guint is_user_resizable : 1;
+  guint use_fog           : 1;
 
   gchar              *title;
+  ClutterActor       *key_focused_actor;
 };
 
 enum
 {
   PROP_0,
-  
+
   PROP_COLOR,
   PROP_FULLSCREEN,
   PROP_OFFSCREEN,
   PROP_CURSOR_VISIBLE,
   PROP_PERSPECTIVE,
   PROP_TITLE,
-  PROP_USER_RESIZE
+  PROP_USER_RESIZE,
+  PROP_USE_FOG
 };
 
 enum
 {
-  EVENT,
-  EVENT_AFTER,
-  BUTTON_PRESS_EVENT,
-  BUTTON_RELEASE_EVENT,
-  SCROLL_EVENT,
-  KEY_PRESS_EVENT,
-  KEY_RELEASE_EVENT,
-  MOTION_EVENT,
-  STAGE_STATE_EVENT,
-  
+  FULLSCREEN,
+  UNFULLSCREEN,
+  ACTIVATE,
+  DEACTIVATE,
   LAST_SIGNAL
 };
 
@@ -99,15 +99,35 @@ clutter_stage_paint (ClutterActor *self)
 {
   ClutterStagePrivate *priv = CLUTTER_STAGE (self)->priv;
 
-  cogl_paint_init (&priv->color); 
+  cogl_paint_init (&priv->color);
+
+  if (priv->use_fog)
+    {
+      cogl_fog_set (&priv->color,
+                    priv->fog.density,
+                    priv->fog.z_near,
+                    priv->fog.z_far);
+    }
 
   CLUTTER_ACTOR_CLASS (clutter_stage_parent_class)->paint (self);
 }
 
 static void
-clutter_stage_set_property (GObject      *object, 
+clutter_stage_pick (ClutterActor       *self,
+		    const ClutterColor *color)
+{
+  /* Paint nothing, cogl_paint_init() effectively paints the stage
+   * silhouette for us - see _clutter_do_pick().
+   * Chain up to the groups paint howerer so our children get picked
+   * - clutter_group_pick
+   */
+  CLUTTER_ACTOR_CLASS (clutter_stage_parent_class)->paint (self);
+}
+
+static void
+clutter_stage_set_property (GObject      *object,
 			    guint         prop_id,
-			    const GValue *value, 
+			    const GValue *value,
 			    GParamSpec   *pspec)
 {
   ClutterStage        *stage;
@@ -118,7 +138,7 @@ clutter_stage_set_property (GObject      *object,
   actor = CLUTTER_ACTOR (stage);
   priv = stage->priv;
 
-  switch (prop_id) 
+  switch (prop_id)
     {
     case PROP_COLOR:
       clutter_stage_set_color (stage, g_value_get_boxed (value));
@@ -129,6 +149,10 @@ clutter_stage_set_property (GObject      *object,
 
       if (CLUTTER_ACTOR_IS_REALIZED (actor))
         {
+          /* Backend needs to check this prop and handle accordingly
+           * in realise. 
+           * FIXME: More 'obvious' implementation needed?
+          */
           clutter_actor_unrealize (actor);
           priv->is_offscreen = g_value_get_boolean (value);
           clutter_actor_realize (actor);
@@ -152,14 +176,17 @@ clutter_stage_set_property (GObject      *object,
         clutter_stage_hide_cursor (stage);
       break;
     case PROP_PERSPECTIVE:
-      clutter_stage_set_perspectivex (stage, g_value_get_boxed (value)); 
+      clutter_stage_set_perspectivex (stage, g_value_get_boxed (value));
       break;
     case PROP_TITLE:
-      clutter_stage_set_title (stage, g_value_get_string (value)); 
+      clutter_stage_set_title (stage, g_value_get_string (value));
       break;
     case PROP_USER_RESIZE:
       clutter_stage_set_user_resizable (stage, g_value_get_boolean (value));
       break;
+    case PROP_USE_FOG:
+      clutter_stage_set_use_fog (stage, g_value_get_boolean (value));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -167,9 +194,9 @@ clutter_stage_set_property (GObject      *object,
 }
 
 static void
-clutter_stage_get_property (GObject    *object, 
+clutter_stage_get_property (GObject    *object,
 			    guint       prop_id,
-			    GValue     *value, 
+			    GValue     *value,
 			    GParamSpec *pspec)
 {
   ClutterStage        *stage;
@@ -180,7 +207,7 @@ clutter_stage_get_property (GObject    *object,
   stage = CLUTTER_STAGE(object);
   priv = stage->priv;
 
-  switch (prop_id) 
+  switch (prop_id)
     {
     case PROP_COLOR:
       clutter_stage_get_color (stage, &color);
@@ -205,10 +232,13 @@ clutter_stage_get_property (GObject    *object,
     case PROP_USER_RESIZE:
       g_value_set_boolean (value, priv->is_user_resizable);
       break;
+    case PROP_USE_FOG:
+      g_value_set_boolean (value, priv->use_fog);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
-    } 
+    }
 }
 
 static void
@@ -221,9 +251,10 @@ clutter_stage_class_init (ClutterStageClass *klass)
   gobject_class->get_property = clutter_stage_get_property;
 
   actor_class->paint = clutter_stage_paint;
+  actor_class->pick = clutter_stage_pick;
 
   /**
-   * ClutterStage:fullscreen
+   * ClutterStage:fullscreen:
    *
    * Whether the stage should be fullscreen or not.
    */
@@ -234,7 +265,11 @@ clutter_stage_class_init (ClutterStageClass *klass)
 			   "Whether the main stage is fullscreen",
 			   FALSE,
 			   G_PARAM_CONSTRUCT | CLUTTER_PARAM_READWRITE));
-
+  /**
+   * ClutterStage:offscreen:
+   *
+   * Whether the stage should be rendered in an offscreen buffer.
+   */
   g_object_class_install_property
     (gobject_class, PROP_OFFSCREEN,
      g_param_spec_boolean ("offscreen",
@@ -242,8 +277,11 @@ clutter_stage_class_init (ClutterStageClass *klass)
 			   "Whether the main stage is renderer offscreen",
 			   FALSE,
 			   G_PARAM_CONSTRUCT | CLUTTER_PARAM_READWRITE));
-
-
+  /**
+   * ClutterStage:cursor-visible:
+   *
+   * Whether the mouse pointer should be visible
+   */
   g_object_class_install_property
     (gobject_class, PROP_CURSOR_VISIBLE,
      g_param_spec_boolean ("cursor-visible",
@@ -251,7 +289,13 @@ clutter_stage_class_init (ClutterStageClass *klass)
 			   "Whether the mouse pointer is visible on the main stage ",
 			   TRUE,
 			   G_PARAM_CONSTRUCT | CLUTTER_PARAM_READWRITE));
-
+  /**
+   * ClutterStage:user-resizable:
+   *
+   * Whether the stage is resizable via user interaction.
+   *
+   * Since: 0.4
+   */
   g_object_class_install_property
     (gobject_class, PROP_USER_RESIZE,
      g_param_spec_boolean ("user-resizable",
@@ -260,7 +304,11 @@ clutter_stage_class_init (ClutterStageClass *klass)
 			   "user interaction",
 			   FALSE,
 			   G_PARAM_CONSTRUCT | CLUTTER_PARAM_READWRITE));
-
+  /**
+   * ClutterStage:color:
+   *
+   * The color of the main stage.
+   */
   g_object_class_install_property
     (gobject_class, PROP_COLOR,
      g_param_spec_boxed ("color",
@@ -268,161 +316,105 @@ clutter_stage_class_init (ClutterStageClass *klass)
 			 "The color of the main stage",
 			 CLUTTER_TYPE_COLOR,
 			 CLUTTER_PARAM_READWRITE));
-
   /**
    * ClutterStage:title:
    *
-   * The stages title - usually displayed in stage windows title decorations.
+   * The stage's title - usually displayed in stage windows title decorations.
    *
    * Since: 0.4
    */
-  g_object_class_install_property 
+  g_object_class_install_property
     (gobject_class, PROP_TITLE,
      g_param_spec_string ("title",
 			  "Title",
 			  "Stage Title",
 			  NULL,
 			  CLUTTER_PARAM_READWRITE));
+  /**
+   * ClutterStage:use-fog:
+   *
+   * Whether the stage should use a linear GL "fog" in creating the
+   * depth-cueing effect, to enhance the perception of depth by fading
+   * actors farther from the viewpoint.
+   *
+   * Since: 0.6
+   */
+  g_object_class_install_property (gobject_class,
+                                   PROP_USE_FOG,
+                                   g_param_spec_boolean ("use-fog",
+                                                         "Use Fog",
+                                                         "Whether to enable depth cueing",
+                                                         FALSE,
+                                                         CLUTTER_PARAM_READWRITE));
 
   /**
-   * ClutterStage::event:
-   * @stage: the actor which received the event
-   * @event: a #ClutterEvent
+   * ClutterStage::fullscreen
+   * @stage: the stage which was fullscreened
    *
-   * The ::event signal is emitted each time and event is received
-   * by the @stage.
+   * The ::fullscreen signal is emitted when the stage is made fullscreen.
+   *
+   * Since: 0.6
    */
-  stage_signals[EVENT] =
-    g_signal_new ("event",
+  stage_signals[FULLSCREEN] =
+    g_signal_new ("fullscreen",
 		  G_TYPE_FROM_CLASS (gobject_class),
 		  G_SIGNAL_RUN_LAST,
-		  G_STRUCT_OFFSET (ClutterStageClass, event),
+		  G_STRUCT_OFFSET (ClutterStageClass, fullscreen),
 		  NULL, NULL,
-		  clutter_marshal_VOID__BOXED,
-		  G_TYPE_NONE, 1,
-		  CLUTTER_TYPE_EVENT | G_SIGNAL_TYPE_STATIC_SCOPE);
+		  clutter_marshal_VOID__VOID,
+		  G_TYPE_NONE, 0);
   /**
-   * ClutterStage::event-after:
-   * @stage: the actor which received the event
-   * @event: a #ClutterEvent
+   * ClutterStage::unfullscreen
+   * @stage: the stage which has left a fullscreen state.
    *
-   * The ::event-after signal is emitted after each event, except for
-   * the "delete-event" is received by @stage.
-   */
-  stage_signals[EVENT_AFTER] =
-    g_signal_new ("event-after",
-                  G_TYPE_FROM_CLASS (gobject_class),
-                  G_SIGNAL_RUN_LAST,
-                  G_STRUCT_OFFSET (ClutterStageClass, event_after),
-                  NULL, NULL,
-                  clutter_marshal_VOID__BOXED,
-                  G_TYPE_NONE, 1,
-                  CLUTTER_TYPE_EVENT | G_SIGNAL_TYPE_STATIC_SCOPE);
-  /**
-   * ClutterStage::button-press-event:
-   * @stage: the actor which received the event
-   * @event: a #ClutterButtonEvent
+   * The ::unfullscreen signal is emitted when the stage leaves a fullscreen
+   * state.
    *
-   * The ::button-press-event signal is emitted each time a mouse button
-   * is pressed on @stage.
+   * Since: 0.6
    */
-  stage_signals[BUTTON_PRESS_EVENT] =
-    g_signal_new ("button-press-event",
+  stage_signals[UNFULLSCREEN] =
+    g_signal_new ("unfullscreen",
 		  G_TYPE_FROM_CLASS (gobject_class),
 		  G_SIGNAL_RUN_LAST,
-		  G_STRUCT_OFFSET (ClutterStageClass, button_press_event),
+		  G_STRUCT_OFFSET (ClutterStageClass, unfullscreen),
 		  NULL, NULL,
-		  clutter_marshal_VOID__BOXED,
-		  G_TYPE_NONE, 1,
-		  CLUTTER_TYPE_EVENT | G_SIGNAL_TYPE_STATIC_SCOPE);
+		  clutter_marshal_VOID__VOID,
+		  G_TYPE_NONE, 0);
   /**
-   * ClutterStage::button-release-event:
-   * @stage: the actor which received the event
-   * @event: a #ClutterButtonEvent
+   * ClutterStage::activate
+   * @stage: the stage which was activated
    *
-   * The ::button-release-event signal is emitted each time a mouse button
-   * is released on @stage.
+   * The ::activate signal is emitted when the stage receives key focus
+   * from the underlying window system.
+   *
+   * Since: 0.6
    */
-  stage_signals[BUTTON_RELEASE_EVENT] =
-    g_signal_new ("button-release-event",
+  stage_signals[ACTIVATE] =
+    g_signal_new ("activate",
 		  G_TYPE_FROM_CLASS (gobject_class),
 		  G_SIGNAL_RUN_LAST,
-		  G_STRUCT_OFFSET (ClutterStageClass, button_release_event),
+		  G_STRUCT_OFFSET (ClutterStageClass, activate),
 		  NULL, NULL,
-		  clutter_marshal_VOID__BOXED,
-		  G_TYPE_NONE, 1,
-		  CLUTTER_TYPE_EVENT | G_SIGNAL_TYPE_STATIC_SCOPE);
+		  clutter_marshal_VOID__VOID,
+		  G_TYPE_NONE, 0);
   /**
-   * ClutterStage::scroll-event:
-   * @stage: the actor which received the event
-   * @event: a #ClutterScrollEvent
+   * ClutterStage::deactivate
+   * @stage: the stage which was deactivated
    *
-   * The ::scroll-event signal is emitted each time a the mouse is
-   * scrolled on @stage
+   * The ::activate signal is emitted when the stage loses key focus
+   * from the underlying window system.
    *
-   * Since: 0.4
+   * Since: 0.6
    */
-  stage_signals[SCROLL_EVENT] =
-    g_signal_new ("scroll-event",
+  stage_signals[DEACTIVATE] =
+    g_signal_new ("deactivate",
 		  G_TYPE_FROM_CLASS (gobject_class),
 		  G_SIGNAL_RUN_LAST,
-		  G_STRUCT_OFFSET (ClutterStageClass, scroll_event),
+		  G_STRUCT_OFFSET (ClutterStageClass, deactivate),
 		  NULL, NULL,
-		  clutter_marshal_VOID__BOXED,
-		  G_TYPE_NONE, 1,
-		  CLUTTER_TYPE_EVENT | G_SIGNAL_TYPE_STATIC_SCOPE);
-  /**
-   * ClutterStage::key-press-event:
-   * @stage: the actor which received the event
-   * @event: a #ClutterKeyEvent
-   *
-   * The ::key-press-event signal is emitted each time a keyboard button
-   * is pressed on @stage.
-   */
-  stage_signals[KEY_PRESS_EVENT] =
-    g_signal_new ("key-press-event",
-		  G_TYPE_FROM_CLASS (gobject_class),
-		  G_SIGNAL_RUN_LAST,
-		  G_STRUCT_OFFSET (ClutterStageClass, key_press_event),
-		  NULL, NULL,
-		  clutter_marshal_VOID__BOXED,
-		  G_TYPE_NONE, 1,
-		  CLUTTER_TYPE_EVENT | G_SIGNAL_TYPE_STATIC_SCOPE);
-  /**
-   * ClutterStage::key-release-event:
-   * @stage: the actor which received the event
-   * @event: a #ClutterKeyEvent
-   *
-   * The ::key-release-event signal is emitted each time a keyboard button
-   * is released on @stage.
-   */
-  stage_signals[KEY_RELEASE_EVENT] =
-    g_signal_new ("key-release-event",
-		  G_TYPE_FROM_CLASS (gobject_class),
-		  G_SIGNAL_RUN_LAST,
-		  G_STRUCT_OFFSET (ClutterStageClass, key_release_event),
-		  NULL, NULL,
-		  clutter_marshal_VOID__BOXED,
-		  G_TYPE_NONE, 1,
-		  CLUTTER_TYPE_EVENT | G_SIGNAL_TYPE_STATIC_SCOPE);
-  /**
-   * ClutterStage::motion-event:
-   * @stage: the actor which received the event
-   * @event: a #ClutterMotionEvent
-   *
-   * The ::motion-event signal is emitted each time the mouse pointer is
-   * moved on @stage.
-   */
-  stage_signals[MOTION_EVENT] =
-    g_signal_new ("motion-event",
-		  G_TYPE_FROM_CLASS (gobject_class),
-		  G_SIGNAL_RUN_LAST,
-		  G_STRUCT_OFFSET (ClutterStageClass, motion_event),
-		  NULL, NULL,
-		  clutter_marshal_VOID__BOXED,
-		  G_TYPE_NONE, 1,
-		  CLUTTER_TYPE_EVENT | G_SIGNAL_TYPE_STATIC_SCOPE);
-  
+		  clutter_marshal_VOID__VOID,
+		  G_TYPE_NONE, 0);
+
   g_type_class_add_private (gobject_class, sizeof (ClutterStagePrivate));
 }
 
@@ -433,13 +425,14 @@ clutter_stage_init (ClutterStage *self)
 
   /* a stage is a top-level object */
   CLUTTER_SET_PRIVATE_FLAGS (self, CLUTTER_ACTOR_IS_TOPLEVEL);
-  
+
   self->priv = priv = CLUTTER_STAGE_GET_PRIVATE (self);
 
   priv->is_offscreen      = FALSE;
   priv->is_fullscreen     = FALSE;
   priv->is_user_resizable = FALSE;
   priv->is_cursor_visible = TRUE;
+  priv->use_fog           = FALSE;
 
   priv->color.red   = 0xff;
   priv->color.green = 0xff;
@@ -451,7 +444,14 @@ clutter_stage_init (ClutterStage *self)
   priv->perspective.z_near = CLUTTER_FLOAT_TO_FIXED (0.1);
   priv->perspective.z_far  = CLUTTER_FLOAT_TO_FIXED (100.0);
 
+  /* depth cueing */
+  priv->fog.density = CLUTTER_FLOAT_TO_FIXED (0.1);
+  priv->fog.z_near  = CLUTTER_FLOAT_TO_FIXED (1.0);
+  priv->fog.z_far   = CLUTTER_FLOAT_TO_FIXED (2.0);
+
   clutter_actor_set_size (CLUTTER_ACTOR (self), 640, 480);
+  clutter_actor_set_reactive (CLUTTER_ACTOR (self), TRUE);
+  clutter_stage_set_key_focus (self, NULL);
 }
 
 /**
@@ -481,7 +481,7 @@ clutter_stage_get_default (void)
  * clutter_stage_set_color
  * @stage: A #ClutterStage
  * @color: A #ClutterColor
- * 
+ *
  * Set the stage color.
  **/
 void
@@ -492,16 +492,16 @@ clutter_stage_set_color (ClutterStage       *stage,
 
   g_return_if_fail (CLUTTER_IS_STAGE (stage));
   g_return_if_fail (color != NULL);
-  
+
   priv = stage->priv;
   priv->color.red = color->red;
   priv->color.green = color->green;
   priv->color.blue = color->blue;
   priv->color.alpha = color->alpha;
 
-  if (CLUTTER_ACTOR_IS_VISIBLE (CLUTTER_ACTOR (stage)))
+  if (CLUTTER_ACTOR_IS_VISIBLE (stage))
     clutter_actor_queue_redraw (CLUTTER_ACTOR (stage));
-  
+
   g_object_notify (G_OBJECT (stage), "color");
 }
 
@@ -509,7 +509,7 @@ clutter_stage_set_color (ClutterStage       *stage,
  * clutter_stage_get_color
  * @stage: A #ClutterStage
  * @color: return location for a #ClutterColor
- * 
+ *
  * Retrieves the stage color.
  */
 void
@@ -517,12 +517,12 @@ clutter_stage_get_color (ClutterStage *stage,
 			 ClutterColor *color)
 {
   ClutterStagePrivate *priv;
-  
+
   g_return_if_fail (CLUTTER_IS_STAGE (stage));
   g_return_if_fail (color != NULL);
 
   priv = stage->priv;
-  
+
   color->red = priv->color.red;
   color->green = priv->color.green;
   color->blue = priv->color.blue;
@@ -533,8 +533,9 @@ clutter_stage_get_color (ClutterStage *stage,
  * clutter_stage_set_perspectivex
  * @stage: A #ClutterStage
  * @perspective: A #ClutterPerspective
- * 
- * Set the stage perspective.
+ *
+ * Set the stage perspective. This is the fixed point version of 
+ * clutter_stage_set_perspective().
  **/
 void
 clutter_stage_set_perspectivex (ClutterStage       *stage,
@@ -543,46 +544,44 @@ clutter_stage_set_perspectivex (ClutterStage       *stage,
   ClutterStagePrivate *priv;
 
   g_return_if_fail (CLUTTER_IS_STAGE (stage));
+  g_return_if_fail (perspective != NULL);
 
   priv = stage->priv;
-  priv->perspective.fovy   = perspective->fovy;
-  priv->perspective.aspect = perspective->aspect;
-  priv->perspective.z_near = perspective->z_near;
-  priv->perspective.z_far  = perspective->z_far;
 
-  CLUTTER_SET_PRIVATE_FLAGS(stage, CLUTTER_ACTOR_SYNC_MATRICES);
+  priv->perspective = *perspective;
+
+  CLUTTER_SET_PRIVATE_FLAGS (stage, CLUTTER_ACTOR_SYNC_MATRICES);
 }
 
 /**
  * clutter_stage_get_perspectivex
  * @stage: A #ClutterStage
  * @perspective: return location for a #ClutterPerspective
- * 
- * Retrieves the stage perspective.
+ *
+ * Retrieves the stage perspective. This is the fixed point version of 
+ * clutter_stage_get_perspective().
  */
 void
 clutter_stage_get_perspectivex (ClutterStage       *stage,
 				ClutterPerspective *perspective)
 {
-  ClutterStagePrivate *priv;
-
   g_return_if_fail (CLUTTER_IS_STAGE (stage));
+  g_return_if_fail (perspective != NULL);
 
-  priv = stage->priv;
-  perspective->fovy   = priv->perspective.fovy;
-  perspective->aspect = priv->perspective.aspect;
-  perspective->z_near = priv->perspective.z_near;
-  perspective->z_far  = priv->perspective.z_far;
+  *perspective = stage->priv->perspective;
 }
 
 /**
  * clutter_stage_set_perspective
  * @stage: A #ClutterStage
- * @fovy: FIXME
- * @aspect: FIXME
- * @z_near: FIXME
- * @z_far: FIXME
- * 
+ * @fovy: the field of view angle, in degrees, in the y direction
+ * @aspect: the aspect ratio that determines the field of view in the x
+ *   direction. The aspect ratio is the ratio of x (width) to y (height)
+ * @z_near: the distance from the viewer to the near clipping
+ *   plane (always positive)
+ * @z_far: the  distance from the viewer to the far clipping
+ *   plane (always positive)
+ *
  * Set the stage perspective.
  *
  * Since: 0.4
@@ -599,23 +598,28 @@ clutter_stage_set_perspective (ClutterStage *stage,
   g_return_if_fail (CLUTTER_IS_STAGE (stage));
 
   priv = stage->priv;
+
   priv->perspective.fovy   = CLUTTER_FLOAT_TO_FIXED(fovy);
   priv->perspective.aspect = CLUTTER_FLOAT_TO_FIXED(aspect);
   priv->perspective.z_near = CLUTTER_FLOAT_TO_FIXED(z_near);
   priv->perspective.z_far  = CLUTTER_FLOAT_TO_FIXED(z_far);
 
-  CLUTTER_SET_PRIVATE_FLAGS(stage, CLUTTER_ACTOR_SYNC_MATRICES);
+  CLUTTER_SET_PRIVATE_FLAGS (stage, CLUTTER_ACTOR_SYNC_MATRICES);
 }
 
 /**
  * clutter_stage_get_perspective
  * @stage: A #ClutterStage
- * @fovy: FIXME
- * @aspect: FIXME
- * @z_near: FIXME
- * @z_far: FIXME
- * 
+ * @fovy: return location for the field of view, in degrees, or %NULL
+ * @aspect: return location for the aspect ratio, or %NULL
+ * @z_near: return location for the distance of the viewer from the
+ *   near clipping plane, or %NULL
+ * @z_far: return location for the distance of the viewer from the
+ *   far clipping plane, or %NULL
+ *
  * Retrieves the stage perspective.
+ *
+ * Since: 0.4
  */
 void
 clutter_stage_get_perspective (ClutterStage       *stage,
@@ -662,12 +666,12 @@ clutter_stage_fullscreen (ClutterStage *stage)
   priv = stage->priv;
   if (!priv->is_fullscreen)
     {
-      priv->is_fullscreen = TRUE;
-
+      /* Only set if backend implements.
+       * Also see clutter_stage_event() for setting priv->is_fullscreen
+       * on state change event.
+      */
       if (CLUTTER_STAGE_GET_CLASS (stage)->set_fullscreen)
-        CLUTTER_STAGE_GET_CLASS (stage)->set_fullscreen (stage, TRUE);
-
-      g_object_notify (G_OBJECT (stage), "fullscreen");
+	CLUTTER_STAGE_GET_CLASS (stage)->set_fullscreen (stage, TRUE);
     }
 }
 
@@ -691,21 +695,18 @@ clutter_stage_unfullscreen (ClutterStage *stage)
   priv = stage->priv;
   if (priv->is_fullscreen)
     {
-      priv->is_fullscreen = FALSE;
-
+      /* Only set if backend implements */
       if (CLUTTER_STAGE_GET_CLASS (stage)->set_fullscreen)
-        CLUTTER_STAGE_GET_CLASS (stage)->set_fullscreen (stage, FALSE);
-
-      g_object_notify (G_OBJECT (stage), "fullscreen");
+	CLUTTER_STAGE_GET_CLASS (stage)->set_fullscreen (stage, FALSE);
     }
 }
 
 /**
  * clutter_stage_set_user_resizable:
  * @stage: a #ClutterStage
- * @resizable: whether if the stage should be user resizable.
+ * @resizable: whether the stage should be user resizable.
  *
- * Sets if the stage is able to be resized by user interaction (i.e via
+ * Sets if the stage is resizable by user interaction (e.g. via
  * window manager controls)
  *
  * Since: 0.4
@@ -738,7 +739,7 @@ clutter_stage_set_user_resizable (ClutterStage *stage,
  *
  * Retrieves the value set with clutter_stage_set_user_resizable().
  *
- * Return value: %TRUE is the stage is resizable by the user.
+ * Return value: %TRUE if the stage is resizable by the user.
  *
  * Since: 0.4
  */
@@ -851,44 +852,7 @@ clutter_stage_get_actor_at_pos (ClutterStage *stage,
                                 gint          x,
                                 gint          y)
 {
-  ClutterMainContext *context;
-  guchar              pixel[4];
-  GLint               viewport[4];
-  ClutterColor        white = { 0xff, 0xff, 0xff, 0xff };
-  guint32             id;
-  gint                r,g,b;
-
-  context = clutter_context_get_default ();
-
-  cogl_paint_init (&white);
-  cogl_enable (0);
-
-  /* Render the entire scence in pick mode - just single colored silhouette's  
-   * are drawn offscreen (as we never swap buffers)
-  */
-  context->pick_mode = TRUE;
-  clutter_actor_paint (CLUTTER_ACTOR (stage));
-  context->pick_mode = FALSE;
-
-  /* Calls should work under both GL and GLES, note GLES needs RGBA */
-  glGetIntegerv(GL_VIEWPORT, viewport);
-
-  /* Below to be safe, particularly on GL ES. an EGL wait call or full 
-   * could be nicer.  
-  */
-  glFinish(); 		
-  
-  glReadPixels(x, viewport[3] - y, 1, 1, GL_RGBA, GL_UNSIGNED_BYTE, pixel);
-
-  if (pixel[0] == 0xff && pixel[1] == 0xff && pixel[2] == 0xff)
-    return CLUTTER_ACTOR(stage);
-
-  cogl_get_bitmasks (&r, &g, &b, NULL);
-
-  /* Decode color back into an ID, taking into account fb depth */
-  id = pixel[2]>>(8-b) | pixel[1]<<b>>(8-g) | pixel[0]<<(g+b)>>(8-r);
-
-  return clutter_group_find_child_by_id (CLUTTER_GROUP (stage), id);
+  return _clutter_do_pick (stage, x, y, CLUTTER_PICK_ALL);
 }
 
 /**
@@ -908,76 +872,68 @@ gboolean
 clutter_stage_event (ClutterStage *stage,
                      ClutterEvent *event)
 {
-  gboolean res = TRUE;
-  gint signal_num = -1;
+  ClutterStagePrivate *priv;
 
   g_return_val_if_fail (CLUTTER_IS_STAGE (stage), FALSE);
   g_return_val_if_fail (event != NULL, FALSE);
 
-  g_object_ref (stage);
+  priv = stage->priv;
 
-  g_signal_emit (stage, stage_signals[EVENT], 0, event);
-  
-  switch (event->type)
+  if (event->type == CLUTTER_DELETE)
     {
-    case CLUTTER_NOTHING:
-      break;
-    case CLUTTER_BUTTON_PRESS:
-    case CLUTTER_2BUTTON_PRESS:
-    case CLUTTER_3BUTTON_PRESS:
-      signal_num = BUTTON_PRESS_EVENT;
-      break;
-    case CLUTTER_BUTTON_RELEASE:
-      signal_num = BUTTON_RELEASE_EVENT;
-      break;
-    case CLUTTER_SCROLL:
-      signal_num = SCROLL_EVENT;
-      break;
-    case CLUTTER_KEY_PRESS:
-      signal_num = KEY_PRESS_EVENT;
-      break;
-    case CLUTTER_KEY_RELEASE:
-      signal_num = KEY_RELEASE_EVENT;
-      break;
-    case CLUTTER_MOTION:
-      signal_num = MOTION_EVENT;
-      break;
-    case CLUTTER_DELETE:
-      signal_num = -1;
-      break;
-    case CLUTTER_STAGE_STATE:
-      signal_num = -1;
-      break;
-    case CLUTTER_DESTROY_NOTIFY:
-      signal_num = -1;
-      break;
-    case CLUTTER_CLIENT_MESSAGE:
-      signal_num = -1;
-      break;
+      gboolean retval = FALSE;
+
+      g_signal_emit_by_name (stage, "event", event, &retval);
+
+      return retval;
     }
 
-  if (signal_num != -1)
+  if (event->type != CLUTTER_STAGE_STATE)
+    return FALSE;
+
+  /* emit raw event */
+  if (clutter_actor_event (CLUTTER_ACTOR (stage), event, FALSE))
+    return TRUE;
+
+  if (event->stage_state.changed_mask & CLUTTER_STAGE_STATE_FULLSCREEN)
     {
-      g_signal_emit (stage, stage_signals[signal_num], 0, event);
-      g_signal_emit (stage, stage_signals[EVENT_AFTER], 0, event);
-      res = TRUE;
+      if (event->stage_state.new_state & CLUTTER_STAGE_STATE_FULLSCREEN)
+	{
+	  priv->is_fullscreen = TRUE;
+	  g_signal_emit (stage, stage_signals[FULLSCREEN], 0);
+
+          g_object_notify (G_OBJECT (stage), "fullscreen");
+	}
+      else
+	{
+	  priv->is_fullscreen = FALSE;
+	  g_signal_emit (stage, stage_signals[UNFULLSCREEN], 0);
+
+          g_object_notify (G_OBJECT (stage), "fullscreen");
+	}
     }
 
-  g_object_unref (stage);
+  if (event->stage_state.changed_mask & CLUTTER_STAGE_STATE_ACTIVATED)
+    {
+      if (event->stage_state.new_state & CLUTTER_STAGE_STATE_ACTIVATED)
+	g_signal_emit (stage, stage_signals[ACTIVATE], 0);
+      else
+	g_signal_emit (stage, stage_signals[DEACTIVATE], 0);
+    }
 
-  return res;
+  return TRUE;
 }
 
 /**
  * clutter_stage_set_title
  * @stage: A #ClutterStage
  * @title: A utf8 string for the stage windows title.
- * 
+ *
  * Sets the stage title.
  *
  * Since 0.4
  **/
-void          
+void
 clutter_stage_set_title (ClutterStage       *stage,
 			 const gchar        *title)
 {
@@ -999,7 +955,7 @@ clutter_stage_set_title (ClutterStage       *stage,
 /**
  * clutter_stage_get_title
  * @stage: A #ClutterStage
- * 
+ *
  * Gets the stage title.
  *
  * Return value: pointer to the title string for the stage. The
@@ -1016,24 +972,322 @@ clutter_stage_get_title (ClutterStage       *stage)
   return stage->priv->title;
 }
 
-/*** Perspective boxed type ******/
+static void
+on_key_focused_weak_notify (gpointer data,
+			    GObject *where_the_object_was)
+{
+  ClutterStagePrivate *priv;
+  ClutterStage        *stage = CLUTTER_STAGE (data);
+
+  g_return_if_fail (CLUTTER_IS_STAGE (stage));
+
+  priv = stage->priv;
+  priv->key_focused_actor = NULL;
+
+  /* focused actor has dissapeared - fall back to stage
+   * FIXME: need some kind of signal dance/block here.
+  */
+  clutter_stage_set_key_focus (stage, NULL);
+}
 
 /**
- * clutter_perspective_copy:
- * @perspective: a #ClutterPerspective
+ * clutter_stage_set_key_focus:
+ * @stage: the #ClutterStage
+ * @actor: the actor to set key focus to, or %NULL
  *
- * Makes a copy of the perspective structure.  The result must be
- * freed using clutter_perspective_free().
+ * Sets the key focus on @actor. An actor with key focus will receive
+ * all the key events. If @actor is %NULL, the stage will receive
+ * focus.
  *
- * Return value: an allocated copy of @perspective.
- *
- * Since: 0.4
+ * Since: 0.6
  */
-ClutterPerspective *
+void
+clutter_stage_set_key_focus (ClutterStage *stage,
+			     ClutterActor *actor)
+{
+  ClutterStagePrivate *priv;
+
+  g_return_if_fail (CLUTTER_IS_STAGE (stage));
+  g_return_if_fail (actor == NULL || CLUTTER_IS_ACTOR (actor));
+
+  priv = stage->priv;
+
+  if (priv->key_focused_actor == actor)
+    return;
+
+  if (priv->key_focused_actor)
+    {
+      g_object_weak_unref (G_OBJECT (priv->key_focused_actor),
+			   on_key_focused_weak_notify,
+			   stage);
+      g_signal_emit_by_name (priv->key_focused_actor, "focus-out");
+
+      priv->key_focused_actor = NULL;
+    }
+  else
+    g_signal_emit_by_name (stage, "focus-out");
+
+  if (actor)
+    {
+      priv->key_focused_actor = actor;
+
+      g_object_weak_ref (G_OBJECT (actor),
+			 on_key_focused_weak_notify,
+			 stage);
+      g_signal_emit_by_name (priv->key_focused_actor, "focus-in");
+    }
+  else
+    g_signal_emit_by_name (stage, "focus-in");
+}
+
+/**
+ * clutter_stage_get_key_focus:
+ * @stage: the #ClutterStage
+ *
+ * Retrieves the actor that is currently under key focus.
+ *
+ * Return value: the actor with key focus, or the stage
+ *
+ * Since: 0.6
+ */
+ClutterActor *
+clutter_stage_get_key_focus (ClutterStage *stage)
+{
+  g_return_val_if_fail (CLUTTER_IS_STAGE (stage), NULL);
+
+  if (stage->priv->key_focused_actor)
+    return stage->priv->key_focused_actor;
+
+  return CLUTTER_ACTOR (stage);
+}
+
+/**
+ * clutter_stage_get_use_fog:
+ * @stage: the #ClutterStage
+ *
+ * Gets whether the depth cueing effect is enabled on @stage.
+ *
+ * Return value: %TRUE if the the depth cueing effect is enabled
+ *
+ * Since: 0.6
+ */
+gboolean
+clutter_stage_get_use_fog (ClutterStage *stage)
+{
+  g_return_val_if_fail (CLUTTER_IS_STAGE (stage), FALSE);
+
+  return stage->priv->use_fog;
+}
+
+/**
+ * clutter_stage_set_use_fog:
+ * @stage: the #ClutterStage
+ * @fog: %TRUE for enabling the depth cueing effect
+ *
+ * Sets whether the depth cueing effect on the stage should be enabled
+ * or not.
+ *
+ * Depth cueing is a 3D effect that makes actors farther away from the
+ * viewing point less opaque, by fading them with the stage color.
+
+ * The parameters of the GL fog used can be changed using the
+ * clutter_stage_set_fog() function.
+ *
+ * Since: 0.6
+ */
+void
+clutter_stage_set_use_fog (ClutterStage *stage,
+                           gboolean      fog)
+{
+  ClutterStagePrivate *priv;
+
+  g_return_if_fail (CLUTTER_IS_STAGE (stage));
+
+  priv = stage->priv;
+
+  if (priv->use_fog != fog)
+    {
+      priv->use_fog = fog;
+
+      CLUTTER_NOTE (MISC, "%s depth-cueing inside stage",
+                    priv->use_fog ? "enabling" : "disabling");
+
+      if (CLUTTER_ACTOR_IS_VISIBLE (stage))
+        clutter_actor_queue_redraw (CLUTTER_ACTOR (stage));
+
+      g_object_notify (G_OBJECT (stage), "use-fog");
+    }
+}
+
+/**
+ * clutter_stage_get_fog:
+ * @stage: a #ClutterStage
+ * @density: return location for the intensity dampening
+ * @z_near: return location for the starting point of the depth cueing
+ * @z_far: return location for the ending point of the depth cueing
+ *
+ * Retrieves the settings used by the GL fog to create the
+ * depth cueing effect on the @stage.
+ *
+ * Since: 0.6
+ */
+void
+clutter_stage_get_fog (ClutterStage *stage,
+                       gdouble      *density,
+                       gdouble      *z_near,
+                       gdouble      *z_far)
+{
+  ClutterStagePrivate *priv;
+
+  g_return_if_fail (CLUTTER_IS_STAGE (stage));
+
+  priv = stage->priv;
+
+  if (density)
+    *density = CLUTTER_FIXED_TO_FLOAT (priv->fog.density);
+  if (z_near)
+    *z_near = CLUTTER_FIXED_TO_FLOAT (priv->fog.z_near);
+  if (z_far)
+    *z_far = CLUTTER_FIXED_TO_FLOAT (priv->fog.z_far);
+}
+
+/**
+ * clutter_stage_set_fog:
+ * @stage: the #ClutterStage
+ * @density: density of the intensity dampening
+ * @z_near: starting point of the depth cueing
+ * @z_far: ending point of the depth cueing
+ *
+ * Sets the GL fog settings used to create the depth cueing effect
+ * on the @stage.
+ *
+ * If the actors are all near the view point you will need a higher @density
+ * and a smaller interval between @z_near and @z_far. On the other hand, if
+ * actors are placed far away from the view point you will need a lower
+ * @density but a bigger interval between @z_near and @z_far.
+ *
+ * Since: 0.6
+ */
+void
+clutter_stage_set_fog (ClutterStage *stage,
+                       gdouble       density,
+                       gdouble       z_near,
+                       gdouble       z_far)
+{
+  ClutterStagePrivate *priv;
+
+  g_return_if_fail (CLUTTER_IS_STAGE (stage));
+
+  priv = stage->priv;
+
+  priv->fog.density = CLUTTER_FLOAT_TO_FIXED (density);
+  priv->fog.z_near  = CLUTTER_FLOAT_TO_FIXED (z_near);
+  priv->fog.z_far   = CLUTTER_FLOAT_TO_FIXED (z_far);
+
+  if (priv->use_fog && CLUTTER_ACTOR_IS_VISIBLE (stage))
+    clutter_actor_queue_redraw (CLUTTER_ACTOR (stage));
+}
+
+/**
+ * clutter_stage_set_fogx:
+ * @stage: the #ClutterStage
+ * @fog: a #ClutterFog structure
+ *
+ * Sets the depth cueing settings for the @stage. This is the fixed point
+ * version of clutter_stage_set_fog().
+ *
+ * Since: 0.6
+ */
+void
+clutter_stage_set_fogx (ClutterStage *stage,
+                        ClutterFog   *fog)
+{
+  ClutterStagePrivate *priv;
+
+  g_return_if_fail (CLUTTER_IS_STAGE (stage));
+  g_return_if_fail (fog != NULL);
+
+  priv = stage->priv;
+
+  priv->fog = *fog;
+
+  if (priv->use_fog && CLUTTER_ACTOR_IS_VISIBLE (stage))
+    clutter_actor_queue_redraw (CLUTTER_ACTOR (stage));
+}
+
+/**
+ * clutter_stage_get_fogx:
+ * @stage: the #ClutterStage
+ * @fog: return location for a #ClutterFog structure
+ *
+ * Retrieves the current depth cueing settings from the stage. This is the
+ * fixed point version of clutter_stage_get_fog().
+ *
+ * Since: 0.6
+ */
+void
+clutter_stage_get_fogx (ClutterStage *stage,
+                        ClutterFog   *fog)
+{
+  g_return_if_fail (CLUTTER_IS_STAGE (stage));
+  g_return_if_fail (fog != NULL);
+
+  *fog = stage->priv->fog;
+}
+
+/**
+ * clutter_stage_get_resolution:
+ * @stage: the #ClutterStage
+ *
+ * Retrieves the resolution (in DPI) of the stage from the default
+ * backend.
+ *
+ * Return value: the resolution of the stage
+ *
+ * Since: 0.6
+ */
+gdouble
+clutter_stage_get_resolution (ClutterStage *stage)
+{
+  ClutterMainContext *context;
+
+  context = clutter_context_get_default ();
+  g_assert (context != NULL);
+
+  return clutter_backend_get_resolution (context->backend);
+}
+
+/**
+ * clutter_stage_get_resolutionx:
+ * @stage: the #ClutterStage
+ *
+ * Fixed point version of clutter_stage_get_resolution().
+ *
+ * Return value: the resolution of the stage
+ *
+ * Since: 0.6
+ */
+ClutterFixed
+clutter_stage_get_resolutionx (ClutterStage *stage)
+{
+  ClutterFixed res;
+  ClutterMainContext *context;
+
+  context = clutter_context_get_default ();
+  g_assert (context != NULL);
+
+  res = clutter_backend_get_resolution (context->backend);
+
+  return CLUTTER_FLOAT_TO_FIXED (res);
+}
+
+/*** Perspective boxed type ******/
+
+static ClutterPerspective *
 clutter_perspective_copy (const ClutterPerspective *perspective)
 {
   ClutterPerspective *result;
-  
+
   g_return_val_if_fail (perspective != NULL, NULL);
 
   result = g_slice_new (ClutterPerspective);
@@ -1042,32 +1296,56 @@ clutter_perspective_copy (const ClutterPerspective *perspective)
   return result;
 }
 
-/**
- * clutter_perspective_free:
- * @perspective: a #ClutterPerspective
- *
- * Frees a perspective structure created with clutter_perspective_copy().
- *
- * Since: 0.4
- */
-void
+static void
 clutter_perspective_free (ClutterPerspective *perspective)
 {
-  g_return_if_fail (perspective != NULL);
-
-  g_slice_free (ClutterPerspective, perspective);
+  if (G_LIKELY (perspective))
+    g_slice_free (ClutterPerspective, perspective);
 }
 
 GType
 clutter_perspective_get_type (void)
 {
   static GType our_type = 0;
-  
+
   if (!our_type)
-    our_type = g_boxed_type_register_static 
-                       ("ClutterPerspective",
-			(GBoxedCopyFunc) clutter_perspective_copy,
-			(GBoxedFreeFunc) clutter_perspective_free);
+    our_type =
+      g_boxed_type_register_static (I_("ClutterPerspective"),
+                                    (GBoxedCopyFunc) clutter_perspective_copy,
+                                    (GBoxedFreeFunc) clutter_perspective_free);
   return our_type;
 }
 
+static ClutterFog *
+clutter_fog_copy (const ClutterFog *fog)
+{
+  ClutterFog *copy;
+
+  g_return_val_if_fail (fog != NULL, NULL);
+
+  copy = g_slice_new0 (ClutterFog);
+  *copy = *fog;
+
+  return copy;
+}
+
+static void
+clutter_fog_free (ClutterFog *fog)
+{
+  if (G_LIKELY (fog))
+    g_slice_free (ClutterFog, fog);
+}
+
+GType
+clutter_fog_get_type (void)
+{
+  static GType our_type = 0;
+
+  if (G_UNLIKELY (our_type == 0))
+    our_type =
+      g_boxed_type_register_static (I_("ClutterFog"),
+                                    (GBoxedCopyFunc) clutter_fog_copy,
+                                    (GBoxedFreeFunc) clutter_fog_free);
+
+  return our_type;
+}
