@@ -33,8 +33,9 @@
 
 G_BEGIN_DECLS
 
-#define CLUTTER_TYPE_PERSPECTIVE  (clutter_perspective_get_type ())
-#define CLUTTER_TYPE_STAGE (clutter_stage_get_type())
+#define CLUTTER_TYPE_PERSPECTIVE        (clutter_perspective_get_type ())
+#define CLUTTER_TYPE_FOG                (clutter_fog_get_type ())
+#define CLUTTER_TYPE_STAGE              (clutter_stage_get_type())
 
 #define CLUTTER_STAGE(obj) \
   (G_TYPE_CHECK_INSTANCE_CAST ((obj), \
@@ -56,13 +57,29 @@ G_BEGIN_DECLS
   (G_TYPE_INSTANCE_GET_CLASS ((obj), \
   CLUTTER_TYPE_STAGE, ClutterStageClass))
 
+/**
+ * CLUTTER_STAGE_WIDTH:
+ *
+ * Macro that evaluates to the current stage width
+ *
+ * Since: 0.2
+ */
 #define CLUTTER_STAGE_WIDTH() \
- clutter_actor_get_width (clutter_stage_get_default ())
+ (clutter_actor_get_width (clutter_stage_get_default ()))
 
+/**
+ * CLUTTER_STAGE_HEIGHT:
+ *
+ * Macro that evaluates to the current stage height
+ *
+ * Since: 0.2
+ */
 #define CLUTTER_STAGE_HEIGHT() \
- clutter_actor_get_height (clutter_stage_get_default ())
+ (clutter_actor_get_height (clutter_stage_get_default ()))
 
 typedef struct _ClutterPerspective  ClutterPerspective;
+typedef struct _ClutterFog          ClutterFog;
+
 typedef struct _ClutterStage        ClutterStage;
 typedef struct _ClutterStageClass   ClutterStageClass;
 typedef struct _ClutterStagePrivate ClutterStagePrivate;
@@ -71,11 +88,11 @@ struct _ClutterStage
 {
   /*< private >*/
   ClutterGroup parent_instance;
-  
-  ClutterStagePrivate *priv;
-}; 
 
-struct _ClutterStageClass 
+  ClutterStagePrivate *priv;
+};
+
+struct _ClutterStageClass
 {
   /*< private >*/
   ClutterGroupClass parent_class;
@@ -86,8 +103,6 @@ struct _ClutterStageClass
                                         gboolean      fullscreen);
   void          (* set_cursor_visible) (ClutterStage *stage,
                                         gboolean      visible);
-  void          (* set_offscreen)      (ClutterStage *stage,
-                                        gboolean      offscreen);
   GdkPixbuf*    (* draw_to_pixbuf)     (ClutterStage *stage,
                                         gint          x,
                                         gint          y,
@@ -98,25 +113,11 @@ struct _ClutterStageClass
   void          (* set_user_resize)    (ClutterStage *stage,
                                         gboolean      value);
 
-  /* signals */
-  void     (* event)                (ClutterStage           *stage,
-                                     ClutterEvent           *event);
-  void     (* event_after)          (ClutterStage           *stage,
-                                     ClutterEvent           *event);
-  void     (* button_press_event)   (ClutterStage           *stage,
-			             ClutterButtonEvent     *event);
-  void     (* button_release_event) (ClutterStage           *stage,
-		  		     ClutterButtonEvent     *event);
-  void     (* scroll_event)         (ClutterStage           *stage,
-                                     ClutterScrollEvent     *event);
-  void     (* key_press_event)      (ClutterStage           *stage,
-		  		     ClutterKeyEvent        *event);
-  void     (* key_release_event)    (ClutterStage           *stage,
-		  		     ClutterKeyEvent        *event);
-  void     (* motion_event)         (ClutterStage           *stage,
-                                     ClutterMotionEvent     *event);
-  void     (* stage_state_event)    (ClutterStage           *stage,
-                                     ClutterStageStateEvent *event);
+  /* events */
+  void     (* fullscreen)      (ClutterStage           *stage);
+  void     (* unfullscreen)    (ClutterStage           *stage);
+  void     (* activate)        (ClutterStage           *stage);
+  void     (* deactivate)      (ClutterStage           *stage);
 
   /*< private >*/
   /* padding for future expansion */
@@ -127,12 +128,16 @@ struct _ClutterStageClass
 
 /**
  * ClutterPerspective:
- * @fovy: FIXME
- * @aspect: FIXME
- * @z_near: FIXME
- * @z_far: FIXME
+ * @fovy: the field of view angle, in degrees, in the y direction
+ * @aspect: the aspect ratio that determines the field of view in the x
+ *   direction. The aspect ratio is the ratio of x (width) to y (height)
+ * @z_near: the distance from the viewer to the near clipping
+ *   plane (always positive)
+ * @z_far: the distance from the viewer to the far clipping
+ *   plane (always positive)
  *
- * Stage perspective definition
+ * Stage perspective definition. #ClutterPerspective is only used by
+ * the fixed point version of clutter_stage_set_perspective().
  *
  * Since: 0.4
  */
@@ -144,10 +149,28 @@ struct _ClutterPerspective
   ClutterFixed z_far;
 };
 
-GType               clutter_perspective_get_type (void) G_GNUC_CONST;
-ClutterPerspective *clutter_perspective_copy     (const ClutterPerspective *perspective);
-void                clutter_perspective_free     (ClutterPerspective       *perspective);
+/**
+ * ClutterFog:
+ * @density: density of the fog
+ * @z_near: starting distance from the viewer to the near clipping
+ *   plane (always positive)
+ * @z_far: final distance from the viewer to the far clipping
+ *   plane (always positive)
+ *
+ * Fog settings used to create the depth cueing effect. #ClutterFog is
+ * useful only when using the fixed point API.
+ *
+ * Since: 0.6
+ */
+struct _ClutterFog
+{
+  ClutterFixed density;
+  ClutterFixed z_near;
+  ClutterFixed z_far;
+};
 
+GType         clutter_perspective_get_type    (void) G_GNUC_CONST;
+GType         clutter_fog_get_type            (void) G_GNUC_CONST;
 GType         clutter_stage_get_type          (void) G_GNUC_CONST;
 
 ClutterActor *clutter_stage_get_default       (void);
@@ -188,9 +211,40 @@ gboolean      clutter_stage_event             (ClutterStage       *stage,
 void                  clutter_stage_set_title          (ClutterStage *stage,
                                                         const gchar  *title);
 G_CONST_RETURN gchar *clutter_stage_get_title          (ClutterStage *stage);
-void                  clutter_stage_set_user_resizable (ClutterStage *stage, 
+void                  clutter_stage_set_user_resizable (ClutterStage *stage,
 						        gboolean      resizable);
 gboolean              clutter_stage_get_user_resizable (ClutterStage *stage);
+void                  clutter_stage_set_use_fog        (ClutterStage *stage,
+                                                        gboolean      fog);
+gboolean              clutter_stage_get_use_fog        (ClutterStage *stage);
+void                  clutter_stage_set_fog            (ClutterStage *stage,
+                                                        gdouble       density,
+                                                        gdouble       z_near,
+                                                        gdouble       z_far);
+void                  clutter_stage_get_fog            (ClutterStage *stage,
+                                                        gdouble      *density,
+                                                        gdouble      *z_near,
+                                                        gdouble      *z_far);
+void                  clutter_stage_set_fogx           (ClutterStage *stage,
+                                                        ClutterFog   *fog);
+void                  clutter_stage_get_fogx           (ClutterStage *stage,
+                                                        ClutterFog   *fog);
+gdouble               clutter_stage_get_resolution     (ClutterStage *stage);
+ClutterFixed          clutter_stage_get_resolutionx    (ClutterStage *stage);
+
+/* New experiental calls */
+void                  clutter_stage_set_key_focus      (ClutterStage *stage,
+                                                        ClutterActor *actor);
+ClutterActor *        clutter_stage_get_key_focus      (ClutterStage *stage);
+
+/* Commodity macro */
+#define clutter_stage_add(stage,actor)                  G_STMT_START {  \
+  if (CLUTTER_IS_STAGE ((stage)) && CLUTTER_IS_ACTOR ((actor)))         \
+    {                                                                   \
+      ClutterContainer *_container = (ClutterContainer *) (stage);      \
+      ClutterActor *_actor = (ClutterActor *) (actor);                  \
+      clutter_container_add_actor (_container, _actor);                 \
+    }                                                   } G_STMT_END
 
 G_END_DECLS
 
