@@ -30,16 +30,17 @@
  * #ClutterTimeline is a base class for managing time based events such
  * as animations.
  *
- * Note: Every timeline shares the same #ClutterTimeoutPool to decrease
- * the possibility of starvating the main loop when using many timelines
+ * Every timeline shares the same #ClutterTimeoutPool to decrease the
+ * possibility of starvating the main loop when using many timelines
  * at the same time; this might cause problems if you are also using
  * a library making heavy use of threads with no GLib main loop integration.
  * In that case you might disable the common timeline pool by setting
  * the %CLUTTER_TIMELINE=no-pool environment variable prior to launching
  * your application.
+ *
  */
 
-#ifndef HAVE_CONFIG_H
+#ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
@@ -49,24 +50,32 @@
 #include "clutter-marshal.h"
 #include "clutter-private.h"
 #include "clutter-debug.h"
+#include "clutter-enum-types.h"
 
 G_DEFINE_TYPE (ClutterTimeline, clutter_timeline, G_TYPE_OBJECT);
 
-#define FPS_TO_INTERVAL(f)              (1000 / (f))
+#define FPS_TO_INTERVAL(f) (1000 / (f))
 #define CLUTTER_TIMELINE_PRIORITY       (G_PRIORITY_DEFAULT + 30)
 
 struct _ClutterTimelinePrivate
 {
+  ClutterTimelineDirection direction;
+
   guint timeout_id;
   guint delay_id;
-  
+
+  gint current_frame_num;
+
   guint fps;
   guint n_frames;
-  guint current_frame_num;
   guint delay;
+  guint duration;
+
+  gint skipped_frames;
 
   gulong last_frame_msecs;
   gulong start_frame_secs;
+  guint  msecs_delta;
 
   guint loop : 1;
 };
@@ -74,11 +83,13 @@ struct _ClutterTimelinePrivate
 enum
 {
   PROP_0,
-  
+
   PROP_FPS,
   PROP_NUM_FRAMES,
   PROP_LOOP,
-  PROP_DELAY
+  PROP_DELAY,
+  PROP_DURATION,
+  PROP_DIRECTION
 };
 
 enum
@@ -124,7 +135,7 @@ timeout_add (guint          interval,
 {
   guint res;
 
-  if (timeline_use_pool)
+  if (G_LIKELY (timeline_use_pool))
     {
       g_assert (timeline_pool != NULL);
       res = clutter_timeout_pool_add (timeline_pool,
@@ -144,7 +155,7 @@ timeout_add (guint          interval,
 static void
 timeout_remove (guint tag)
 {
-  if (timeline_use_pool)
+  if (G_LIKELY (timeline_use_pool))
     {
       g_assert (timeline_pool != NULL);
       clutter_timeout_pool_remove (timeline_pool, tag);
@@ -155,10 +166,10 @@ timeout_remove (guint tag)
 
 /* Object */
 
-static void 
-clutter_timeline_set_property (GObject      *object, 
+static void
+clutter_timeline_set_property (GObject      *object,
 			       guint         prop_id,
-			       const GValue *value, 
+			       const GValue *value,
 			       GParamSpec   *pspec)
 {
   ClutterTimeline        *timeline;
@@ -167,7 +178,7 @@ clutter_timeline_set_property (GObject      *object,
   timeline = CLUTTER_TIMELINE(object);
   priv = timeline->priv;
 
-  switch (prop_id) 
+  switch (prop_id)
     {
     case PROP_FPS:
       clutter_timeline_set_speed (timeline, g_value_get_uint (value));
@@ -181,16 +192,22 @@ clutter_timeline_set_property (GObject      *object,
     case PROP_DELAY:
       priv->delay = g_value_get_uint (value);
       break;
+    case PROP_DURATION:
+      clutter_timeline_set_duration (timeline, g_value_get_uint (value));
+      break;
+    case PROP_DIRECTION:
+      clutter_timeline_set_direction (timeline, g_value_get_enum (value));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
   }
 }
 
-static void 
-clutter_timeline_get_property (GObject    *object, 
+static void
+clutter_timeline_get_property (GObject    *object,
 			       guint       prop_id,
-			       GValue     *value, 
+			       GValue     *value,
 			       GParamSpec *pspec)
 {
   ClutterTimeline        *timeline;
@@ -199,7 +216,7 @@ clutter_timeline_get_property (GObject    *object,
   timeline = CLUTTER_TIMELINE(object);
   priv = timeline->priv;
 
-  switch (prop_id) 
+  switch (prop_id)
     {
     case PROP_FPS:
       g_value_set_uint (value, priv->fps);
@@ -213,19 +230,25 @@ clutter_timeline_get_property (GObject    *object,
     case PROP_DELAY:
       g_value_set_uint (value, priv->delay);
       break;
+    case PROP_DURATION:
+      g_value_set_uint (value, priv->duration);
+      break;
+    case PROP_DIRECTION:
+      g_value_set_enum (value, priv->direction);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
     }
 }
 
-static void 
+static void
 clutter_timeline_finalize (GObject *object)
 {
   G_OBJECT_CLASS (clutter_timeline_parent_class)->finalize (object);
 }
 
-static void 
+static void
 clutter_timeline_dispose (GObject *object)
 {
   ClutterTimeline *self = CLUTTER_TIMELINE(object);
@@ -274,8 +297,8 @@ clutter_timeline_class_init (ClutterTimelineClass *klass)
                                                       "Frames Per Second",
                                                       "Timeline frames per second",
                                                       1, 1000,
-                                                      50,
-                                                      G_PARAM_CONSTRUCT | CLUTTER_PARAM_READWRITE));
+                                                      60,
+                                                      CLUTTER_PARAM_READWRITE));
   /**
    * ClutterTimeline:num-frames:
    *
@@ -288,7 +311,7 @@ clutter_timeline_class_init (ClutterTimelineClass *klass)
                                                       "Timelines total number of frames",
                                                       1, G_MAXUINT,
                                                       1,
-                                                      G_PARAM_CONSTRUCT | CLUTTER_PARAM_READWRITE));
+                                                      CLUTTER_PARAM_READWRITE));
   /**
    * ClutterTimeline:loop:
    *
@@ -300,7 +323,7 @@ clutter_timeline_class_init (ClutterTimelineClass *klass)
                                                          "Loop",
                                                          "Should the timeline automatically restart",
                                                          FALSE,
-                                                         G_PARAM_CONSTRUCT | CLUTTER_PARAM_READWRITE));
+                                                         CLUTTER_PARAM_READWRITE));
   /**
    * ClutterTimeline:delay:
    *
@@ -316,12 +339,45 @@ clutter_timeline_class_init (ClutterTimelineClass *klass)
                                                       "Delay before start",
                                                       0, G_MAXUINT,
                                                       0,
-                                                      G_PARAM_CONSTRUCT | CLUTTER_PARAM_READWRITE));
+                                                      CLUTTER_PARAM_READWRITE));
+  /**
+   * ClutterTimeline:duration:
+   *
+   * Duration of the timeline in milliseconds, depending on the
+   * ClutterTimeline:fps value.
+   *
+   * Since: 0.6
+   */
+  g_object_class_install_property (object_class,
+                                   PROP_DURATION,
+                                   g_param_spec_uint ("duration",
+                                                      "Duration",
+                                                      "Duration of the timeline in milliseconds",
+                                                      0, G_MAXUINT,
+                                                      1000,
+                                                      CLUTTER_PARAM_READWRITE));
+  /**
+   * ClutterTimeline:direction:
+   * 
+   * The direction of the timeline, either %CLUTTER_TIMELINE_FORWARD or
+   * %CLUTTER_TIMELINE_BACKWARD.
+   *
+   * Since: 0.6
+   */
+  g_object_class_install_property (object_class,
+                                   PROP_DIRECTION,
+                                   g_param_spec_enum ("direction",
+                                                      "Direction",
+                                                      "Direction of the timeline",
+                                                      CLUTTER_TYPE_TIMELINE_DIRECTION,
+                                                      CLUTTER_TIMELINE_FORWARD,
+                                                      CLUTTER_PARAM_READWRITE));
 
   /**
    * ClutterTimeline::new-frame:
    * @timeline: the timeline which received the signal
-   * @frame_num: the number of the new frame
+   * @frame_num: the number of the new frame between 0 and 
+   * ClutterTimeline:num-frames
    *
    * The ::new-frame signal is emitted each time a new frame in the
    * timeline is reached.
@@ -333,7 +389,7 @@ clutter_timeline_class_init (ClutterTimelineClass *klass)
 		  G_STRUCT_OFFSET (ClutterTimelineClass, new_frame),
 		  NULL, NULL,
 		  clutter_marshal_VOID__INT,
-		  G_TYPE_NONE, 
+		  G_TYPE_NONE,
 		  1, G_TYPE_INT);
   /**
    * ClutterTimeline::completed:
@@ -389,6 +445,10 @@ clutter_timeline_init (ClutterTimeline *self)
   self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self,
 					    CLUTTER_TYPE_TIMELINE,
 					    ClutterTimelinePrivate);
+
+  self->priv->fps = clutter_get_default_frame_rate ();
+  self->priv->n_frames = 0;
+  self->priv->msecs_delta = 0;
 }
 
 static gboolean
@@ -410,13 +470,13 @@ timeline_timeout_func (gpointer data)
                     timeline);
       return FALSE;
     }
- 
+
   if (!priv->timeout_id)
     {
       CLUTTER_NOTE (SCHEDULER,
                     "The timeline [%p] has been removed",
                     timeline);
-  
+
       return FALSE;
     }
 
@@ -439,45 +499,68 @@ timeline_timeout_func (gpointer data)
       g_object_unref (timeline);
       return FALSE;
     }
-  
+
   if (priv->last_frame_msecs)
     {
-      /* Check time diff from out last call and adjust number 
-       * of frames to advance accordingly. 
+      /* Check time diff from out last call and adjust number
+       * of frames to advance accordingly.
       */
-      msecs = ((timeval.tv_sec - priv->start_frame_secs) * 1000) 
+      msecs = ((timeval.tv_sec - priv->start_frame_secs) * 1000)
               + (timeval.tv_usec / 1000);
-      n_frames =  (msecs - priv->last_frame_msecs )
-                  / (1000 / priv->fps);
+      n_frames = (msecs - priv->last_frame_msecs)
+                 / (1000 / priv->fps);
+      priv->msecs_delta = msecs - priv->last_frame_msecs;
 
       if (n_frames <= 0)
-        n_frames = 1;
+        {
+          n_frames = 1;
+          priv->skipped_frames = 0;
+        }
       else if (n_frames > 1)
 	{
-	  CLUTTER_TIMESTAMP (SCHEDULER, 
-			     "Timeline [%p], skipping %d frames\n", 
+	  CLUTTER_TIMESTAMP (SCHEDULER,
+			     "Timeline [%p], skipping %d frames\n",
 			     timeline,
                              n_frames);
+
+          priv->skipped_frames = n_frames - 1;
 	}
+      else
+        priv->skipped_frames = 0;
     }
-  else 
+  else
     {
       /* First frame, set up timings.*/
       priv->start_frame_secs = timeval.tv_sec;
-      
+      priv->skipped_frames   = 0;
+      priv->msecs_delta      = 0;
+
       msecs     = timeval.tv_usec / 1000;
       n_frames  = 1;
     }
 
-  priv->last_frame_msecs = msecs; 
+  priv->last_frame_msecs = msecs;
 
   /* Advance frames */
-  priv->current_frame_num += n_frames;;
+  if (priv->direction == CLUTTER_TIMELINE_FORWARD)
+    priv->current_frame_num += n_frames;
+  else
+    priv->current_frame_num -= n_frames;
 
   /* Handle loop or stop */
-  if (priv->current_frame_num > priv->n_frames)
+  if (((priv->direction == CLUTTER_TIMELINE_FORWARD) &&
+       (priv->current_frame_num > priv->n_frames)) ||
+      ((priv->direction == CLUTTER_TIMELINE_BACKWARD) &&
+       (priv->current_frame_num < 0)))
     {
-      guint frame_diff;
+      if (priv->direction == CLUTTER_TIMELINE_FORWARD)
+        {
+          priv->current_frame_num = priv->n_frames;
+        }
+      else if (priv->direction == CLUTTER_TIMELINE_BACKWARD)
+        {
+          priv->current_frame_num = 0;
+        }
 
       CLUTTER_NOTE (SCHEDULER,
                     "Timeline [%p] completed (cur: %d, tot: %d, drop: %d)",
@@ -486,21 +569,16 @@ timeline_timeout_func (gpointer data)
                     priv->n_frames,
                     n_frames - 1);
 
-      frame_diff = priv->current_frame_num - priv->n_frames;
-      priv->current_frame_num = priv->n_frames;
-
       /* if we skipped some frame to get here let's see whether we still need
        * to emit the last new-frame signal with the last frame
        */
-      if (frame_diff)
-  	{
-          g_signal_emit (timeline, timeline_signals[NEW_FRAME], 0,
-                         priv->current_frame_num);
-	}
+      if (n_frames > 1)
+	g_signal_emit (timeline, timeline_signals[NEW_FRAME], 0,
+                       priv->current_frame_num);
 
       if (priv->loop)
 	{
-	  g_signal_emit (timeline, timeline_signals[COMPLETED], 0); 
+	  g_signal_emit (timeline, timeline_signals[COMPLETED], 0);
 	  clutter_timeline_rewind (timeline);
 
           retval = TRUE;
@@ -515,8 +593,8 @@ timeline_timeout_func (gpointer data)
 
           priv->last_frame_msecs = 0;
 
-	  g_signal_emit (timeline, timeline_signals[COMPLETED], 0); 
-	  clutter_timeline_rewind (timeline);
+          g_signal_emit (timeline, timeline_signals[COMPLETED], 0);
+          clutter_timeline_rewind (timeline);
 
 	  retval = FALSE;
 	}
@@ -538,7 +616,7 @@ delay_timeout_func (gpointer data)
   priv->timeout_id = timeout_add (FPS_TO_INTERVAL (priv->fps),
                                   timeline_timeout_func,
                                   timeline, NULL);
-  
+
   g_signal_emit (timeline, timeline_signals[STARTED], 0);
 
   return FALSE;
@@ -556,12 +634,15 @@ clutter_timeline_start (ClutterTimeline *timeline)
   ClutterTimelinePrivate *priv;
 
   g_return_if_fail (CLUTTER_IS_TIMELINE (timeline));
-  
+
   priv = timeline->priv;
 
   if (priv->delay_id || priv->timeout_id)
     return;
 
+  if (priv->n_frames == 0)
+    return;
+
   if (priv->delay)
     {
       priv->delay_id = timeout_add (priv->delay,
@@ -635,7 +716,7 @@ clutter_timeline_set_loop (ClutterTimeline *timeline,
 			   gboolean         loop)
 {
   g_return_if_fail (CLUTTER_IS_TIMELINE (timeline));
-  
+
   if (timeline->priv->loop != loop)
     {
       timeline->priv->loop = loop;
@@ -664,14 +745,23 @@ clutter_timeline_get_loop (ClutterTimeline *timeline)
  * clutter_timeline_rewind:
  * @timeline: A #ClutterTimeline
  *
- * Rewinds #ClutterTimeline to frame 0.
+ * Rewinds #ClutterTimeline to the first frame if its direction is
+ * CLUTTER_TIMELINE_FORWARD and the last frame if it is
+ * CLUTTER_TIMELINE_BACKWARD.
  **/
 void
 clutter_timeline_rewind (ClutterTimeline *timeline)
 {
+  ClutterTimelinePrivate *priv;
+
   g_return_if_fail (CLUTTER_IS_TIMELINE (timeline));
-  
-  clutter_timeline_advance (timeline, 0);
+
+  priv = timeline->priv;
+
+  if (priv->direction == CLUTTER_TIMELINE_FORWARD)
+    clutter_timeline_advance (timeline, 0);
+  else if (priv->direction == CLUTTER_TIMELINE_BACKWARD)
+    clutter_timeline_advance (timeline, priv->n_frames);
 }
 
 /**
@@ -691,10 +781,20 @@ clutter_timeline_skip (ClutterTimeline *timeline,
 
   priv = timeline->priv;
 
-  priv->current_frame_num += n_frames;
+  if (priv->direction == CLUTTER_TIMELINE_FORWARD)
+    {
+      priv->current_frame_num += n_frames;
 
-  if (priv->current_frame_num > priv->n_frames)
-    priv->current_frame_num = 1;
+      if (priv->current_frame_num > priv->n_frames)
+        priv->current_frame_num = 1;
+    }
+  else if (priv->direction == CLUTTER_TIMELINE_BACKWARD)
+    {
+      priv->current_frame_num -= n_frames;
+
+      if (priv->current_frame_num < 1)
+        priv->current_frame_num = priv->n_frames - 1;
+    }
 }
 
 /**
@@ -714,8 +814,7 @@ clutter_timeline_advance (ClutterTimeline *timeline,
 
   priv = timeline->priv;
 
-  if (frame_num < priv->n_frames)
-    priv->current_frame_num = frame_num;
+  priv->current_frame_num = CLAMP (frame_num, 0, priv->n_frames);
 }
 
 /**
@@ -770,12 +869,9 @@ clutter_timeline_set_n_frames (ClutterTimeline *timeline,
 
   if (priv->n_frames != n_frames)
     {
-      g_object_ref (timeline);
-
       priv->n_frames = n_frames;
 
       g_object_notify (G_OBJECT (timeline), "num-frames");
-      g_object_unref (timeline);
     }
 }
 
@@ -807,7 +903,7 @@ clutter_timeline_set_speed (ClutterTimeline *timeline,
       if (priv->timeout_id)
         {
           timeout_remove (priv->timeout_id);
-     
+
           priv->timeout_id = timeout_add (FPS_TO_INTERVAL (priv->fps),
                                           timeline_timeout_func,
                                           timeline, NULL);
@@ -846,7 +942,7 @@ gboolean
 clutter_timeline_is_playing (ClutterTimeline *timeline)
 {
   g_return_val_if_fail (CLUTTER_IS_TIMELINE (timeline), FALSE);
-  
+
   return (timeline->priv->timeout_id != 0);
 }
 
@@ -875,11 +971,32 @@ clutter_timeline_clone (ClutterTimeline *timeline)
                        "num-frames", clutter_timeline_get_n_frames (timeline),
                        "loop", clutter_timeline_get_loop (timeline),
                        "delay", clutter_timeline_get_delay (timeline),
+                       "direction", clutter_timeline_get_direction (timeline),
                        NULL);
 
   return copy;
 }
 
+/**
+ * clutter_timeline_new_for_duration:
+ * @msecs: Duration of the timeline in milliseconds
+ *
+ * Creates a new #ClutterTimeline with a duration of @msecs using
+ * the value of the ClutterTimeline:fps property to compute the
+ * equivalent number of frames. 
+ *
+ * Return value: the newly created #ClutterTimeline
+ *
+ * Since: 0.6
+ */
+ClutterTimeline *
+clutter_timeline_new_for_duration (guint msecs)
+{
+  return g_object_new (CLUTTER_TYPE_TIMELINE,
+                       "duration", msecs,
+                       NULL);
+}
+
 /**
  * clutter_timeline_new:
  * @n_frames: the number of frames
@@ -896,9 +1013,9 @@ clutter_timeline_new (guint n_frames,
   g_return_val_if_fail (n_frames > 0, NULL);
   g_return_val_if_fail (fps > 0, NULL);
 
-  return g_object_new (CLUTTER_TYPE_TIMELINE, 
-		       "fps", fps, 
-		       "num-frames", n_frames, 
+  return g_object_new (CLUTTER_TYPE_TIMELINE,
+		       "fps", fps,
+		       "num-frames", n_frames,
 		       NULL);
 }
 
@@ -946,3 +1063,196 @@ clutter_timeline_set_delay (ClutterTimeline *timeline,
       g_object_notify (G_OBJECT (timeline), "delay");
     }
 }
+
+/**
+ * clutter_timeline_get_duration:
+ * @timeline: a #ClutterTimeline
+ *
+ * Retrieves the duration of a #ClutterTimeline in milliseconds.
+ * See clutter_timeline_set_duration().
+ *
+ * Return value: the duration of the timeline, in milliseconds.
+ *
+ * Since: 0.6
+ */
+guint
+clutter_timeline_get_duration (ClutterTimeline *timeline)
+{
+  g_return_val_if_fail (CLUTTER_IS_TIMELINE (timeline), 0);
+
+  return timeline->priv->duration;
+}
+
+/**
+ * clutter_timeline_set_duration:
+ * @timeline: a #ClutterTimeline
+ * @msecs: duration of the timeline in milliseconds
+ *
+ * Sets the duration of the timeline, in milliseconds. The speed
+ * of the timeline depends on the ClutterTimeline:fps setting.
+ *
+ * Since: 0.6
+ */
+void
+clutter_timeline_set_duration (ClutterTimeline *timeline,
+                               guint            msecs)
+{
+  ClutterTimelinePrivate *priv;
+
+  g_return_if_fail (CLUTTER_IS_TIMELINE (timeline));
+
+  priv = timeline->priv;
+
+  if (priv->duration != msecs)
+    {
+      g_object_ref (timeline);
+
+      g_object_freeze_notify (G_OBJECT (timeline));
+
+      priv->duration = msecs;
+
+      priv->n_frames = priv->duration * priv->fps / 1000;
+
+      g_object_notify (G_OBJECT (timeline), "num-frames");
+      g_object_notify (G_OBJECT (timeline), "duration");
+
+      g_object_thaw_notify (G_OBJECT (timeline));
+      g_object_unref (timeline);
+    }
+}
+
+/**
+ * clutter_timeline_get_progress:
+ * @timeline: a #ClutterTimeline
+ *
+ * The position of the timeline in a [0, 1] interval.
+ *
+ * Return value: the position of the timeline.
+ *
+ * Since: 0.6
+ */
+gdouble
+clutter_timeline_get_progress (ClutterTimeline *timeline)
+{
+  g_return_val_if_fail (CLUTTER_IS_TIMELINE (timeline), 0.);
+
+  return CLUTTER_FIXED_TO_DOUBLE (clutter_timeline_get_progressx (timeline));
+}
+
+/**
+ * clutter_timeline_get_progressx:
+ * @timeline: a #ClutterTimeline
+ *
+ * Fixed point version of clutter_timeline_get_progress().
+ *
+ * Return value: the position of the timeline as a fixed point value
+ *
+ * Since: 0.6
+ */
+ClutterFixed
+clutter_timeline_get_progressx (ClutterTimeline *timeline)
+{
+  ClutterTimelinePrivate *priv;
+
+  g_return_val_if_fail (CLUTTER_IS_TIMELINE (timeline), 0);
+
+  priv = timeline->priv;
+
+  if (priv->direction == CLUTTER_TIMELINE_FORWARD)
+    return CLUTTER_FIXED_DIV (CLUTTER_INT_TO_FIXED (priv->current_frame_num),
+                              CLUTTER_INT_TO_FIXED (priv->n_frames));
+  else
+    return CLUTTER_FIXED_DIV (CLUTTER_INT_TO_FIXED (priv->n_frames),
+                              CLUTTER_INT_TO_FIXED (priv->current_frame_num));
+}
+
+/**
+ * clutter_timeline_get_direction:
+ * @timeline: a #ClutterTimeline
+ *
+ * Retrieves the direction of the timeline set with
+ * clutter_timeline_set_direction().
+ *
+ * Return value: the direction of the timeline
+ *
+ * Since: 0.6
+ */
+ClutterTimelineDirection
+clutter_timeline_get_direction (ClutterTimeline *timeline)
+{
+  g_return_val_if_fail (CLUTTER_IS_TIMELINE (timeline), CLUTTER_TIMELINE_FORWARD);
+
+  return timeline->priv->direction;
+}
+
+/**
+ * clutter_timeline_set_direction:
+ * @timeline: a #ClutterTimeline
+ * @direction: the direction of the timeline
+ *
+ * Sets the direction of @timeline, either %CLUTTER_TIMELINE_FORWARD or
+ * %CLUTTER_TIMELINE_BACKWARD.
+ *
+ * Since: 0.6
+ */
+void
+clutter_timeline_set_direction (ClutterTimeline          *timeline,
+                                ClutterTimelineDirection  direction)
+{
+  ClutterTimelinePrivate *priv;
+
+  g_return_if_fail (CLUTTER_IS_TIMELINE (timeline));
+
+  priv = timeline->priv;
+
+  if (priv->direction != direction)
+    {
+      priv->direction = direction;
+
+      if (priv->current_frame_num == 0)
+        priv->current_frame_num = priv->n_frames;
+
+      g_object_notify (G_OBJECT (timeline), "direction");
+    }
+}
+
+/**
+ * clutter_timeline_get_delta:
+ * @timeline: a #ClutterTimeline
+ * @msecs: return location for the milliseconds elapsed since the last
+ *   frame, or %NULL
+ *
+ * Retrieves the number of frames and the amount of time elapsed since
+ * the last ClutterTimeline::new-frame signal.
+ *
+ * This function is only useful inside handlers for the ::new-frame
+ * signal, and its behaviour is undefined if the timeline is not
+ * playing.
+ *
+ * Return value: the amount of frames elapsed since the last one
+ *
+ * Since: 0.6
+ */
+guint
+clutter_timeline_get_delta (ClutterTimeline *timeline,
+                            guint           *msecs)
+{
+  ClutterTimelinePrivate *priv;
+
+  g_return_val_if_fail (CLUTTER_IS_TIMELINE (timeline), 0);
+
+  if (!clutter_timeline_is_playing (timeline))
+    {
+      if (msecs)
+        *msecs = 0;
+
+      return 0;
+    }
+
+  priv = timeline->priv;
+
+  if (msecs)
+    *msecs = timeline->priv->msecs_delta;
+
+  return priv->skipped_frames + 1;
+}
