@@ -30,12 +30,6 @@
 
 G_BEGIN_DECLS
 
-void
-clutter_util_trap_x_errors(void);
-
-int
-clutter_util_untrap_x_errors(void);
-
 int 
 clutter_util_next_p2 (int a);
 
