@@ -46,21 +46,21 @@
  *
  * The minor version of the Clutter library (2, if %CLUTTER_VERSION is 1.2.3)
  */
-#define CLUTTER_MINOR_VERSION   (4)
+#define CLUTTER_MINOR_VERSION   (6)
 
 /**
  * CLUTTER_MICRO_VERSION:
  *
  * The micro version of the Clutter library (3, if %CLUTTER_VERSION is 1.2.3)
  */
-#define CLUTTER_MICRO_VERSION   (2)
+#define CLUTTER_MICRO_VERSION   (0)
 
 /**
  * CLUTTER_VERSION:
  *
  * The full version of the Clutter library, like 1.2.3
  */
-#define CLUTTER_VERSION         0.4.2
+#define CLUTTER_VERSION         0.6.0
 
 /**
  * CLUTTER_VERSION_S:
@@ -68,7 +68,7 @@
  * The full version of the Clutter library, in string form (suited for
  * string concatenation)
  */
-#define CLUTTER_VERSION_S       "0.4.2"
+#define CLUTTER_VERSION_S       "0.6.0"
 
 /**
  * CLUTTER_VERSION_HEX:
@@ -91,7 +91,7 @@
 #define CLUTTER_CHECK_VERSION(major,minor,micro) \
         (CLUTTER_MAJOR_VERSION > (major) || \
          (CLUTTER_MAJOR_VERSION == (major) && CLUTTER_MINOR_VERSION > (minor)) || \
-         (CLUTTER_MAJOR_VERSION == (major) && CLUTTER_MINOR_VERSION == (minor) && CLUTTER_MICRO_VERSION > (micro)))
+         (CLUTTER_MAJOR_VERSION == (major) && CLUTTER_MINOR_VERSION == (minor) && CLUTTER_MICRO_VERSION >= (micro)))
 
 /**
  * CLUTTER_FLAVOUR:
@@ -117,8 +117,9 @@
  *
  * Set to 1 if Clutter was built without FPU (i.e fixed math), 0 otherwise
  *
- * Since: 0.4
+ * @Deprecated: 0.6: This macro is no longer defined (identical code is used
+ *  regardless the presence of FPU).
  */
-#define CLUTTER_NO_FPU          (0)
+#define CLUTTER_NO_FPU          CLUTTER_NO_FPU_MACRO_WAS_REMOVED
 
 #endif /* __CLUTTER_VERSION_H__ */
