@@ -39,28 +39,30 @@
 #include "clutter-behaviour-path.h"
 #include "clutter-behaviour-rotate.h"
 #include "clutter-behaviour-scale.h"
-#include "clutter-box.h"
 #include "clutter-clone-texture.h"
+#include "clutter-deprecated.h"
 #include "clutter-effect.h"
 #include "clutter-entry.h"
 #include "clutter-event.h"
 #include "clutter-feature.h"
 #include "clutter-rectangle.h"
 #include "clutter-group.h"
-#include "clutter-hbox.h"
 #include "clutter-keysyms.h" 
 #include "clutter-label.h"
-#include "clutter-layout.h"
+#include "clutter-list-model.h"
 #include "clutter-main.h"
 #include "clutter-media.h"
+#include "clutter-model.h"
 #include "clutter-stage.h"
 #include "clutter-texture.h"
 #include "clutter-timeout-pool.h"
 #include "clutter-timeline.h"
+#include "clutter-score.h"
+#include "clutter-script.h"
+#include "clutter-scriptable.h"
 #include "clutter-types.h"
 #include "clutter-units.h"
 #include "clutter-util.h"
-#include "clutter-vbox.h"
 #include "clutter-version.h"
 
 #include "clutter-enum-types.h"
