@@ -26,12 +26,23 @@
 #ifndef __COGL_DEFINES_H__
 #define __COGL_DEFINES_H__
 
+#ifdef WIN32
+
+#include <windows.h>
+#include <GL/GLee.h>
+
+#else
+
 #include <GL/gl.h>
 
+#endif /* WIN32 */
+
 G_BEGIN_DECLS
 
 typedef GLenum COGLenum;
 typedef GLint COGLint;
+typedef GLuint COGLuint;
+typedef GLhandleARB COGLhandle;
 
 /* FIXME + DOCUMENT */
 
@@ -678,8 +689,8 @@ typedef GLint COGLint;
 #define CGL_TEXTURE_RECTANGLE_ARB 0
 #endif
 
-#ifdef GL_YCBCR_MESA 
-#define CGL_YCBCR_MESA GL_YCBCR_MESA 
+#ifdef GL_YCBCR_MESA
+#define CGL_YCBCR_MESA GL_YCBCR_MESA
 #define CGL_UNSIGNED_SHORT_8_8_REV_MESA GL_UNSIGNED_SHORT_8_8_REV_MESA
 #define CGL_UNSIGNED_SHORT_8_8_MESA GL_UNSIGNED_SHORT_8_8_MESA
 #else
@@ -688,6 +699,13 @@ typedef GLint COGLint;
 #define CGL_UNSIGNED_SHORT_8_8_MESA 0
 #endif
 
+#define CGL_FRAGMENT_SHADER GL_FRAGMENT_SHADER_ARB
+#define CGL_VERTEX_SHADER   GL_VERTEX_SHADER_ARB
+
+#define CGL_OBJECT_COMPILE_STATUS  GL_OBJECT_COMPILE_STATUS_ARB
+
+#define CLUTTER_COGL_HAS_GL 1
+
 G_END_DECLS
 
 #endif
