@@ -439,6 +439,8 @@ G_BEGIN_DECLS
 
 typedef GLenum COGLenum;
 typedef GLint  COGLint;
+typedef GLuint COGLuint;
+typedef GLuint COGLhandle;
 
 /* extras */
 
@@ -459,6 +461,26 @@ typedef GLint  COGLint;
 #define CGL_UNSIGNED_SHORT_8_8_REV_MESA 0
 #define CGL_UNSIGNED_SHORT_8_8_MESA 0
 
+#ifdef GL_FRAGMENT_SHADER
+#define CGL_FRAGMENT_SHADER GL_FRAGMENT_SHADER
+#else
+#define CGL_FRAGMENT_SHADER 0
+#endif
+
+#ifdef GL_VERTEX_SHADER
+#define CGL_VERTEX_SHADER GL_VERTEX_SHADER
+#else
+#define CGL_VERTEX_SHADER 0
+#endif
+
+#ifdef GL_OBJECT_COMPILE_STATUS
+#define CGL_OBJECT_COMPILE_STATUS  GL_OBJECT_COMPILE_STATUS
+#else
+#define CGL_OBJECT_COMPILE_STATUS 0
+#endif
+
+#define CLUTTER_COGL_HAS_GLES 1
+
 G_END_DECLS
 
 #endif
