@@ -39,19 +39,6 @@
 
 G_BEGIN_DECLS
 
-void     clutter_eglx_trap_x_errors       (void);
-gint     clutter_eglx_untrap_x_errors     (void);
-
-Display *clutter_eglx_get_default_xdisplay (void);
-gint     clutter_eglx_get_default_screen  (void);
-Window   clutter_eglx_get_default_root_window (void);
-
-Window       clutter_eglx_get_stage_window (ClutterStage *stage);
-XVisualInfo *clutter_eglx_get_stage_visual (ClutterStage *stage);
-
-void         clutter_eglx_set_stage_foreign (ClutterStage *stage,
-					     Window        window);
-
 EGLDisplay
 clutter_eglx_display (void);
 
