@@ -34,7 +34,7 @@
 
 #include "cogl.h"
 
-/* 
+/*
  * Texture cache support code
  */
 
@@ -49,7 +49,7 @@ typedef struct {
 
 typedef struct tc_texture {
   struct tc_texture *next;
-  guint name;
+  COGLuint name;
   int avail;
 } tc_texture;
 
@@ -58,7 +58,7 @@ typedef struct tc_slice {
   int avail, y;
 } tc_slice;
 
-static int tc_generation;
+static int tc_generation = 0;
 static tc_slice slices[TC_HEIGHT / TC_ROUND];
 static tc_texture *first_texture;
 
@@ -78,20 +78,27 @@ tc_clear ()
       first_texture = next;
     }
 
-  printf("freeing textures\n");
-
   ++tc_generation;
 }
 
 static void
 tc_get (tc_area *area, int width, int height)
 {
-  int slice_height = MIN (height + TC_ROUND - 1, TC_HEIGHT) & ~(TC_ROUND - 1);
-  tc_slice *slice = slices + slice_height / TC_ROUND;
+  int       slice_height;
+  tc_slice *slice;
 
   area->w = width;
   area->h = height;
 
+  /* Provide for blank rows/columns of pixels between adjecant glyphs in the
+   * texture cache to avoid bilinear interpolation spillage at edges of glyphs.
+   */
+  width += 1;
+  height += 1;
+
+  slice_height = MIN (height + TC_ROUND - 1, TC_HEIGHT) & ~(TC_ROUND - 1);
+  slice = slices + slice_height / TC_ROUND;
+
   width = MIN (width, TC_WIDTH);
 
   if (!slice->name || slice->avail < width)
@@ -106,8 +113,16 @@ tc_get (tc_area *area, int width, int height)
       /* create a new texture if necessary */
       if (!match)
         {
-	  CLUTTER_NOTE (PANGO, g_message ("creating new texture %i x %i\n",
-                                          TC_WIDTH, TC_HEIGHT));
+	  COGLenum texture_target_type = CGL_TEXTURE_2D;
+
+	  CLUTTER_NOTE (PANGO, "creating new texture %i x %i",
+			TC_WIDTH, TC_HEIGHT);
+
+	  /* Use NPOTS if available as it simply makes shaders easier to
+	   * work with on text.
+	  */
+	  if (clutter_feature_available (CLUTTER_FEATURE_TEXTURE_RECTANGLE))
+	    texture_target_type = CGL_TEXTURE_RECTANGLE_ARB;
 
           match = g_slice_new (tc_texture);
           match->next = first_texture;
@@ -116,16 +131,21 @@ tc_get (tc_area *area, int width, int height)
 
 	  cogl_textures_create (1, &match->name);
 
-	  cogl_texture_bind (CGL_TEXTURE_2D, match->name);
+	  cogl_texture_bind (texture_target_type, match->name);
 
-	  cogl_texture_set_filters (CGL_TEXTURE_2D, CGL_NEAREST, CGL_NEAREST);
+        /* We might even want to use mipmapping instead of CGL_LINEAR here
+         * that should allow rerendering of glyphs to look nice even at scales
+         * far below 50%.
+         */
+	  cogl_texture_set_filters (texture_target_type,
+				    CGL_LINEAR, CGL_NEAREST);
 
-	  cogl_texture_image_2d (CGL_TEXTURE_2D, 
+	  cogl_texture_image_2d (texture_target_type,
 				 CGL_ALPHA,
 				 TC_WIDTH,
 				 TC_HEIGHT,
-				 CGL_ALPHA, 
-				 CGL_UNSIGNED_BYTE, 
+				 CGL_ALPHA,
+				 CGL_UNSIGNED_BYTE,
 				 NULL);
         }
 
@@ -175,6 +195,7 @@ struct _PangoClutterRenderer
   ClutterColor  color;
   int           flags;
   guint         curtex; /* current texture */
+  COGLenum      texture_target_type; /* 2D or Rect */
 };
 
 G_DEFINE_TYPE (PangoClutterRenderer,   \
@@ -196,9 +217,9 @@ temp_buffer (size_t size)
   if (size > alloc)
     {
       size = (size + 4095) & ~4095;
-      free (buffer);
+      g_free (buffer);
       alloc = size;
-      buffer = malloc (size);
+      buffer = g_malloc (size);
     }
 
   return buffer;
@@ -232,11 +253,11 @@ render_box (Glyph *glyph, int width, int height, int top)
   memset (glyph->bitmap, 0, glyph->stride * height);
 
   for (i = width; i--; )
-    glyph->bitmap [i] 
+    glyph->bitmap [i]
       = glyph->bitmap [i + (height - 1) * glyph->stride] = 0xff;
 
   for (i = height; i--; )
-    glyph->bitmap [i * glyph->stride] 
+    glyph->bitmap [i * glyph->stride]
       = glyph->bitmap [i * glyph->stride + (width - 1)] = 0xff;
 }
 
@@ -266,7 +287,7 @@ font_render_glyph (Glyph *glyph, PangoFont *font, int glyph_index)
     }
 
   face = pango_clutter_font_get_face (font);
-  
+
   if (face)
     {
       PangoClutterFont *glfont = (PangoClutterFont *)font;
@@ -283,16 +304,16 @@ font_render_glyph (Glyph *glyph, PangoFont *font, int glyph_index)
     }
   else
     generic_box:
-      render_box (glyph, PANGO_UNKNOWN_GLYPH_WIDTH, 
+      render_box (glyph, PANGO_UNKNOWN_GLYPH_WIDTH,
 		  PANGO_UNKNOWN_GLYPH_HEIGHT, PANGO_UNKNOWN_GLYPH_HEIGHT);
 }
 
-typedef struct glyph_info 
+typedef struct glyph_info
 {
   tc_area tex;
   int     left, top;
   int     generation;
-} 
+}
 glyph_info;
 
 static void
@@ -303,20 +324,20 @@ free_glyph_info (glyph_info *g)
 }
 
 static void
-draw_glyph (PangoRenderer *renderer_, 
-	    PangoFont     *font, 
-	    PangoGlyph     glyph, 
-	    double         x, 
-	    double         y)
+draw_glyph (PangoRenderer *renderer_,
+	    PangoFont     *font,
+	    PangoGlyph     glyph,
+	    double         xd,
+	    double         yd)
 {
   PangoClutterRenderer *renderer = PANGO_CLUTTER_RENDERER (renderer_);
   glyph_info           *g;
-  struct { float x1, y1, x2, y2; } box;
+  gint                  x = (gint)xd, y = (gint)yd;
 
   if (glyph & PANGO_GLYPH_UNKNOWN_FLAG)
     {
       glyph = pango_clutter_get_unknown_glyph (font);
-      
+
       if (glyph == PANGO_GLYPH_EMPTY)
 	glyph = PANGO_GLYPH_UNKNOWN_FLAG;
     }
@@ -328,77 +349,98 @@ draw_glyph (PangoRenderer *renderer_,
       Glyph bm;
       font_render_glyph (&bm, font, glyph);
 
+      if (bm.width < 1 || bm.height < 1 || bm.bitmap == NULL)
+        {
+          /* Safety on */
+          if (g)
+            {
+              x += g->left;
+              y -= g->top;
+            }
+          return;
+        }
+
+      if (bm.height > TC_HEIGHT)
+        {
+          g_warning ("%s: Glyph too large for cache, increase TC_HEIGHT",
+                     G_STRLOC);
+        }
+
       if (g)
         g->generation = tc_generation;
       else
         {
           g = g_slice_new (glyph_info);
 
-          _pango_clutter_font_set_glyph_cache_destroy 
+          _pango_clutter_font_set_glyph_cache_destroy
 	                        (font, (GDestroyNotify)free_glyph_info);
           _pango_clutter_font_set_cache_glyph_data (font, glyph, g);
         }
 
-      /*
-      if (renderer->curtex)
-        glEnd ();
-      */
-
       tc_get (&g->tex, bm.width, bm.height);
 
       g->left = bm.left;
       g->top  = bm.top;
 
-      CLUTTER_NOTE (PANGO, g_message ("cache fail; subimage2d %i\n", glyph));
+      CLUTTER_NOTE (PANGO, "cache fail; subimage2d %i", glyph);
 
+      cogl_texture_bind (renderer->texture_target_type, g->tex.name);
 
-      cogl_texture_bind (CGL_TEXTURE_2D, g->tex.name);
+      cogl_texture_set_alignment (renderer->texture_target_type,
+				  1, bm.stride);
 
-      cogl_texture_set_alignment (CGL_TEXTURE_2D, 1, bm.stride); 
-
-      cogl_texture_sub_image_2d (CGL_TEXTURE_2D,
-				 g->tex.x, 
-				 g->tex.y, 
-				 bm.width, 
-				 bm.height, 
-				 CGL_ALPHA, 
-				 CGL_UNSIGNED_BYTE, 
+      cogl_texture_sub_image_2d (renderer->texture_target_type,
+				 g->tex.x,
+				 g->tex.y,
+				 bm.width,
+				 bm.height,
+				 CGL_ALPHA,
+				 CGL_UNSIGNED_BYTE,
 				 bm.bitmap);
 
+      glTexParameteri (renderer->texture_target_type,
+		       GL_GENERATE_MIPMAP, FALSE);
+
       renderer->curtex = g->tex.name;
-      /* glBegin (GL_QUADS); */
     }
-  else CLUTTER_NOTE (PANGO, g_message ("cache succsess %i\n", glyph));
+  else CLUTTER_NOTE (PANGO, "cache succsess %i\n", glyph);
 
   x += g->left;
   y -= g->top;
 
-  box.x1 = g->tex.x * (1. / TC_WIDTH );
-  box.y1 = g->tex.y * (1. / TC_HEIGHT);
-  box.x2 = g->tex.w * (1. / TC_WIDTH ) + box.x1;
-  box.y2 = g->tex.h * (1. / TC_HEIGHT) + box.y1;
-
   if (g->tex.name != renderer->curtex)
     {
-      /*
-	if (renderer->curtex)
-          glEnd ();
-      */
-
-      cogl_texture_bind (CGL_TEXTURE_2D, g->tex.name);
+      cogl_texture_bind (renderer->texture_target_type, g->tex.name);
       renderer->curtex = g->tex.name;
-
-      /* glBegin (GL_QUADS); */
     }
 
-  cogl_texture_quad (x, 
-		     x + g->tex.w, 
-		     y,
-		     y + g->tex.h,
-		     CLUTTER_FLOAT_TO_FIXED (box.x1), 
-		     CLUTTER_FLOAT_TO_FIXED (box.y1), 
-		     CLUTTER_FLOAT_TO_FIXED (box.x2), 
-		     CLUTTER_FLOAT_TO_FIXED (box.y2));
+  if (clutter_feature_available (CLUTTER_FEATURE_TEXTURE_RECTANGLE))
+    {
+      cogl_texture_quad (x,
+			 x + g->tex.w,
+			 y,
+			 y + g->tex.h,
+			 CLUTTER_INT_TO_FIXED (g->tex.x),
+			 CLUTTER_INT_TO_FIXED (g->tex.y),
+			 CLUTTER_INT_TO_FIXED (g->tex.w + g->tex.x),
+			 CLUTTER_INT_TO_FIXED (g->tex.h + g->tex.y));
+    }
+  else
+    {
+      ClutterFixed fx, fy;
+
+      fx = CLUTTER_INT_TO_FIXED (g->tex.x) / TC_WIDTH;
+      fy = CLUTTER_INT_TO_FIXED (g->tex.y) / TC_HEIGHT;
+
+      cogl_texture_quad (x,
+			 x + g->tex.w,
+			 y,
+			 y + g->tex.h,
+			 fx,
+			 fy,
+			 CLUTTER_INT_TO_FIXED (g->tex.w) / TC_WIDTH + fx,
+			 CLUTTER_INT_TO_FIXED (g->tex.h) / TC_HEIGHT + fy);
+    }
 }
 
 static void
@@ -430,12 +472,16 @@ draw_trapezoid (PangoRenderer   *renderer_,
 		  (gint) x22);
 
   /* Turn it back on again */
-  cogl_enable (CGL_ENABLE_TEXTURE_2D|CGL_ENABLE_BLEND);
+  if (clutter_feature_available (CLUTTER_FEATURE_TEXTURE_RECTANGLE))
+    cogl_enable (CGL_ENABLE_TEXTURE_RECT|CGL_ENABLE_BLEND);
+  else
+    cogl_enable (CGL_ENABLE_TEXTURE_2D|CGL_ENABLE_BLEND);
 }
 
-void 
+
+void
 pango_clutter_render_layout_subpixel (PangoLayout *layout,
-				      int           x, 
+				      int           x,
 				      int           y,
 				      ClutterColor *color,
 				      int           flags)
@@ -446,30 +492,30 @@ pango_clutter_render_layout_subpixel (PangoLayout *layout,
 
   context = pango_layout_get_context (layout);
   fontmap = pango_context_get_font_map (context);
-  renderer = _pango_clutter_font_map_get_renderer 
+  renderer = _pango_clutter_font_map_get_renderer
                      (PANGO_CLUTTER_FONT_MAP (fontmap));
 
-  memcpy (&(PANGO_CLUTTER_RENDERER (renderer)->color), 
+  memcpy (&(PANGO_CLUTTER_RENDERER (renderer)->color),
 	  color, sizeof(ClutterColor));
-  
+
   pango_renderer_draw_layout (renderer, layout, x, y);
 }
 
-void 
+void
 pango_clutter_render_layout (PangoLayout  *layout,
-			     int           x, 
+			     int           x,
 			     int           y,
 			     ClutterColor *color,
 			     int           flags)
 {
-  pango_clutter_render_layout_subpixel (layout, 
-					x * PANGO_SCALE, 
-					y * PANGO_SCALE, 
+  pango_clutter_render_layout_subpixel (layout,
+					x * PANGO_SCALE,
+					y * PANGO_SCALE,
 					color,
 					flags);
 }
 
-void 
+void
 pango_clutter_render_layout_line (PangoLayoutLine *line,
 				  int              x,
 				  int              y,
@@ -481,16 +527,16 @@ pango_clutter_render_layout_line (PangoLayoutLine *line,
 
   context = pango_layout_get_context (line->layout);
   fontmap = pango_context_get_font_map (context);
-  renderer = _pango_clutter_font_map_get_renderer 
+  renderer = _pango_clutter_font_map_get_renderer
                      (PANGO_CLUTTER_FONT_MAP (fontmap));
 
-  memcpy (&(PANGO_CLUTTER_RENDERER (renderer)->color), 
+  memcpy (&(PANGO_CLUTTER_RENDERER (renderer)->color),
 	  color, sizeof(ClutterColor));
-  
+
   pango_renderer_draw_layout_line (renderer, line, x, y);
 }
 
-void 
+void
 pango_clutter_render_clear_caches (void)
 {
   tc_clear();
@@ -499,6 +545,11 @@ pango_clutter_render_clear_caches (void)
 static void
 pango_clutter_renderer_init (PangoClutterRenderer *renderer)
 {
+  renderer->texture_target_type = CGL_TEXTURE_2D;
+
+  if (clutter_feature_available (CLUTTER_FEATURE_TEXTURE_RECTANGLE))
+    renderer->texture_target_type = CGL_TEXTURE_RECTANGLE_ARB;
+
   memset (&renderer->color, 0xff, sizeof(ClutterColor));
 }
 
@@ -516,17 +567,17 @@ prepare_run (PangoRenderer *renderer, PangoLayoutRun *run)
   for (l = run->item->analysis.extra_attrs; l; l = l->next)
     {
       PangoAttribute *attr = l->data;
-      
+
       switch (attr->klass->type)
 	{
 	case PANGO_ATTR_UNDERLINE:
 	  renderer->underline = ((PangoAttrInt *)attr)->value;
 	  break;
-	  
+
 	case PANGO_ATTR_STRIKETHROUGH:
 	  renderer->strikethrough = ((PangoAttrInt *)attr)->value;
 	  break;
-	  
+
 	case PANGO_ATTR_FOREGROUND:
           fg = &((PangoAttrColor *)attr)->color;
 	  break;
@@ -541,7 +592,7 @@ prepare_run (PangoRenderer *renderer, PangoLayoutRun *run)
       col.green = (fg->green * 255) / 65535;
       col.blue  = (fg->blue  * 255) / 65535;
     }
-  else 
+  else
     {
       col.red   = glrenderer->color.red;
       col.green = glrenderer->color.green;
@@ -555,8 +606,8 @@ prepare_run (PangoRenderer *renderer, PangoLayoutRun *run)
       col.red   ^= 0xffU;
       col.green ^= 0xffU;
       col.blue   ^= 0xffU;
-    } 
-  
+    }
+
   cogl_color(&col);
 }
 
@@ -567,17 +618,10 @@ draw_begin (PangoRenderer *renderer_)
 
   renderer->curtex = 0;
 
-  cogl_enable (CGL_ENABLE_TEXTURE_2D
-	       |CGL_ENABLE_BLEND);
-
-#if 0
-  gl_BlendFuncSeparate (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA,
-                        GL_ONE      , GL_ONE_MINUS_SRC_ALPHA);
-
-  glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
-  glEnable (GL_ALPHA_TEST);
-  glAlphaFunc (GL_GREATER, 0.01f);
-#endif
+  if (clutter_feature_available (CLUTTER_FEATURE_TEXTURE_RECTANGLE))
+    cogl_enable (CGL_ENABLE_TEXTURE_RECT|CGL_ENABLE_BLEND);
+  else
+    cogl_enable (CGL_ENABLE_TEXTURE_2D|CGL_ENABLE_BLEND);
 }
 
 static void
